@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,6 +36,8 @@
 package net.sourceforge.plantuml;
 
 import java.awt.Font;
+import java.io.IOException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -43,64 +45,180 @@ import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Objects;
 import java.util.Set;
 import java.util.TreeSet;
 
+import net.sourceforge.plantuml.command.BlocLines;
 import net.sourceforge.plantuml.command.regex.Matcher2;
 import net.sourceforge.plantuml.command.regex.MyPattern;
 import net.sourceforge.plantuml.command.regex.Pattern2;
-import net.sourceforge.plantuml.creole.CommandCreoleMonospaced;
+import net.sourceforge.plantuml.creole.Parser;
 import net.sourceforge.plantuml.cucadiagram.LinkStyle;
 import net.sourceforge.plantuml.cucadiagram.Rankdir;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.cucadiagram.dot.DotSplines;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
-import net.sourceforge.plantuml.graphic.HtmlColorSetSimple;
-import net.sourceforge.plantuml.graphic.HtmlColorUtils;
-import net.sourceforge.plantuml.graphic.IHtmlColorSet;
 import net.sourceforge.plantuml.graphic.SkinParameter;
 import net.sourceforge.plantuml.graphic.color.Colors;
+import net.sourceforge.plantuml.skin.ActorStyle;
 import net.sourceforge.plantuml.skin.ArrowDirection;
+import net.sourceforge.plantuml.skin.Padder;
+import net.sourceforge.plantuml.sprite.Sprite;
+import net.sourceforge.plantuml.sprite.SpriteImage;
+import net.sourceforge.plantuml.style.FromSkinparamToStyle;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleBuilder;
+import net.sourceforge.plantuml.style.StyleLoader;
+import net.sourceforge.plantuml.svek.ConditionEndStyle;
 import net.sourceforge.plantuml.svek.ConditionStyle;
 import net.sourceforge.plantuml.svek.PackageStyle;
-import net.sourceforge.plantuml.ugraphic.ColorMapper;
-import net.sourceforge.plantuml.ugraphic.ColorMapperIdentity;
-import net.sourceforge.plantuml.ugraphic.ColorMapperMonochrome;
-import net.sourceforge.plantuml.ugraphic.ColorMapperReverse;
-import net.sourceforge.plantuml.ugraphic.ColorOrder;
+import net.sourceforge.plantuml.svg.LengthAdjust;
 import net.sourceforge.plantuml.ugraphic.UFont;
 import net.sourceforge.plantuml.ugraphic.UStroke;
-import net.sourceforge.plantuml.ugraphic.sprite.Sprite;
-import net.sourceforge.plantuml.ugraphic.sprite.SpriteImage;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapper;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapperIdentity;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapperLightnessInverse;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapperMonochrome;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapperReverse;
+import net.sourceforge.plantuml.ugraphic.color.ColorOrder;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorSet;
+import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
+import net.sourceforge.plantuml.ugraphic.color.NoSuchColorException;
 
 public class SkinParam implements ISkinParam {
 
+	// TODO not clear whether SkinParam or ImageBuilder is responsible for defaults
+	public static final String DEFAULT_PRESERVE_ASPECT_RATIO = "none";
+
+	// private String skin = "debug.skin";
+
+	private String skin = "plantuml.skin";
+	private StyleBuilder styleBuilder;
+
+	private SkinParam(UmlDiagramType type) {
+		UseStyle.setBetaStyle(false);
+		this.type = type;
+		if (type == UmlDiagramType.MINDMAP) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.WBS) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.GANTT) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.JSON) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.GIT) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.BOARD) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.YAML) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.NWDIAG) {
+			UseStyle.setBetaStyle(true);
+		}
+		if (type == UmlDiagramType.SEQUENCE) {
+			// skin = "debug.skin";
+			// USE_STYLE2.set(true);
+		}
+		// if (type == UmlDiagramType.ACTIVITY) {
+		// // skin = "debug.skin";
+		// USE_STYLE2.set(true);
+		// }
+	}
+
+	public StyleBuilder getCurrentStyleBuilder() {
+		if (styleBuilder == null && UseStyle.useBetaStyle()) {
+			try {
+				this.styleBuilder = getCurrentStyleBuilderInternal();
+			} catch (IOException e) {
+				e.printStackTrace();
+			}
+		}
+		return styleBuilder;
+	}
+
+	public void muteStyle(Style modifiedStyle) {
+		if (UseStyle.useBetaStyle()) {
+			styleBuilder = getCurrentStyleBuilder().muteStyle(modifiedStyle);
+		}
+	}
+
+	public String getDefaultSkin() {
+		return skin;
+	}
+
+	public void setDefaultSkin(String newSkin) {
+		this.skin = newSkin;
+	}
+
+	public StyleBuilder getCurrentStyleBuilderInternal() throws IOException {
+		final StyleLoader tmp = new StyleLoader(this);
+		StyleBuilder result = tmp.loadSkin(this.getDefaultSkin());
+		if (result == null) {
+			result = tmp.loadSkin("plantuml.skin");
+		}
+
+		return result;
+	}
+
+	public static int zeroMargin(int defaultValue) {
+		return defaultValue;
+	}
+
 	private static final String stereoPatternString = "\\<\\<(.*?)\\>\\>";
 	private static final Pattern2 stereoPattern = MyPattern.cmpile(stereoPatternString);
 
 	private final Map<String, String> params = new HashMap<String, String>();
+	private final Map<String, String> svgCharSizes = new HashMap<String, String>();
 	private Rankdir rankdir = Rankdir.TOP_TO_BOTTOM;
-	private String dotExecutable;
 	private final UmlDiagramType type;
+	private boolean useVizJs;
 
-	public String getDotExecutable() {
-		return dotExecutable;
+	public void copyAllFrom(ISkinSimple other) {
+		this.params.putAll(other.values());
 	}
 
-	public void setDotExecutable(String dotExecutable) {
-		Log.info("Overwritting dot in skinparam " + dotExecutable);
-		this.dotExecutable = dotExecutable;
+	public Map<String, String> values() {
+		return Collections.unmodifiableMap(params);
 	}
 
 	public void setParam(String key, String value) {
 		for (String key2 : cleanForKey(key)) {
 			params.put(key2, StringUtils.trin(value));
+			if (key2.startsWith("usebetastyle")) {
+				final boolean betastyle = "true".equalsIgnoreCase(value);
+				UseStyle.setBetaStyle(betastyle);
+			}
+			if (UseStyle.useBetaStyle()) {
+				final FromSkinparamToStyle convertor = new FromSkinparamToStyle(key2, value, getCurrentStyleBuilder());
+				for (Style style : convertor.getStyles()) {
+					muteStyle(style);
+				}
+			}
+		}
+		if ("style".equalsIgnoreCase(key) && "strictuml".equalsIgnoreCase(value)) {
+			if (UseStyle.useBetaStyle()) {
+				final InputStream internalIs = StyleLoader.class.getResourceAsStream("/skin/strictuml.skin");
+				final StyleBuilder styleBuilder = this.getCurrentStyleBuilder();
+				try {
+					final BlocLines lines = BlocLines.load(internalIs, null);
+					for (Style modifiedStyle : StyleLoader.getDeclaredStyles(lines, styleBuilder)) {
+						this.muteStyle(modifiedStyle);
+					}
+				} catch (IOException e) {
+					e.printStackTrace();
+				}
+			}
 		}
-	}
-
-	private SkinParam(UmlDiagramType type) {
-		this.type = type;
 	}
 
 	public static SkinParam create(UmlDiagramType type) {
@@ -139,7 +257,7 @@ public class SkinParam implements ISkinParam {
 		key = key.replaceAll("sequencearrow", "arrow");
 		key = key.replaceAll("align$", "alignment");
 		final Matcher2 mm = stereoPattern.matcher(key);
-		final List<String> result = new ArrayList<String>();
+		final List<String> result = new ArrayList<>();
 		while (mm.find()) {
 			final String s = mm.group(1);
 			result.add(key.replaceAll(stereoPatternString, "") + "<<" + s + ">>");
@@ -157,18 +275,21 @@ public class SkinParam implements ISkinParam {
 		return s.replaceAll(src, target);
 	}
 
-	public HtmlColor getHyperlinkColor() {
-		final HtmlColor result = getHtmlColor(ColorParam.hyperlink, null, false);
+	public HColor getHyperlinkColor() {
+		final HColor result = getHtmlColor(ColorParam.hyperlink, null, false);
 		if (result == null) {
-			return HtmlColorUtils.BLUE;
+			return HColorUtils.BLUE;
 		}
 		return result;
 	}
 
-	public HtmlColor getBackgroundColor() {
-		final HtmlColor result = getHtmlColor(ColorParam.background, null, false);
+	public HColor getBackgroundColor(boolean replaceTransparentByWhite) {
+		final HColor result = getHtmlColor(ColorParam.background, null, false);
 		if (result == null) {
-			return HtmlColorUtils.WHITE;
+			return HColorUtils.WHITE;
+		}
+		if (replaceTransparentByWhite && HColorUtils.transparent().equals(result)) {
+			return HColorUtils.WHITE;
 		}
 		return result;
 	}
@@ -183,6 +304,19 @@ public class SkinParam implements ISkinParam {
 		return null;
 	}
 
+	public String getValue(String key, String defaultValue) {
+		final String result = getValue(key);
+		return result == null ? defaultValue : result;
+	}
+
+	private boolean valueIs(String key, String expected) {
+		return expected.equalsIgnoreCase(getValue(key));
+	}
+
+	private boolean isTrue(String key) {
+		return valueIs(key, "true");
+	}
+
 	static String humanName(String key) {
 		final StringBuilder sb = new StringBuilder();
 		boolean upper = true;
@@ -198,13 +332,13 @@ public class SkinParam implements ISkinParam {
 		return sb.toString();
 	}
 
-	public HtmlColor getHtmlColor(ColorParam param, Stereotype stereotype, boolean clickable) {
+	public HColor getHtmlColor(ColorParam param, Stereotype stereotype, boolean clickable) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
 			for (String s : stereotype.getMultipleLabels()) {
 				final String value2 = getValue(param.name() + "color" + "<<" + s + ">>");
-				if (value2 != null && getIHtmlColorSet().getColorIfValid(value2) != null) {
-					return getIHtmlColorSet().getColorIfValid(value2);
+				if (value2 != null && getIHtmlColorSet().getColorOrWhite(themeStyle, value2) != null) {
+					return getIHtmlColorSet().getColorOrWhite(themeStyle, value2);
 				}
 			}
 		}
@@ -212,35 +346,41 @@ public class SkinParam implements ISkinParam {
 		if (value == null) {
 			return null;
 		}
-		final boolean acceptTransparent = param == ColorParam.background
-				|| param == ColorParam.sequenceGroupBodyBackground || param == ColorParam.sequenceBoxBackground;
-		return getIHtmlColorSet().getColorIfValid(value, acceptTransparent);
+		if ((param == ColorParam.background || param == ColorParam.arrowHead)
+				&& (value.equalsIgnoreCase("transparent") || value.equalsIgnoreCase("none"))) {
+			return HColorUtils.transparent();
+		}
+		if (param == ColorParam.background) {
+			return getIHtmlColorSet().getColorOrWhite(themeStyle, value);
+		}
+		assert param != ColorParam.background;
+//		final boolean acceptTransparent = param == ColorParam.background
+//				|| param == ColorParam.sequenceGroupBodyBackground || param == ColorParam.sequenceBoxBackground;
+		return getIHtmlColorSet().getColorOrWhite(themeStyle, value, getBackgroundColor(false));
 	}
 
 	public char getCircledCharacter(Stereotype stereotype) {
-		if (stereotype == null) {
-			throw new IllegalArgumentException();
-		}
-		final String value2 = getValue("spotchar" + stereotype.getLabel(false));
+		final String value2 = getValue(
+				"spotchar" + Objects.requireNonNull(stereotype).getLabel(Guillemet.DOUBLE_COMPARATOR));
 		if (value2 != null && value2.length() > 0) {
 			return value2.charAt(0);
 		}
 		return 0;
 	}
 
-	public Colors getColors(ColorParam param, Stereotype stereotype) {
+	public Colors getColors(ColorParam param, Stereotype stereotype) throws NoSuchColorException {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue(param.name() + "color" + stereotype.getLabel(false));
-			if (value2 != null && getIHtmlColorSet().getColorIfValid(value2) != null) {
-				return new Colors(value2, getIHtmlColorSet(), param.getColorType());
+			final String value2 = getValue(param.name() + "color" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
+			if (value2 != null) {
+				return new Colors(themeStyle, value2, getIHtmlColorSet(), param.getColorType());
 			}
 		}
 		final String value = getValue(getParamName(param, false));
 		if (value == null) {
 			return Colors.empty();
 		}
-		return new Colors(value, getIHtmlColorSet(), param.getColorType());
+		return new Colors(themeStyle, value, getIHtmlColorSet(), param.getColorType());
 	}
 
 	private String getParamName(ColorParam param, boolean clickable) {
@@ -254,7 +394,8 @@ public class SkinParam implements ISkinParam {
 	}
 
 	private void checkStereotype(Stereotype stereotype) {
-		// if (stereotype.startsWith("<<") == false || stereotype.endsWith(">>") == false) {
+		// if (stereotype.startsWith("<<") == false || stereotype.endsWith(">>") ==
+		// false) {
 		// throw new IllegalArgumentException();
 		// }
 	}
@@ -262,7 +403,8 @@ public class SkinParam implements ISkinParam {
 	private int getFontSize(Stereotype stereotype, FontParam... param) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getFirstValueNonNullWithSuffix("fontsize" + stereotype.getLabel(false), param);
+			final String value2 = getFirstValueNonNullWithSuffix(
+					"fontsize" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR), param);
 			if (value2 != null && value2.matches("\\d+")) {
 				return Integer.parseInt(value2);
 			}
@@ -280,7 +422,8 @@ public class SkinParam implements ISkinParam {
 	private String getFontFamily(Stereotype stereotype, FontParam... param) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getFirstValueNonNullWithSuffix("fontname" + stereotype.getLabel(false), param);
+			final String value2 = getFirstValueNonNullWithSuffix(
+					"fontname" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR), param);
 			if (value2 != null) {
 				return StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(value2);
 			}
@@ -299,22 +442,26 @@ public class SkinParam implements ISkinParam {
 		return param[0].getDefaultFamily();
 	}
 
-	public HtmlColor getFontHtmlColor(Stereotype stereotype, FontParam... param) {
+	public HColor getFontHtmlColor(Stereotype stereotype, FontParam... param) {
 		String value = null;
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			value = getFirstValueNonNullWithSuffix("fontcolor" + stereotype.getLabel(false), param);
+			value = getFirstValueNonNullWithSuffix("fontcolor" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR),
+					param);
 		}
-		if (value == null || getIHtmlColorSet().getColorIfValid(value) == null) {
+		if (value == null) {
 			value = getFirstValueNonNullWithSuffix("fontcolor", param);
 		}
-		if (value == null || getIHtmlColorSet().getColorIfValid(value) == null) {
+		if (value == null) {
 			value = getValue("defaultfontcolor");
 		}
-		if (value == null || getIHtmlColorSet().getColorIfValid(value) == null) {
+		if (value == null) {
 			value = param[0].getDefaultColor();
 		}
-		return getIHtmlColorSet().getColorIfValid(value);
+		if (value == null) {
+			return null;
+		}
+		return getIHtmlColorSet().getColorOrWhite(themeStyle, value);
 	}
 
 	private String getFirstValueNonNullWithSuffix(String suffix, FontParam... param) {
@@ -331,7 +478,8 @@ public class SkinParam implements ISkinParam {
 		String value = null;
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			value = getFirstValueNonNullWithSuffix("fontstyle" + stereotype.getLabel(false), param);
+			value = getFirstValueNonNullWithSuffix("fontstyle" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR),
+					param);
 		}
 		if (value == null) {
 			value = getFirstValueNonNullWithSuffix("fontstyle", param);
@@ -363,26 +511,16 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public int getCircledCharacterRadius() {
-		final String value = getValue("circledcharacterradius");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		// return 11;
-		// Log.println("SIZE1="+getFontSize(FontParam.CIRCLED_CHARACTER));
-		// Log.println("SIZE1="+getFontSize(FontParam.CIRCLED_CHARACTER)/3);
-		return getFontSize(null, FontParam.CIRCLED_CHARACTER) / 3 + 6;
+		final int value = getAsInt("circledCharacterRadius", -1);
+		return value == -1 ? getFontSize(null, FontParam.CIRCLED_CHARACTER) / 3 + 6 : value;
 	}
 
 	public int classAttributeIconSize() {
-		final String value = getValue("classAttributeIconSize");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		return 10;
+		return getAsInt("classAttributeIconSize", 10);
 	}
 
 	public static Collection<String> getPossibleValues() {
-		final Set<String> result = new TreeSet<String>();
+		final Set<String> result = new TreeSet<>();
 		result.add("Monochrome");
 		// result.add("BackgroundColor");
 		result.add("CircledCharacterRadius");
@@ -412,9 +550,12 @@ public class SkinParam implements ISkinParam {
 		result.add("Style");
 		result.add("SequenceParticipant");
 		result.add("ConditionStyle");
+		result.add("ConditionEndStyle");
 		result.add("SameClassWidth");
 		result.add("HyperlinkUnderline");
 		result.add("Padding");
+		result.add("BoxPadding");
+		result.add("ParticipantPadding");
 		result.add("Guillemet");
 		result.add("SvglinkTarget");
 		result.add("DefaultMonospacedFontName");
@@ -431,6 +572,8 @@ public class SkinParam implements ISkinParam {
 		result.add("WrapWidth");
 		result.add("SwimlaneWidth");
 		result.add("SwimlaneWrapTitleWidth");
+		result.add("FixCircleLabelOverlapping");
+		result.add("LifelineStrategy");
 
 		for (FontParam p : EnumSet.allOf(FontParam.class)) {
 			final String h = humanName(p.name());
@@ -459,11 +602,7 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public int getDpi() {
-		final String value = getValue("dpi");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		return 96;
+		return getAsInt("dpi", 96);
 	}
 
 	public DotSplines getDotSplines() {
@@ -477,7 +616,8 @@ public class SkinParam implements ISkinParam {
 		return DotSplines.SPLINES;
 	}
 
-	public HorizontalAlignment getHorizontalAlignment(AlignmentParam param, ArrowDirection arrowDirection) {
+	public HorizontalAlignment getHorizontalAlignment(AlignmentParam param, ArrowDirection arrowDirection,
+			boolean isReverseDefine, HorizontalAlignment overrideDefault) {
 		final String value;
 		switch (param) {
 		case sequenceMessageAlignment:
@@ -489,6 +629,19 @@ public class SkinParam implements ISkinParam {
 		default:
 			value = getValue(param.name());
 		}
+		if ("first".equalsIgnoreCase(value)) {
+			if (arrowDirection == ArrowDirection.RIGHT_TO_LEFT_REVERSE) {
+				if (isReverseDefine) {
+					return HorizontalAlignment.LEFT;
+				}
+				return HorizontalAlignment.RIGHT;
+			} else {
+				if (isReverseDefine) {
+					return HorizontalAlignment.RIGHT;
+				}
+				return HorizontalAlignment.LEFT;
+			}
+		}
 		if ("direction".equalsIgnoreCase(value)) {
 			if (arrowDirection == ArrowDirection.LEFT_TO_RIGHT_NORMAL) {
 				return HorizontalAlignment.LEFT;
@@ -513,7 +666,9 @@ public class SkinParam implements ISkinParam {
 		}
 		final HorizontalAlignment result = HorizontalAlignment.fromString(value);
 		if (result == null && param == AlignmentParam.noteTextAlignment) {
-			return getDefaultTextAlignment(HorizontalAlignment.LEFT);
+			return getDefaultTextAlignment(overrideDefault == null ? HorizontalAlignment.LEFT : overrideDefault);
+		} else if (result == null && param == AlignmentParam.stateMessageAlignment) {
+			return getDefaultTextAlignment(HorizontalAlignment.CENTER);
 		} else if (result == null) {
 			return param.getDefaultValue();
 		}
@@ -529,6 +684,15 @@ public class SkinParam implements ISkinParam {
 		return result;
 	}
 
+	public HorizontalAlignment getStereotypeAlignment() {
+		final String value = getValue("stereotypealignment");
+		final HorizontalAlignment result = HorizontalAlignment.fromString(value);
+		if (result == null) {
+			return HorizontalAlignment.CENTER;
+		}
+		return result;
+	}
+
 	private String getArg(String value, int i) {
 		if (value == null) {
 			return null;
@@ -552,6 +716,9 @@ public class SkinParam implements ISkinParam {
 		if (value == null) {
 			return new ColorMapperIdentity();
 		}
+		if ("dark".equalsIgnoreCase(value)) {
+			return new ColorMapperLightnessInverse();
+		}
 		final ColorOrder order = ColorOrder.fromString(value);
 		if (order == null) {
 			return new ColorMapperIdentity();
@@ -559,7 +726,14 @@ public class SkinParam implements ISkinParam {
 		return new ColorMapperReverse(order);
 	}
 
-	public boolean shadowing() {
+	public boolean shadowing(Stereotype stereotype) {
+		if (stereotype != null) {
+			checkStereotype(stereotype);
+			final String value2 = getValue("shadowing" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
+			if (value2 != null) {
+				return value2.equalsIgnoreCase("true");
+			}
+		}
 		final String value = getValue("shadowing");
 		if ("false".equalsIgnoreCase(value)) {
 			return false;
@@ -576,7 +750,7 @@ public class SkinParam implements ISkinParam {
 	public boolean shadowingForNote(Stereotype stereotype) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue("note" + "shadowing" + stereotype.getLabel(false));
+			final String value2 = getValue("note" + "shadowing" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null) {
 				return value2.equalsIgnoreCase("true");
 			}
@@ -585,17 +759,22 @@ public class SkinParam implements ISkinParam {
 		if (value2 != null) {
 			return value2.equalsIgnoreCase("true");
 		}
-		return shadowing();
+		return shadowing(stereotype);
 	}
 
-	public boolean shadowing2(SkinParameter skinParameter) {
-		if (skinParameter == null) {
-			throw new IllegalArgumentException();
+	public boolean shadowing2(Stereotype stereotype, SkinParameter skinParameter) {
+		final String name = Objects.requireNonNull(skinParameter).getUpperCaseName();
+		if (stereotype != null) {
+			checkStereotype(stereotype);
+			final String value2 = getValue(name + "shadowing" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
+			if (value2 != null) {
+				return value2.equalsIgnoreCase("true");
+			}
 		}
-		final String name = skinParameter.getUpperCaseName();
+
 		final String value = getValue(name + "shadowing");
 		if (value == null) {
-			return shadowing();
+			return shadowing(stereotype);
 		}
 		if ("false".equalsIgnoreCase(value)) {
 			return false;
@@ -609,17 +788,12 @@ public class SkinParam implements ISkinParam {
 		return true;
 	}
 
-	public PackageStyle getPackageStyle() {
-		final String value = getValue("packageStyle");
-		final PackageStyle p = PackageStyle.fromString(value);
-		if (p == null) {
-			return PackageStyle.FOLDER;
-		}
-		return p;
-	}
-
 	private final Map<String, Sprite> sprites = new HashMap<String, Sprite>();
 
+	public Collection<String> getAllSpriteNames() {
+		return Collections.unmodifiableCollection(new TreeSet<>(sprites.keySet()));
+	}
+
 	public void addSprite(String name, Sprite sprite) {
 		sprites.put(name, sprite);
 	}
@@ -632,49 +806,52 @@ public class SkinParam implements ISkinParam {
 		return result;
 	}
 
-	public boolean useUml2ForComponent() {
+	public PackageStyle packageStyle() {
+		final String value = getValue("packageStyle");
+		final PackageStyle p = PackageStyle.fromString(value);
+		if (p == null) {
+			return PackageStyle.FOLDER;
+		}
+		return p;
+	}
+
+	public ComponentStyle componentStyle() {
 		if (strictUmlStyle()) {
-			return true;
+			return ComponentStyle.UML2;
 		}
 		final String value = getValue("componentstyle");
-		return "uml2".equalsIgnoreCase(value);
+		if ("uml1".equalsIgnoreCase(value))
+			return ComponentStyle.UML1;
+		if ("uml2".equalsIgnoreCase(value))
+			return ComponentStyle.UML2;
+		if ("rectangle".equalsIgnoreCase(value))
+			return ComponentStyle.RECTANGLE;
+		return ComponentStyle.UML2;
 	}
 
 	public boolean stereotypePositionTop() {
-		final String value = getValue("stereotypePosition");
-		if ("bottom".equalsIgnoreCase(value)) {
-			return false;
-		}
-		return true;
+		return !valueIs("stereotypePosition", "bottom");
 	}
 
 	public boolean useSwimlanes(UmlDiagramType type) {
 		if (type != UmlDiagramType.ACTIVITY) {
 			return false;
 		}
-		if ("true".equalsIgnoreCase(getValue("swimlane"))) {
-			return true;
-		}
-		if ("true".equalsIgnoreCase(getValue("swimlanes"))) {
-			return true;
-		}
-		return false;
+		return swimlanes();
+	}
+
+	public boolean swimlanes() {
+		return isTrue("swimlane") || isTrue("swimlanes");
 	}
 
 	public double getNodesep() {
-		final String value = getValue("nodesep");
-		if (value != null && value.matches("\\d+")) {
-			return Double.parseDouble(value);
-		}
-		return 0;
+		// TODO strange, this returns a double but only accepts integer values
+		return getAsInt("nodesep", 0);
 	}
 
 	public double getRanksep() {
-		final String value = getValue("ranksep");
-		if (value != null && value.matches("\\d+")) {
-			return Double.parseDouble(value);
-		}
-		return 0;
+		// TODO strange, this returns a double but only accepts integer values
+		return getAsInt("ranksep", 0);
 	}
 
 	public double getDiagonalCorner(CornerParam param, Stereotype stereotype) {
@@ -711,7 +888,7 @@ public class SkinParam implements ISkinParam {
 
 	private Double getCornerInternal(String key, CornerParam param, Stereotype stereotype) {
 		if (stereotype != null) {
-			key += stereotype.getLabel(false);
+			key += stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR);
 		}
 		final String value = getValue(key);
 		if (value != null && value.matches("\\d+")) {
@@ -725,12 +902,14 @@ public class SkinParam implements ISkinParam {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
 
-			final String styleValue = getValue(param.name() + "style" + stereotype.getLabel(false));
+			final String styleValue = getValue(
+					param.name() + "style" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (styleValue != null) {
 				style = LinkStyle.fromString2(styleValue);
 			}
 
-			final String value2 = getValue(param.name() + "thickness" + stereotype.getLabel(false));
+			final String value2 = getValue(
+					param.name() + "thickness" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null && value2.matches("[\\d.]+")) {
 				if (style == null) {
 					style = LinkStyle.NORMAL();
@@ -776,40 +955,37 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public boolean strictUmlStyle() {
-		final String value = getValue("style");
-		if ("strictuml".equalsIgnoreCase(value)) {
-			return true;
-		}
-		return false;
+		return valueIs("style", "strictuml");
 	}
 
 	public boolean forceSequenceParticipantUnderlined() {
-		final String value = getValue("sequenceParticipant");
-		if ("underline".equalsIgnoreCase(value)) {
-			return true;
-		}
-		return false;
+		return valueIs("sequenceParticipant", "underline");
 	}
 
 	public ConditionStyle getConditionStyle() {
 		final String value = getValue("conditionStyle");
 		final ConditionStyle p = ConditionStyle.fromString(value);
 		if (p == null) {
-			return ConditionStyle.INSIDE;
+			return ConditionStyle.INSIDE_HEXAGON;
+		}
+		return p;
+	}
+
+	public ConditionEndStyle getConditionEndStyle() {
+		final String value = getValue("conditionEndStyle");
+		final ConditionEndStyle p = ConditionEndStyle.fromString(value);
+		if (p == null) {
+			return ConditionEndStyle.DIAMOND;
 		}
 		return p;
 	}
 
 	public double minClassWidth() {
-		final String value = getValue("minclasswidth");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		return 0;
+		return getAsInt("minclasswidth", 0);
 	}
 
 	public boolean sameClassWidth() {
-		return "true".equals(getValue("sameclasswidth"));
+		return isTrue("sameclasswidth");
 	}
 
 	public final Rankdir getRankdir() {
@@ -824,7 +1000,7 @@ public class SkinParam implements ISkinParam {
 		String value = getValue("activityshape");
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue("activityshape" + stereotype.getLabel(false));
+			final String value2 = getValue("activityshape" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null) {
 				value = value2;
 			}
@@ -838,100 +1014,61 @@ public class SkinParam implements ISkinParam {
 		return false;
 	}
 
-	private final IHtmlColorSet htmlColorSet = new HtmlColorSetSimple();
+	private final HColorSet htmlColorSet = HColorSet.instance();
 
-	public IHtmlColorSet getIHtmlColorSet() {
+	public HColorSet getIHtmlColorSet() {
 		return htmlColorSet;
 	}
 
 	public boolean useUnderlineForHyperlink() {
-		final String value = getValue("hyperlinkunderline");
-		if ("false".equalsIgnoreCase(value)) {
-			return false;
-		}
-		return true;
+		return !valueIs("hyperlinkunderline", "false");
 	}
 
 	public int groupInheritance() {
-		final String value = getValue("groupinheritance");
-		int result = Integer.MAX_VALUE;
-		if (value != null && value.matches("\\d+")) {
-			result = Integer.parseInt(value);
-		}
-		if (result <= 1) {
-			result = Integer.MAX_VALUE;
-		}
-		return result;
+		final int value = getAsInt("groupinheritance", Integer.MAX_VALUE);
+		return value <= 1 ? Integer.MAX_VALUE : value;
 	}
 
-	public boolean useGuillemet() {
+	public Guillemet guillemet() {
 		final String value = getValue("guillemet");
-		if ("false".equalsIgnoreCase(value)) {
-			return false;
-		}
-		return true;
+		return Guillemet.GUILLEMET.fromDescription(value);
 	}
 
 	public boolean handwritten() {
-		final String value = getValue("handwritten");
-		if ("true".equalsIgnoreCase(value)) {
-			return true;
-		}
-		return false;
+		return isTrue("handwritten");
 	}
 
 	public String getSvgLinkTarget() {
-		final String value = getValue("svglinktarget");
-		if (value == null) {
-			return "_top";
-		}
-		return value;
+		return getValue("svglinktarget", "_top");
+	}
+
+	public String getPreserveAspectRatio() {
+		return getValue("preserveaspectratio", DEFAULT_PRESERVE_ASPECT_RATIO);
 	}
 
 	public String getMonospacedFamily() {
-		final String value = getValue("defaultMonospacedFontName");
-		if (value == null) {
-			return CommandCreoleMonospaced.MONOSPACED;
-		}
-		return value;
+		return getValue("defaultMonospacedFontName", Parser.MONOSPACED);
 	}
 
 	public int getTabSize() {
-		final String value = getValue("tabsize");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		return 8;
+		return getAsInt("tabsize", 8);
 	}
 
 	public int maxAsciiMessageLength() {
-		final String value = getValue("maxasciimessagelength");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		return -1;
+		return getAsInt("maxasciimessagelength", -1);
 	}
 
 	public int colorArrowSeparationSpace() {
-		final String value = getValue("colorarrowseparationspace");
-		if (value != null && value.matches("\\d+")) {
-			return Integer.parseInt(value);
-		}
-		return 0;
+		return getAsInt("colorarrowseparationspace", 0);
 	}
 
 	public SplitParam getSplitParam() {
 		final String border = getValue("pageBorderColor");
 		final String external = getValue("pageExternalColor");
-
-		final String marginString = getValue("pageMargin");
-		int margin = 0;
-		if (marginString != null && marginString.matches("\\d+")) {
-			margin = Integer.parseInt(marginString);
-		}
-
-		return new SplitParam(getIHtmlColorSet().getColorIfValid(border), getIHtmlColorSet().getColorIfValid(external),
-				margin);
+		final HColor borderColor = border == null ? null : getIHtmlColorSet().getColorOrWhite(themeStyle, border);
+		final HColor externalColor = external == null ? null : getIHtmlColorSet().getColorOrWhite(themeStyle, external);
+		int margin = getAsInt("pageMargin", 0);
+		return new SplitParam(borderColor, externalColor, margin);
 	}
 
 	public int swimlaneWidth() {
@@ -949,28 +1086,38 @@ public class SkinParam implements ISkinParam {
 		return type;
 	}
 
-	public HtmlColor getHoverPathColor() {
+	public HColor hoverPathColor() {
 		final String value = getValue("pathhovercolor");
 		if (value == null) {
 			return null;
 		}
-		return getIHtmlColorSet().getColorIfValid(value, false);
+		return getIHtmlColorSet().getColorOrWhite(themeStyle, value, null);
 	}
 
 	public double getPadding() {
-		final String value = getValue("padding");
+		final String name = "padding";
+		return getAsDouble(name);
+	}
+
+	public double getPadding(PaddingParam param) {
+		final String name = param.getSkinName();
+		return getAsDouble(name);
+	}
+
+	private double getAsDouble(final String name) {
+		final String value = getValue(name);
 		if (value != null && value.matches("\\d+(\\.\\d+)?")) {
 			return Double.parseDouble(value);
 		}
 		return 0;
 	}
 
-	public double getPadding(PaddingParam param) {
-		final String value = getValue(param.getSkinName());
-		if (value != null && value.matches("\\d+(\\.\\d+)?")) {
-			return Double.parseDouble(value);
+	private int getAsInt(String key, int defaultValue) {
+		final String value = getValue(key);
+		if (value != null && value.matches("\\d+")) {
+			return Integer.parseInt(value);
 		}
-		return 0;
+		return defaultValue;
 	}
 
 	public boolean useRankSame() {
@@ -978,19 +1125,11 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public boolean displayGenericWithOldFashion() {
-		final String value = getValue("genericDisplay");
-		if ("old".equalsIgnoreCase(value)) {
-			return true;
-		}
-		return false;
+		return valueIs("genericDisplay", "old");
 	}
 
 	public boolean responseMessageBelowArrow() {
-		final String value = getValue("responsemessagebelowarrow");
-		if ("true".equalsIgnoreCase(value)) {
-			return true;
-		}
-		return false;
+		return isTrue("responsemessagebelowarrow");
 	}
 
 	public TikzFontDistortion getTikzFontDistortion() {
@@ -999,11 +1138,82 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public boolean svgDimensionStyle() {
-		final String value = getValue("svgdimensionstyle");
-		if ("false".equalsIgnoreCase(value)) {
-			return false;
+		return !valueIs("svgdimensionstyle", "false");
+	}
+
+	public boolean fixCircleLabelOverlapping() {
+		return isTrue("fixcirclelabeloverlapping");
+	}
+
+	public void setUseVizJs(boolean useVizJs) {
+		this.useVizJs = useVizJs;
+	}
+
+	public boolean isUseVizJs() {
+		return useVizJs;
+	}
+
+	public Padder sequenceDiagramPadder() {
+		final double padding = getAsDouble("SequenceMessagePadding");
+		final double margin = getAsDouble("SequenceMessageMargin");
+		final String borderColor = getValue("SequenceMessageBorderColor");
+		final String backgroundColor = getValue("SequenceMessageBackGroundColor");
+		if (padding == 0 && margin == 0 && borderColor == null && backgroundColor == null) {
+			return Padder.NONE;
 		}
-		return true;
+		final HColor border = borderColor == null ? null : getIHtmlColorSet().getColorOrWhite(themeStyle, borderColor);
+		final HColor background = backgroundColor == null ? null
+				: getIHtmlColorSet().getColorOrWhite(themeStyle, backgroundColor);
+		final double roundCorner = getRoundCorner(CornerParam.DEFAULT, null);
+		return Padder.NONE.withMargin(margin).withPadding(padding).withBackgroundColor(background)
+				.withBorderColor(border).withRoundCorner(roundCorner);
+	}
+
+	public ActorStyle actorStyle() {
+		final String value = getValue("actorstyle");
+		if ("awesome".equalsIgnoreCase(value)) {
+			return ActorStyle.AWESOME;
+		}
+		if ("hollow".equalsIgnoreCase(value)) {
+			return ActorStyle.HOLLOW;
+		}
+		return ActorStyle.STICKMAN;
+	}
+
+	public void setSvgSize(String origin, String sizeToUse) {
+		svgCharSizes.put(StringUtils.manageUnicodeNotationUplus(origin),
+				StringUtils.manageUnicodeNotationUplus(sizeToUse));
+	}
+
+	public String transformStringForSizeHack(String s) {
+		for (Entry<String, String> ent : svgCharSizes.entrySet()) {
+			s = s.replace(ent.getKey(), ent.getValue());
+		}
+		return s;
+	}
+
+	public LengthAdjust getlengthAdjust() {
+		final String value = getValue("lengthAdjust");
+		if ("spacingAndGlyphs".equalsIgnoreCase(value)) {
+			return LengthAdjust.SPACING_AND_GLYPHS;
+		}
+		if ("spacing".equalsIgnoreCase(value)) {
+			return LengthAdjust.SPACING;
+		}
+		if ("none".equalsIgnoreCase(value)) {
+			return LengthAdjust.NONE;
+		}
+		return LengthAdjust.defaultValue();
+	}
+
+	private ThemeStyle themeStyle = ThemeStyle.LIGHT;
+
+	public void assumeTransparent(ThemeStyle style) {
+		this.themeStyle = style;
+	}
+
+	public ThemeStyle getThemeStyle() {
+		return themeStyle;
 	}
 
 }
