@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,11 +35,13 @@
  */
 package net.sourceforge.plantuml;
 
+import static net.sourceforge.plantuml.ugraphic.ImageBuilder.plainImageBuilder;
+
+import java.awt.Color;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Dimension2D;
 import java.awt.image.BufferedImage;
 import java.io.BufferedOutputStream;
-import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileNotFoundException;
@@ -49,17 +51,11 @@ import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.util.List;
 
-import javax.imageio.ImageIO;
-import javax.script.ScriptException;
-
-import net.sourceforge.plantuml.anim.Animation;
-import net.sourceforge.plantuml.anim.AnimationDecoder;
 import net.sourceforge.plantuml.api.ImageDataSimple;
 import net.sourceforge.plantuml.command.CommandExecutionResult;
 import net.sourceforge.plantuml.core.Diagram;
 import net.sourceforge.plantuml.core.ImageData;
 import net.sourceforge.plantuml.core.UmlSource;
-import net.sourceforge.plantuml.cucadiagram.DisplayPositionned;
 import net.sourceforge.plantuml.cucadiagram.DisplaySection;
 import net.sourceforge.plantuml.cucadiagram.UnparsableGraphvizException;
 import net.sourceforge.plantuml.flashcode.FlashCodeFactory;
@@ -67,60 +63,36 @@ import net.sourceforge.plantuml.flashcode.FlashCodeUtils;
 import net.sourceforge.plantuml.fun.IconLoader;
 import net.sourceforge.plantuml.graphic.GraphicPosition;
 import net.sourceforge.plantuml.graphic.GraphicStrings;
-import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
-import net.sourceforge.plantuml.graphic.HtmlColorUtils;
 import net.sourceforge.plantuml.graphic.UDrawable;
-import net.sourceforge.plantuml.graphic.VerticalAlignment;
 import net.sourceforge.plantuml.mjpeg.MJPEGGenerator;
 import net.sourceforge.plantuml.pdf.PdfConverter;
+import net.sourceforge.plantuml.security.ImageIO;
+import net.sourceforge.plantuml.security.SFile;
+import net.sourceforge.plantuml.security.SecurityUtils;
+import net.sourceforge.plantuml.style.NoStyleAvailableException;
 import net.sourceforge.plantuml.svek.EmptySvgException;
 import net.sourceforge.plantuml.svek.GraphvizCrash;
 import net.sourceforge.plantuml.svek.TextBlockBackcolored;
-import net.sourceforge.plantuml.ugraphic.ColorMapperIdentity;
-import net.sourceforge.plantuml.ugraphic.ImageBuilder;
+import net.sourceforge.plantuml.ugraphic.AffineTransformType;
+import net.sourceforge.plantuml.ugraphic.PixelImage;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UImage;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
-import net.sourceforge.plantuml.ugraphic.sprite.Sprite;
 import net.sourceforge.plantuml.version.Version;
 
-public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Annotated, WithSprite {
+public abstract class UmlDiagram extends TitledDiagram implements Diagram, Annotated, WithSprite {
 
 	private boolean rotation;
 	private boolean hideUnlinkedData;
 
 	private int minwidth = Integer.MAX_VALUE;
 
-	private DisplayPositionned title = DisplayPositionned.none(HorizontalAlignment.CENTER, VerticalAlignment.TOP);
-	private DisplayPositionned caption = DisplayPositionned.none(HorizontalAlignment.CENTER, VerticalAlignment.BOTTOM);
-	private DisplayPositionned legend = DisplayPositionned.none(HorizontalAlignment.CENTER, VerticalAlignment.BOTTOM);
-	private final DisplaySection header = DisplaySection.none();
-	private final DisplaySection footer = DisplaySection.none();
-
-	private final Pragma pragma = new Pragma();
-	private Animation animation;
-
-	private final SkinParam skinParam = SkinParam.create(getUmlDiagramType());
-
-	final public void setTitle(DisplayPositionned title) {
-		if (title.isNull() || title.getDisplay().isWhite()) {
-			return;
-		}
-		this.title = title;
+	public UmlDiagram(UmlSource source, UmlDiagramType type) {
+		super(source, type);
 	}
 
-	final public void setCaption(DisplayPositionned caption) {
-		this.caption = caption;
-	}
-
-	final public DisplayPositionned getCaption() {
-		return caption;
-	}
-
-	@Override
-	final public DisplayPositionned getTitle() {
-		return title;
+	public UmlDiagram(UmlSource source, UmlDiagramType type, ISkinSimple orig) {
+		super(source, type, orig);
 	}
 
 	final public int getMinwidth() {
@@ -139,22 +111,6 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 		this.rotation = rotation;
 	}
 
-	public final ISkinParam getSkinParam() {
-		return skinParam;
-	}
-
-	public void setParam(String key, String value) {
-		skinParam.setParam(StringUtils.goLowerCase(key), value);
-	}
-
-	public final DisplaySection getHeader() {
-		return header;
-	}
-
-	public final DisplaySection getFooter() {
-		return footer;
-	}
-
 	public final DisplaySection getFooterOrHeaderTeoz(FontParam param) {
 		if (param == FontParam.FOOTER) {
 			return getFooter();
@@ -165,37 +121,6 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 		throw new IllegalArgumentException();
 	}
 
-	abstract public UmlDiagramType getUmlDiagramType();
-
-	public Pragma getPragma() {
-		return pragma;
-	}
-
-	final public void setAnimation(Iterable<CharSequence> animationData) {
-		try {
-			final AnimationDecoder animationDecoder = new AnimationDecoder(animationData);
-			this.animation = Animation.create(animationDecoder.decode());
-		} catch (ScriptException e) {
-			e.printStackTrace();
-		}
-
-	}
-
-	final public Animation getAnimation() {
-		return animation;
-	}
-
-	public final double getDpiFactor(FileFormatOption fileFormatOption) {
-		if (getSkinParam().getDpi() == 96) {
-			return 1.0;
-		}
-		return getSkinParam().getDpi() / 96.0;
-	}
-
-	public final int getDpi(FileFormatOption fileFormatOption) {
-		return getSkinParam().getDpi();
-	}
-
 	public final boolean isHideUnlinkedData() {
 		return hideUnlinkedData;
 	}
@@ -205,16 +130,10 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 	}
 
 	@Override
-	final protected ImageData exportDiagramNow(OutputStream os, int index, FileFormatOption fileFormatOption, long seed)
+	final protected ImageData exportDiagramNow(OutputStream os, int index, FileFormatOption fileFormatOption)
 			throws IOException {
 
-		final HtmlColor hover = getSkinParam().getHoverPathColor();
-		fileFormatOption = fileFormatOption.withSvgLinkTarget(getSkinParam().getSvgLinkTarget());
 		fileFormatOption = fileFormatOption.withTikzFontDistortion(getSkinParam().getTikzFontDistortion());
-		if (hover != null) {
-			fileFormatOption = fileFormatOption.withHoverColor(StringUtils.getAsHtml(getSkinParam().getColorMapper()
-					.getMappedColor(hover)));
-		}
 
 		if (fileFormatOption.getFileFormat() == FileFormat.PDF) {
 			return exportDiagramInternalPdf(os, index);
@@ -224,19 +143,22 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 			final ImageData imageData = exportDiagramInternal(os, index, fileFormatOption);
 			this.lastInfo = new Dimension2DDouble(imageData.getWidth(), imageData.getHeight());
 			return imageData;
+		} catch (NoStyleAvailableException e) {
+			// e.printStackTrace();
+			exportDiagramError(os, e, fileFormatOption, null);
 		} catch (UnparsableGraphvizException e) {
 			e.printStackTrace();
-			exportDiagramError(os, e.getCause(), fileFormatOption, seed, e.getGraphvizVersion());
-		} catch (Exception e) {
-			e.printStackTrace();
-			exportDiagramError(os, e, fileFormatOption, seed, null);
+			exportDiagramError(os, e.getCause(), fileFormatOption, e.getGraphvizVersion());
+		} catch (Throwable e) {
+			//e.printStackTrace();
+			exportDiagramError(os, e, fileFormatOption, null);
 		}
 		return ImageDataSimple.error();
 	}
 
-	private void exportDiagramError(OutputStream os, Throwable exception, FileFormatOption fileFormat, long seed,
+	private void exportDiagramError(OutputStream os, Throwable exception, FileFormatOption fileFormat,
 			String graphvizVersion) throws IOException {
-		exportDiagramError(os, exception, fileFormat, seed, getMetadata(), getFlashData(),
+		exportDiagramError(os, exception, fileFormat, seed(), getMetadata(), getFlashData(),
 				getFailureText1(exception, graphvizVersion, getFlashData()));
 	}
 
@@ -250,35 +172,40 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 
 		strings.addAll(CommandExecutionResult.getStackTrace(exception));
 
-		final ImageBuilder imageBuilder = new ImageBuilder(new ColorMapperIdentity(), 1.0, HtmlColorUtils.WHITE,
-				metadata, null, 0, 0, null, false);
-
-		final FlashCodeUtils utils = FlashCodeFactory.getFlashCodeUtils();
-		final BufferedImage im = utils.exportFlashcode(flash);
-		if (im != null) {
-			GraphvizCrash.addDecodeHint(strings);
+		BufferedImage im2 = null;
+		if (flash != null) {
+			final FlashCodeUtils utils = FlashCodeFactory.getFlashCodeUtils();
+			try {
+				im2 = utils.exportFlashcode(flash, Color.BLACK, Color.WHITE);
+			} catch (Throwable e) {
+				Log.error("Issue in flashcode generation " + e);
+				// e.printStackTrace();
+			}
+			if (im2 != null) {
+				GraphvizCrash.addDecodeHint(strings);
+			}
 		}
-
+		final BufferedImage im = im2;
 		final TextBlockBackcolored graphicStrings = GraphicStrings.createBlackOnWhite(strings, IconLoader.getRandom(),
 				GraphicPosition.BACKGROUND_CORNER_TOP_RIGHT);
 
-		if (im == null) {
-			imageBuilder.setUDrawable(graphicStrings);
-		} else {
-			imageBuilder.setUDrawable(new UDrawable() {
+		final UDrawable drawable = (im == null) ? graphicStrings : new UDrawable() {
 				public void drawU(UGraphic ug) {
 					graphicStrings.drawU(ug);
 					final double height = graphicStrings.calculateDimension(ug.getStringBounder()).getHeight();
-					ug = ug.apply(new UTranslate(0, height));
-					ug.draw(new UImage(im).scaleNearestNeighbor(3));
+					ug = ug.apply(UTranslate.dy(height));
+					ug.draw(new UImage(new PixelImage(im, AffineTransformType.TYPE_NEAREST_NEIGHBOR)).scale(3));
 				}
-			});
-		}
-		imageBuilder.writeImageTOBEMOVED(fileFormat, seed, os);
+			};
+
+		plainImageBuilder(drawable, fileFormat)
+				.metadata(metadata)
+				.seed(seed)
+				.write(os);
 	}
 
 	private static void exportDiagramErrorText(OutputStream os, Throwable exception, List<String> strings) {
-		final PrintWriter pw = new PrintWriter(os);
+		final PrintWriter pw = SecurityUtils.createPrintWriter(os);
 		exception.printStackTrace(pw);
 		pw.println();
 		pw.println();
@@ -329,7 +256,7 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 	}
 
 	private void exportDiagramInternalMjpeg(OutputStream os) throws IOException {
-		final File f = new File("c:/test.avi");
+		final SFile f = new SFile("c:/test.avi");
 		final int nb = 150;
 		final double framerate = 30;
 		final MJPEGGenerator m = new MJPEGGenerator(f, 640, 480, framerate, nb);
@@ -339,9 +266,10 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 			final double coef = (nb - 1 - i) * 1.0 / nb;
 			at.setToShear(coef, coef);
 			final ByteArrayOutputStream baos = new ByteArrayOutputStream();
-			// exportDiagramTOxxBEREMOVED(baos, null, 0, new FileFormatOption(FileFormat.PNG, at));
+			// exportDiagramTOxxBEREMOVED(baos, null, 0, new
+			// FileFormatOption(FileFormat.PNG, at));
 			baos.close();
-			final BufferedImage im = ImageIO.read(new ByteArrayInputStream(baos.toByteArray()));
+			final BufferedImage im = ImageIO.read(baos.toByteArray());
 			m.addImage(im);
 		}
 		m.finishAVI();
@@ -351,11 +279,12 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 	private Dimension2D lastInfo;
 
 	private ImageData exportDiagramInternalPdf(OutputStream os, int index) throws IOException {
-		final File svg = FileUtils.createTempFile("pdf", ".svf");
-		final File pdfFile = FileUtils.createTempFile("pdf", ".pdf");
-		final OutputStream fos = new BufferedOutputStream(new FileOutputStream(svg));
-		final ImageData result = exportDiagram(fos, index, new FileFormatOption(FileFormat.SVG));
-		fos.close();
+		final File svg = FileUtils.createTempFileLegacy("pdf", ".svf");
+		final File pdfFile = FileUtils.createTempFileLegacy("pdf", ".pdf");
+		final ImageData result;
+		try (OutputStream fos = new BufferedOutputStream(new FileOutputStream(svg))) {
+			result = exportDiagram(fos, index, new FileFormatOption(FileFormat.SVG));
+		}
 		PdfConverter.convert(svg, pdfFile);
 		FileUtils.copyToStream(pdfFile, os);
 		return result;
@@ -367,19 +296,12 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 	final protected void exportCmap(SuggestedFile suggestedFile, int index, final ImageData cmapdata)
 			throws FileNotFoundException {
 		final String name = changeName(suggestedFile.getFile(index).getAbsolutePath());
-		final File cmapFile = new File(name);
-		PrintWriter pw = null;
-		try {
+		final SFile cmapFile = new SFile(name);
+		try (PrintWriter pw = cmapFile.createPrintWriter()) {
 			if (PSystemUtils.canFileBeWritten(cmapFile) == false) {
 				return;
 			}
-			pw = new PrintWriter(cmapFile);
 			pw.print(cmapdata.getCMapData(cmapFile.getName().substring(0, cmapFile.getName().length() - 6)));
-			pw.close();
-		} finally {
-			if (pw != null) {
-				pw.close();
-			}
 		}
 	}
 
@@ -410,29 +332,7 @@ public abstract class UmlDiagram extends AbstractPSystem implements Diagram, Ann
 		return null;
 	}
 
-	public void addSprite(String name, Sprite sprite) {
-		skinParam.addSprite(name, sprite);
+	public void setHideEmptyDescription(boolean hideEmptyDescription) {
 	}
 
-	public final DisplayPositionned getLegend() {
-		return legend;
-	}
-
-	public final void setLegend(DisplayPositionned legend) {
-		this.legend = legend;
-	}
-
-	private boolean useJDot;
-
-	public void setUseJDot(boolean useJDot) {
-		this.useJDot = useJDot;
-	}
-
-	public boolean isUseJDot() {
-		return useJDot;
-	}
-
-	public void setDotExecutable(String dotExecutable) {
-		skinParam.setDotExecutable(dotExecutable);
-	}
 }
