@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,6 +35,9 @@
  */
 package net.sourceforge.plantuml.activitydiagram3;
 
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Objects;
 import java.util.Set;
 
 import net.sourceforge.plantuml.activitydiagram3.ftile.BoxStyle;
@@ -42,82 +45,112 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.Ftile;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileFactory;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileKilled;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
+import net.sourceforge.plantuml.command.CommandExecutionResult;
 import net.sourceforge.plantuml.cucadiagram.Display;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.color.Colors;
 import net.sourceforge.plantuml.sequencediagram.NotePosition;
 import net.sourceforge.plantuml.sequencediagram.NoteType;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public class InstructionRepeat implements Instruction {
 
-	private final InstructionList repeatList = new InstructionList();
+	private final InstructionList repeatList;
 	private final Instruction parent;
 	private final LinkRendering nextLinkRenderer;
 	private final Swimlane swimlane;
-	private final HtmlColor color;
+	private Swimlane swimlaneOut;
+	private BoxStyle boxStyle;
 	private boolean killed = false;
+	private final BoxStyle boxStyleIn;
 
 	private Display backward = Display.NULL;
+
+	private LinkRendering incoming1 = LinkRendering.none();
+	private LinkRendering incoming2 = LinkRendering.none();
+	private List<PositionedNote> backwardNotes = new ArrayList<>();
 	private Display test = Display.NULL;
 	private Display yes = Display.NULL;
 	private Display out = Display.NULL;
 	private final Display startLabel;
 	private boolean testCalled = false;
 	private LinkRendering endRepeatLinkRendering = LinkRendering.none();
-	private LinkRendering backRepeatLinkRendering = LinkRendering.none();
 
-	public InstructionRepeat(Swimlane swimlane, Instruction parent, LinkRendering nextLinkRenderer, HtmlColor color,
-			Display startLabel) {
+	private final Colors colors;
+
+	public boolean containsBreak() {
+		return repeatList.containsBreak();
+	}
+
+	public InstructionRepeat(Swimlane swimlane, Instruction parent, LinkRendering nextLinkRenderer, HColor color,
+			Display startLabel, BoxStyle boxStyleIn, Colors colors) {
+		this.repeatList = new InstructionList(swimlane);
+		this.boxStyleIn = boxStyleIn;
 		this.startLabel = startLabel;
 		this.parent = parent;
 		this.swimlane = swimlane;
-		this.nextLinkRenderer = nextLinkRenderer;
-		if (nextLinkRenderer == null) {
-			throw new IllegalArgumentException();
+		this.nextLinkRenderer = Objects.requireNonNull(nextLinkRenderer);
+		this.colors = colors;
+	}
+
+	private boolean isLastOfTheParent() {
+		if (parent instanceof InstructionList) {
+			return ((InstructionList) parent).getLast() == this;
 		}
-		this.color = color;
+		return false;
 	}
 
-	public void setBackward(Display label) {
+	public void setBackward(Display label, Swimlane swimlaneOut, BoxStyle boxStyle, LinkRendering incoming1,
+			LinkRendering incoming2) {
 		this.backward = label;
+		this.swimlaneOut = swimlaneOut;
+		this.boxStyle = boxStyle;
+		this.incoming1 = incoming1;
+		this.incoming2 = incoming2;
 	}
 
-	public void add(Instruction ins) {
-		repeatList.add(ins);
+	public boolean hasBackward() {
+		return this.backward != Display.NULL;
+	}
+
+	public CommandExecutionResult add(Instruction ins) {
+		return repeatList.add(ins);
 	}
 
 	public Ftile createFtile(FtileFactory factory) {
-		final Ftile back = Display.isNull(backward) ? null : factory.activity(backward, swimlane, BoxStyle.PLAIN,
-				Colors.empty());
-		final Ftile result = factory.repeat(swimlane, repeatList.getSwimlaneOut(),
-				startLabel, factory.decorateOut(repeatList.createFtile(factory), endRepeatLinkRendering), test, yes, out,
-				color, backRepeatLinkRendering, back);
+		final Ftile back = getBackward(factory);
+		final Ftile decorateOut = factory.decorateOut(repeatList.createFtile(factory), endRepeatLinkRendering);
+		final Ftile result = factory.repeat(boxStyleIn, swimlane, swimlaneOut, startLabel, decorateOut, test, yes, out,
+				colors, back, isLastOfTheParent(), incoming1, incoming2);
 		if (killed) {
 			return new FtileKilled(result);
 		}
 		return result;
 	}
 
+	private Ftile getBackward(FtileFactory factory) {
+		if (Display.isNull(backward)) {
+			return null;
+		}
+		Ftile result = factory.activity(backward, swimlaneOut, boxStyle, Colors.empty(), null);
+		if (backwardNotes.size() > 0) {
+			result = factory.addNote(result, swimlaneOut, backwardNotes);
+		}
+		return result;
+	}
+
 	public Instruction getParent() {
 		return parent;
 	}
 
 	public void setTest(Display test, Display yes, Display out, LinkRendering endRepeatLinkRendering,
-			LinkRendering backRepeatLinkRendering) {
-		this.test = test;
-		this.yes = yes;
-		this.out = out;
-		if (test == null) {
-			throw new IllegalArgumentException();
-		}
-		if (yes == null) {
-			throw new IllegalArgumentException();
-		}
-		if (out == null) {
-			throw new IllegalArgumentException();
-		}
+			LinkRendering back, Swimlane swimlaneOut) {
+		this.swimlaneOut = swimlaneOut;
+		this.test = Objects.requireNonNull(test);
+		this.yes = Objects.requireNonNull(yes);
+		this.out = Objects.requireNonNull(out);
 		this.endRepeatLinkRendering = endRepeatLinkRendering;
-		this.backRepeatLinkRendering = backRepeatLinkRendering;
+		if (back.isNone() == false)
+			this.incoming1 = back;
 		this.testCalled = true;
 	}
 
@@ -134,7 +167,12 @@ public class InstructionRepeat implements Instruction {
 	}
 
 	public boolean addNote(Display note, NotePosition position, NoteType type, Colors colors, Swimlane swimlaneNote) {
-		return repeatList.addNote(note, position, type, colors, swimlaneNote);
+		if (Display.isNull(backward)) {
+			return repeatList.addNote(note, position, type, colors, swimlaneNote);
+		}
+		this.backwardNotes.add(new PositionedNote(note, position, type, colors, swimlaneNote));
+		return true;
+
 	}
 
 	public Set<Swimlane> getSwimlanes() {
