@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -56,12 +56,14 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.FtileEmpty;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileFactory;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileGeometry;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileUtils;
+import net.sourceforge.plantuml.activitydiagram3.ftile.MergeStrategy;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Snake;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamond;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondInside;
 import net.sourceforge.plantuml.graphic.Rainbow;
 import net.sourceforge.plantuml.graphic.StringBounder;
+import net.sourceforge.plantuml.svek.ConditionEndStyle;
 import net.sourceforge.plantuml.ugraphic.UEmpty;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
@@ -72,6 +74,7 @@ public class FtileIfDown extends AbstractFtile {
 	private final Ftile diamond1;
 	private final Ftile diamond2;
 	private final Ftile optionalStop;
+	private final ConditionEndStyle conditionEndStyle;
 
 	@Override
 	public Collection<Ftile> getMyChildren() {
@@ -82,7 +85,7 @@ public class FtileIfDown extends AbstractFtile {
 	}
 
 	public Set<Swimlane> getSwimlanes() {
-		final Set<Swimlane> result = new HashSet<Swimlane>(thenBlock.getSwimlanes());
+		final Set<Swimlane> result = new HashSet<>(thenBlock.getSwimlanes());
 		result.add(getSwimlaneIn());
 		return result;
 	}
@@ -98,27 +101,35 @@ public class FtileIfDown extends AbstractFtile {
 		return thenBlock.getSwimlaneOut();
 	}
 
-	private FtileIfDown(Ftile thenBlock, Ftile diamond1, Ftile diamond2, Ftile optionalStop) {
+	private FtileIfDown(Ftile thenBlock, Ftile diamond1, Ftile diamond2, Ftile optionalStop,
+			ConditionEndStyle conditionEndStyle) {
 		super(thenBlock.skinParam());
 		this.thenBlock = thenBlock;
 		this.diamond1 = diamond1;
 		this.diamond2 = diamond2;
 		this.optionalStop = optionalStop;
+		this.conditionEndStyle = conditionEndStyle;
 	}
 
 	public static Ftile create(Ftile diamond1, Ftile diamond2, Swimlane swimlane, Ftile thenBlock, Rainbow arrowColor,
-			FtileFactory ftileFactory, Ftile optionalStop, Rainbow elseColor) {
+			ConditionEndStyle conditionEndStyle, FtileFactory ftileFactory, Ftile optionalStop, Rainbow elseColor) {
 
 		elseColor = elseColor.withDefault(arrowColor);
-		final FtileIfDown result = new FtileIfDown(thenBlock, diamond1, optionalStop == null ? diamond2
-				: new FtileEmpty(ftileFactory.skinParam()), optionalStop);
+		final FtileIfDown result = new FtileIfDown(thenBlock, diamond1,
+				optionalStop == null ? diamond2 : new FtileEmpty(ftileFactory.skinParam()), optionalStop,
+				conditionEndStyle);
 
-		final List<Connection> conns = new ArrayList<Connection>();
+		final List<Connection> conns = new ArrayList<>();
 		conns.add(result.new ConnectionIn(thenBlock.getInLinkRendering().getRainbow(arrowColor)));
 		final boolean hasPointOut1 = thenBlock.calculateDimension(ftileFactory.getStringBounder()).hasPointOut();
 		if (optionalStop == null) {
 			if (hasPointOut1) {
-				conns.add(result.new ConnectionElse(elseColor));
+				if (conditionEndStyle == ConditionEndStyle.DIAMOND) {
+					conns.add(result.new ConnectionElse(elseColor));
+				} else if (conditionEndStyle == ConditionEndStyle.HLINE) {
+					conns.add(result.new ConnectionElseHline(elseColor));
+					conns.add(result.new ConnectionHline(elseColor));
+				}
 			} else {
 				conns.add(result.new ConnectionElseNoDiamond(elseColor));
 			}
@@ -144,7 +155,7 @@ public class FtileIfDown extends AbstractFtile {
 			final Point2D p2 = getP2(stringBounder);
 			// p2 = new Point2D.Double(p2.getX(), p1.getY());
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToRight());
+			final Snake snake = Snake.create(color, Arrows.asToRight());
 			snake.addPoint(p1);
 			snake.addPoint(p2);
 			ug.draw(snake);
@@ -175,30 +186,31 @@ public class FtileIfDown extends AbstractFtile {
 		}
 
 		private Point2D getP1(final StringBounder stringBounder) {
-			return getTranslateDiamond1(stringBounder).getTranslated(
-					getFtile1().calculateDimension(stringBounder).getPointOut());
+			return getTranslateDiamond1(stringBounder)
+					.getTranslated(getFtile1().calculateDimension(stringBounder).getPointOut());
 		}
 
 		private Point2D getP2(final StringBounder stringBounder) {
-			return getTranslateForThen(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			return getTranslateForThen(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
 		}
 
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			snake.addPoint(getP1(stringBounder));
 			snake.addPoint(getP2(stringBounder));
 
 			ug.draw(snake);
 		}
 
+		@Override
 		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
 			final StringBounder stringBounder = ug.getStringBounder();
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			final Point2D mp1a = translate1.getTranslated(p1);
 			final Point2D mp2b = translate2.getTranslated(p2);
 			final double middle = (mp1a.getY() + mp2b.getY()) / 2.0;
@@ -219,13 +231,21 @@ public class FtileIfDown extends AbstractFtile {
 		}
 
 		private Point2D getP1(final StringBounder stringBounder) {
-			return getTranslateForThen(stringBounder).getTranslated(
-					getFtile1().calculateDimension(stringBounder).getPointOut());
+			return getTranslateForThen(stringBounder)
+					.getTranslated(getFtile1().calculateDimension(stringBounder).getPointOut());
 		}
 
 		private Point2D getP2(final StringBounder stringBounder) {
-			return getTranslateDiamond2(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			return getTranslateDiamond2(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
+		}
+
+		private Point2D getP2hline(final StringBounder stringBounder) {
+			final FtileGeometry dimDiamond2 = getFtile2().calculateDimension(stringBounder);
+			final double x = dimDiamond2.getWidth();
+			final double half = (dimDiamond2.getOutY() - dimDiamond2.getInY()) / 2;
+			return getTranslateDiamond2(stringBounder)
+					.getTranslated(new Point2D.Double(x, dimDiamond2.getInY() + half));
 		}
 
 		public void drawU(UGraphic ug) {
@@ -235,13 +255,19 @@ public class FtileIfDown extends AbstractFtile {
 				return;
 			}
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			snake.addPoint(getP1(stringBounder));
-			snake.addPoint(getP2(stringBounder));
+
+			if (conditionEndStyle == ConditionEndStyle.DIAMOND) {
+				snake.addPoint(getP2(stringBounder));
+			} else if (conditionEndStyle == ConditionEndStyle.HLINE) {
+				snake.addPoint(getP2hline(stringBounder));
+			}
 
 			ug.draw(snake);
 		}
 
+		@Override
 		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
 
 			if (getFtile1().calculateDimension(ug.getStringBounder()).hasPointOut() == false) {
@@ -251,7 +277,7 @@ public class FtileIfDown extends AbstractFtile {
 			final StringBounder stringBounder = ug.getStringBounder();
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			final Point2D mp1a = translate1.getTranslated(p1);
 			final Point2D mp2b = translate2.getTranslated(p2);
 			final double middle = (mp1a.getY() + mp2b.getY()) / 2.0;
@@ -271,6 +297,125 @@ public class FtileIfDown extends AbstractFtile {
 			this.endInlinkColor = endInlinkColor;
 		}
 
+		protected Point2D getP1(StringBounder stringBounder) {
+			final FtileGeometry dimDiamond1 = diamond1.calculateDimension(stringBounder);
+			final double x = dimDiamond1.getWidth();
+			final double half = (dimDiamond1.getOutY() - dimDiamond1.getInY()) / 2;
+			return getTranslateDiamond1(stringBounder)
+					.getTranslated(new Point2D.Double(x, dimDiamond1.getInY() + half));
+		}
+
+		protected Point2D getP2(final StringBounder stringBounder) {
+			final FtileGeometry dimDiamond2 = diamond2.calculateDimension(stringBounder);
+			final double x = dimDiamond2.getWidth();
+			final double half = (dimDiamond2.getOutY() - dimDiamond2.getInY()) / 2;
+			return getTranslateDiamond2(stringBounder)
+					.getTranslated(new Point2D.Double(x, dimDiamond2.getInY() + half));
+		}
+
+		public void drawU(UGraphic ug) {
+			final StringBounder stringBounder = ug.getStringBounder();
+
+			final Point2D p1 = getP1(stringBounder);
+			if (calculateDimension(stringBounder).hasPointOut() == false) {
+				return;
+			}
+			final Point2D p2 = getP2(stringBounder);
+
+			final double x1 = p1.getX();
+			final double y1 = p1.getY();
+			final double x2 = p2.getX();
+			final double y2 = p2.getY();
+
+			final FtileGeometry thenGeom = thenBlock.calculateDimension(stringBounder);
+			final double xmax = Math.max(x1 + Diamond.diamondHalfSize,
+					getTranslateForThen(stringBounder).getDx() + thenGeom.getWidth());
+
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToLeft()).emphasizeDirection(Direction.DOWN);
+			snake.addPoint(x1, y1);
+			snake.addPoint(xmax, y1);
+			snake.addPoint(xmax, y2);
+			snake.addPoint(x2, y2);
+			ug.apply(new UTranslate(x2, y2 - Diamond.diamondHalfSize)).draw(new UEmpty(5, Diamond.diamondHalfSize));
+			ug.draw(snake);
+
+		}
+
+	}
+
+	class ConnectionElseHline extends ConnectionElse {
+		private final Rainbow endInlinkColor;
+
+		public ConnectionElseHline(Rainbow endInlinkColor) {
+			super(endInlinkColor);
+			this.endInlinkColor = endInlinkColor;
+		}
+
+		@Override
+		public void drawU(UGraphic ug) {
+			final StringBounder stringBounder = ug.getStringBounder();
+
+			final Point2D p1 = getP1(stringBounder);
+			if (calculateDimension(stringBounder).hasPointOut() == false) {
+				return;
+			}
+			final Point2D p2 = getP2(stringBounder);
+
+			final double x1 = p1.getX();
+			final double y1 = p1.getY();
+			final double x2 = p2.getX();
+			final double y2 = p2.getY();
+
+			final FtileGeometry thenGeom = thenBlock.calculateDimension(stringBounder);
+			final double xmax = Math.max(x1 + Diamond.diamondHalfSize,
+					getTranslateForThen(stringBounder).getDx() + thenGeom.getWidth());
+
+			/*
+			 * if( conditionEndStyle == ConditionEndStyle.DIAMOND ) { final Snake snake =
+			 * new Snake(arrowHorizontalAlignment(), endInlinkColor, Arrows.asToLeft());
+			 * snake.addPoint(x1, y1); snake.addPoint(xmax, y1); snake.addPoint(xmax, y2);
+			 * snake.addPoint(x2, y2); snake.emphasizeDirection(Direction.DOWN);
+			 * ug.apply(new UTranslate(x2, y2 - Diamond.diamondHalfSize)).draw(new UEmpty(5,
+			 * Diamond.diamondHalfSize)); ug.draw(snake); }
+			 */
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToDown());
+			snake.addPoint(x1, y1);
+			snake.addPoint(xmax, y1);
+			snake.addPoint(xmax, y2);
+			ug.apply(new UTranslate(xmax, y2 - Diamond.diamondHalfSize)).draw(new UEmpty(5, Diamond.diamondHalfSize));
+			ug.draw(snake);
+			/*
+			 * final Snake snake2 = Snake.create(arrowHorizontalAlignment(),
+			 * endInlinkColor); snake2.addPoint(xmax, y2); snake2.addPoint(x2, y2);
+			 * ug.draw(snake2);
+			 */
+
+		}
+
+	}
+
+	class ConnectionElseNoDiamond extends ConnectionElse {
+
+		public ConnectionElseNoDiamond(Rainbow endInlinkColor) {
+			super(endInlinkColor);
+		}
+
+		@Override
+		protected Point2D getP2(final StringBounder stringBounder) {
+			return calculateDimension(stringBounder).getPointOut();
+		}
+
+	}
+
+	// copied from FtileIfLongHorizontal to use with ConditionEndStyle.HLINE
+	class ConnectionHline extends AbstractConnection {
+		private final Rainbow endInlinkColor;
+
+		public ConnectionHline(Rainbow endInlinkColor) {
+			super(diamond1, diamond2);
+			this.endInlinkColor = endInlinkColor;
+		}
+
 		private Point2D getP1(StringBounder stringBounder) {
 			final FtileGeometry dimDiamond1 = diamond1.calculateDimension(stringBounder);
 			final double x = dimDiamond1.getWidth();
@@ -287,47 +432,45 @@ public class FtileIfDown extends AbstractFtile {
 					.getTranslated(new Point2D.Double(x, dimDiamond2.getInY() + half));
 		}
 
+		// the bottom or south point of the diamond that we omitted
+		protected Point2D getP3(final StringBounder stringBounder) {
+			final FtileGeometry dimDiamond2 = diamond2.calculateDimension(stringBounder);
+			final double x = dimDiamond2.getWidth();
+			return getTranslateDiamond2(stringBounder).getTranslated(new Point2D.Double(x, dimDiamond2.getOutY()));
+		}
+
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), endInlinkColor, Arrows.asToLeft());
 			final Point2D p1 = getP1(stringBounder);
+			if (calculateDimension(stringBounder).hasPointOut() == false) {
+				return;
+			}
 			final Point2D p2 = getP2(stringBounder);
+			final Point2D p3 = getP3(stringBounder);
 
 			final double x1 = p1.getX();
 			final double y1 = p1.getY();
 			final double x2 = p2.getX();
 			final double y2 = p2.getY();
 
-			snake.addPoint(x1, y1);
+			final double x3 = p3.getX();
+			final double y3 = p3.getY();
 
 			final FtileGeometry thenGeom = thenBlock.calculateDimension(stringBounder);
-			final double xmax = Math.max(x1 + Diamond.diamondHalfSize, getTranslateForThen(stringBounder).getDx()
-					+ thenGeom.getWidth());
+			final double xmax = Math.max(x1 + Diamond.diamondHalfSize,
+					getTranslateForThen(stringBounder).getDx() + thenGeom.getWidth());
 
-			snake.addPoint(xmax, y1);
+			final Snake snake = Snake.create(endInlinkColor).withMerge(MergeStrategy.NONE);
 			snake.addPoint(xmax, y2);
+			// ug.apply(new UTranslate(xmax, y2 - Diamond.diamondHalfSize)).draw(new
+			// UEmpty(5,
+			// Diamond.diamondHalfSize));
 			snake.addPoint(x2, y2);
-			snake.emphasizeDirection(Direction.DOWN);
-			ug.apply(new UTranslate(x2, y2 - Diamond.diamondHalfSize)).draw(new UEmpty(5, Diamond.diamondHalfSize));
-
+			snake.addPoint(x3, y3);
 			ug.draw(snake);
 
 		}
-
-	}
-
-	class ConnectionElseNoDiamond extends ConnectionElse {
-
-		public ConnectionElseNoDiamond(Rainbow endInlinkColor) {
-			super(endInlinkColor);
-		}
-
-		@Override
-		protected Point2D getP2(final StringBounder stringBounder) {
-			return calculateDimension(stringBounder).getPointOut();
-		}
-
 	}
 
 	public void drawU(UGraphic ug) {
@@ -341,7 +484,8 @@ public class FtileIfDown extends AbstractFtile {
 		}
 	}
 
-	public FtileGeometry calculateDimension(StringBounder stringBounder) {
+	@Override
+	protected FtileGeometry calculateDimensionFtile(StringBounder stringBounder) {
 		final FtileGeometry geoDiamond1 = diamond1.calculateDimension(stringBounder);
 		final FtileGeometry geoThen = thenBlock.calculateDimension(stringBounder);
 		final FtileGeometry geoDiamond2 = diamond2.calculateDimension(stringBounder);
@@ -352,7 +496,11 @@ public class FtileIfDown extends AbstractFtile {
 		if (optionalStop != null) {
 			width += optionalStop.calculateDimension(stringBounder).getWidth() + getAdditionalWidth(stringBounder);
 		}
-		return new FtileGeometry(width, height, geo.getLeft(), geoDiamond1.getInY(), height);
+		final FtileGeometry result = new FtileGeometry(width, height, geo.getLeft(), geoDiamond1.getInY(), height);
+		if (geoThen.hasPointOut() == false && optionalStop != null) {
+			return result.withoutPointOut();
+		}
+		return result;
 
 	}
 
