@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -42,6 +42,7 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Objects;
 import java.util.Set;
 
 import net.sourceforge.plantuml.Dimension2DDouble;
@@ -56,23 +57,22 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.Ftile;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileAssemblySimple;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileFactory;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileGeometry;
-import net.sourceforge.plantuml.activitydiagram3.ftile.FtileMinWidth;
+import net.sourceforge.plantuml.activitydiagram3.ftile.FtileMinWidthCentered;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileUtils;
 import net.sourceforge.plantuml.activitydiagram3.ftile.MergeStrategy;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Snake;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
-import net.sourceforge.plantuml.activitydiagram3.ftile.vcompact.cond.FtileIfWithLinks;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondInside2;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.Rainbow;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.svek.ConditionStyle;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 class FtileIfLongHorizontal extends AbstractFtile {
 
@@ -81,7 +81,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 	private final List<Ftile> tiles;
 	private final Ftile tile2;
 	private final List<Ftile> diamonds;
-	private final List<Ftile> couples = new ArrayList<Ftile>();
+	private final List<Ftile> couples = new ArrayList<>();
 
 	private final Rainbow arrowColor;
 
@@ -97,8 +97,8 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			couples.add(FtileUtils.addHorizontalMargin(tmp, inlabelSizes.get(i), 0));
 		}
 		this.tile2 = tile2;
-		this.diamonds = new ArrayList<Ftile>(diamonds);
-		this.tiles = new ArrayList<Ftile>(tiles);
+		this.diamonds = new ArrayList<>(diamonds);
+		this.tiles = new ArrayList<>(tiles);
 
 		this.arrowColor = arrowColor;
 
@@ -109,7 +109,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 		for (Ftile diamond : diamonds) {
 			maxOutY = Math.max(maxOutY, diamond.calculateDimension(stringBounder).getOutY());
 		}
-		final List<Ftile> result = new ArrayList<Ftile>();
+		final List<Ftile> result = new ArrayList<>();
 		for (int i = 0; i < diamonds.size(); i++) {
 			Ftile diamond = diamonds.get(i);
 			final double missing = maxOutY - diamond.calculateDimension(stringBounder).getOutY();
@@ -121,7 +121,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 	}
 
 	public Set<Swimlane> getSwimlanes() {
-		final Set<Swimlane> result = new HashSet<Swimlane>();
+		final Set<Swimlane> result = new HashSet<>();
 		if (getSwimlaneIn() != null) {
 			result.add(getSwimlaneIn());
 		}
@@ -140,28 +140,28 @@ class FtileIfLongHorizontal extends AbstractFtile {
 		return getSwimlaneIn();
 	}
 
-	static Ftile create(Swimlane swimlane, HtmlColor borderColor, HtmlColor backColor, Rainbow arrowColor,
+	static Ftile create(Swimlane swimlane, HColor borderColor, HColor backColor, Rainbow arrowColor,
 			FtileFactory ftileFactory, ConditionStyle conditionStyle, List<Branch> thens, Branch branch2,
-			FontConfiguration fc, LinkRendering topInlinkRendering, LinkRendering afterEndwhile) {
-		if (afterEndwhile == null) {
-			throw new IllegalArgumentException();
-		}
-		final List<Ftile> tiles = new ArrayList<Ftile>();
+			FontConfiguration fcArrow, LinkRendering topInlinkRendering, LinkRendering afterEndwhile,
+			FontConfiguration fcTest) {
+		Objects.requireNonNull(afterEndwhile);
+		final List<Ftile> tiles = new ArrayList<>();
 
 		for (Branch branch : thens) {
-			tiles.add(new FtileMinWidth(branch.getFtile(), 30));
+			tiles.add(new FtileMinWidthCentered(branch.getFtile(), 30));
 		}
 
-		final Ftile tile2 = new FtileMinWidth(branch2.getFtile(), 30);
+		final Ftile tile2 = new FtileMinWidthCentered(branch2.getFtile(), 30);
 
-		List<Ftile> diamonds = new ArrayList<Ftile>();
-		List<Double> inlabelSizes = new ArrayList<Double>();
+		List<Ftile> diamonds = new ArrayList<>();
+		List<Double> inlabelSizes = new ArrayList<>();
 		for (Branch branch : thens) {
-			final TextBlock tb1 = branch.getLabelPositive().create(fc, HorizontalAlignment.LEFT,
+			final TextBlock tb1 = branch.getLabelPositive().create(fcArrow, HorizontalAlignment.LEFT,
 					ftileFactory.skinParam());
-			final TextBlock tbTest = branch.getLabelTest().create(fc, HorizontalAlignment.LEFT,
+			final TextBlock tbTest = branch.getLabelTest().create(fcTest,
+					ftileFactory.skinParam().getDefaultTextAlignment(HorizontalAlignment.LEFT),
 					ftileFactory.skinParam());
-			final HtmlColor diamondColor = branch.getColor() == null ? backColor : branch.getColor();
+			final HColor diamondColor = branch.getColor() == null ? backColor : branch.getColor();
 
 			FtileDiamondInside2 diamond = new FtileDiamondInside2(branch.skinParam(), diamondColor, borderColor,
 					swimlane, tbTest);
@@ -169,7 +169,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			if (Display.isNull(branch.getInlabel())) {
 				inlabelSizes.add(0.0);
 			} else {
-				tbInlabel = branch.getInlabel().create(fc, HorizontalAlignment.LEFT, ftileFactory.skinParam());
+				tbInlabel = branch.getInlabel().create(fcArrow, HorizontalAlignment.LEFT, ftileFactory.skinParam());
 				inlabelSizes.add(tbInlabel.calculateDimension(ftileFactory.getStringBounder()).getWidth());
 				diamond = diamond.withWest(tbInlabel);
 			}
@@ -177,38 +177,69 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			diamonds.add(diamond);
 		}
 
-		final TextBlock tb2 = branch2.getLabelPositive().create(fc, HorizontalAlignment.LEFT, ftileFactory.skinParam());
+		final TextBlock tb2 = branch2.getLabelPositive().create(fcArrow, HorizontalAlignment.LEFT,
+				ftileFactory.skinParam());
 		final int last = diamonds.size() - 1;
 		diamonds.set(last, ((FtileDiamondInside2) diamonds.get(last)).withEast(tb2));
 
 		diamonds = alignDiamonds(diamonds, ftileFactory.getStringBounder());
 
-		final FtileIfLongHorizontal result = new FtileIfLongHorizontal(diamonds, inlabelSizes, tiles, tile2, arrowColor);
-		final List<Connection> conns = new ArrayList<Connection>();
+		final FtileIfLongHorizontal result = new FtileIfLongHorizontal(diamonds, inlabelSizes, tiles, tile2,
+				arrowColor);
+		final List<Connection> conns = new ArrayList<>();
+
+		int nbOut = 0;
 
 		for (int i = 0; i < thens.size(); i++) {
 			final Ftile ftile = tiles.get(i);
 			final Ftile diam = diamonds.get(i);
 
-			final Rainbow rainbowIn = FtileIfWithLinks.getInColor(thens.get(i), arrowColor);
+			final Rainbow rainbowIn = thens.get(i).getInColor(arrowColor);
 			final Branch branch = thens.get(i);
-			final Rainbow rainbowOut = branch.getInlinkRenderingColorAndStyle();
-			conns.add(result.new ConnectionVerticalIn(diam, ftile, rainbowIn.size() == 0 ? arrowColor : rainbowIn));
-			conns.add(result.new ConnectionVerticalOut(ftile, rainbowOut.size() == 0 ? arrowColor : rainbowOut));
+
+			if (branch.getFtile().calculateDimension(ftileFactory.getStringBounder()).hasPointOut()) {
+				nbOut++;
+			}
+			final Rainbow rainbowOut = branch.getOut();
+			TextBlock out2 = null;
+			if (branch.getSpecial() != null) {
+				out2 = branch.getSpecial().getDisplay().create(fcTest, HorizontalAlignment.LEFT,
+						ftileFactory.skinParam());
+			}
+			final Rainbow rainbow1 = rainbowIn.size() == 0 ? arrowColor : rainbowIn;
+			final Rainbow rainbow2 = rainbowOut.size() == 0 ? arrowColor : rainbowOut;
+
+			conns.add(result.new ConnectionVerticalIn(diam, ftile, rainbow1));
+			conns.add(result.new ConnectionVerticalOut(ftile, rainbow2, out2));
 		}
 
 		final Rainbow topInColor = topInlinkRendering.getRainbow(arrowColor);
 		for (int i = 0; i < diamonds.size() - 1; i++) {
 			final Ftile diam1 = diamonds.get(i);
 			final Ftile diam2 = diamonds.get(i + 1);
-			conns.add(result.new ConnectionHorizontal(diam1, diam2, arrowColor));
+
+			final Branch branch = thens.get(i + 1);
+
+			conns.add(result.new ConnectionHorizontal(diam1, diam2, branch.getInRainbow(arrowColor)));
 		}
 		conns.add(result.new ConnectionIn(topInColor));
-		conns.add(result.new ConnectionLastElseIn(FtileIfWithLinks.getInColor(branch2, arrowColor)));
-		conns.add(result.new ConnectionLastElseOut(arrowColor));
-		final Rainbow horizontalOutColor = afterEndwhile.getRainbow(arrowColor);
-		conns.add(result.new ConnectionHline(horizontalOutColor));
-		// conns.add(result.new ConnectionHline(HtmlColorUtils.BLUE));
+		TextBlock out2 = null;
+		if (branch2.getSpecial() != null) {
+			out2 = branch2.getSpecial().getDisplay().create(fcTest, HorizontalAlignment.LEFT, ftileFactory.skinParam());
+		}
+
+		final Rainbow rainbowOut = branch2.getOut();
+		final Rainbow rainbowIn = branch2.getInColor(arrowColor);
+		final Rainbow rainbow1 = rainbowIn.size() == 0 ? arrowColor : rainbowIn;
+		final Rainbow rainbow2 = rainbowOut.size() == 0 ? arrowColor : rainbowOut;
+
+		conns.add(result.new ConnectionLastElseIn(rainbow1));
+		conns.add(result.new ConnectionLastElseOut(rainbow2, out2, nbOut));
+		final boolean horizontalOut = nbOut > 0;
+		if (horizontalOut) {
+			final Rainbow horizontalOutColor = afterEndwhile.getRainbow(arrowColor);
+			conns.add(result.new ConnectionHline(horizontalOutColor));
+		}
 
 		return FtileUtils.addConnection(result, conns);
 	}
@@ -227,7 +258,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToRight());
+			final Snake snake = Snake.create(color, Arrows.asToRight());
 			snake.addPoint(p1);
 			snake.addPoint(p2);
 			ug.draw(snake);
@@ -235,8 +266,8 @@ class FtileIfLongHorizontal extends AbstractFtile {
 
 		private Point2D getP1(StringBounder stringBounder) {
 			final FtileGeometry dimDiamond1 = getFtile1().calculateDimension(stringBounder);
-			final Point2D p = new Point2D.Double(dimDiamond1.getLeft() * 2, getYdiamontOutToLeft(dimDiamond1,
-					stringBounder));
+			final Point2D p = new Point2D.Double(dimDiamond1.getLeft() * 2,
+					getYdiamontOutToLeft(dimDiamond1, stringBounder));
 
 			return getTranslateDiamond1(getFtile1(), stringBounder).getTranslated(p);
 		}
@@ -265,7 +296,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final UTranslate tr = getTranslateDiamond1(getFtile2(), ug.getStringBounder());
 			final Point2D p2 = tr.getTranslated(getFtile2().calculateDimension(ug.getStringBounder()).getPointIn());
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			final Point2D p1 = calculateDimensionInternal(ug.getStringBounder()).getPointIn();
 
 			snake.addPoint(p1);
@@ -289,7 +320,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			final Point2D p1 = getP1(ug.getStringBounder());
 			final UTranslate tr2 = getTranslate2(ug.getStringBounder());
 			final Point2D p2 = tr2.getTranslated(getFtile2().calculateDimension(ug.getStringBounder()).getPointIn());
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			snake.addPoint(p1);
 			snake.addPoint(p2.getX(), p1.getY());
 			snake.addPoint(p2);
@@ -298,8 +329,8 @@ class FtileIfLongHorizontal extends AbstractFtile {
 
 		private Point2D getP1(StringBounder stringBounder) {
 			final FtileGeometry dimDiamond1 = getFtile1().calculateDimension(stringBounder);
-			final Point2D p = new Point2D.Double(dimDiamond1.getLeft() * 2, getYdiamontOutToLeft(dimDiamond1,
-					stringBounder));
+			final Point2D p = new Point2D.Double(dimDiamond1.getLeft() * 2,
+					getYdiamontOutToLeft(dimDiamond1, stringBounder));
 			return getTranslateDiamond1(getFtile1(), stringBounder).getTranslated(p);
 		}
 
@@ -308,10 +339,14 @@ class FtileIfLongHorizontal extends AbstractFtile {
 	class ConnectionLastElseOut extends AbstractConnection {
 
 		private final Rainbow arrowColor;
+		private final TextBlock out2;
+		private final int nbOut;
 
-		public ConnectionLastElseOut(Rainbow arrowColor) {
+		public ConnectionLastElseOut(Rainbow arrowColor, TextBlock out2, int nbOut) {
 			super(tile2, null);
 			this.arrowColor = arrowColor;
+			this.out2 = out2;
+			this.nbOut = nbOut;
 		}
 
 		public void drawU(UGraphic ug) {
@@ -322,12 +357,16 @@ class FtileIfLongHorizontal extends AbstractFtile {
 				return;
 			}
 			final Point2D p1 = tr1.getTranslated(dim.getPointOut());
-			final double totalHeight = calculateDimensionInternal(stringBounder).getHeight();
+			final FtileGeometry full = calculateDimensionInternal(stringBounder);
+			final double totalHeight = full.getHeight();
 			final Point2D p2 = new Point2D.Double(p1.getX(), totalHeight);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown()).withLabel(out2, arrowHorizontalAlignment());
 			snake.addPoint(p1);
 			snake.addPoint(p2);
+			if (nbOut == 0) {
+				snake.addPoint(new Point2D.Double(full.getLeft(), totalHeight));
+			}
 			ug.draw(snake);
 		}
 
@@ -347,7 +386,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToDown());
+			final Snake snake = Snake.create(color, Arrows.asToDown());
 			snake.addPoint(p1);
 			snake.addPoint(p2);
 			ug.draw(snake);
@@ -363,11 +402,12 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			return getTranslate1(getFtile2(), stringBounder).getTranslated(p);
 		}
 
+		@Override
 		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
 			final Point2D p1 = getP1(ug.getStringBounder());
 			final Point2D p2 = getP2(ug.getStringBounder());
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToDown());
+			final Snake snake = Snake.create(color, Arrows.asToDown());
 
 			final Point2D mp1a = translate1.getTranslated(p1);
 			final Point2D mp2b = translate2.getTranslated(p2);
@@ -384,10 +424,12 @@ class FtileIfLongHorizontal extends AbstractFtile {
 	class ConnectionVerticalOut extends AbstractConnection {
 
 		private final Rainbow color;
+		private final TextBlock out2;
 
-		public ConnectionVerticalOut(Ftile tile, Rainbow color) {
+		public ConnectionVerticalOut(Ftile tile, Rainbow color, TextBlock out2) {
 			super(tile, null);
 			this.color = color;
+			this.out2 = out2;
 		}
 
 		public void drawU(UGraphic ug) {
@@ -399,7 +441,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			}
 			final Point2D p2 = new Point2D.Double(p1.getX(), totalHeight);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToDown());
+			final Snake snake = Snake.create(color, Arrows.asToDown()).withLabel(out2, arrowHorizontalAlignment());
 			snake.addPoint(p1);
 			snake.addPoint(p2);
 			ug.draw(snake);
@@ -429,55 +471,130 @@ class FtileIfLongHorizontal extends AbstractFtile {
 			final StringBounder stringBounder = ug.getStringBounder();
 			final Dimension2D totalDim = calculateDimensionInternal(stringBounder);
 
-			final Swimlane intoSw;
+			final List<Ftile> allTiles = new ArrayList<>(couples);
+			allTiles.add(tile2);
+
+			final double[] minmax;
 			if (ug instanceof UGraphicInterceptorOneSwimlane) {
-				intoSw = ((UGraphicInterceptorOneSwimlane) ug).getSwimlane();
+				final UGraphicInterceptorOneSwimlane interceptor = (UGraphicInterceptorOneSwimlane) ug;
+				final List<Swimlane> allSwimlanes = interceptor.getOrderedListOfAllSwimlanes();
+				minmax = getMinmax(stringBounder, totalDim.getWidth(), allTiles, interceptor.getSwimlane(),
+						allSwimlanes);
 			} else {
-				intoSw = null;
+				minmax = getMinmaxSimple(stringBounder, totalDim.getWidth(), allTiles);
 			}
 
-			final List<Ftile> all = new ArrayList<Ftile>(couples);
-			all.add(tile2);
-			double minX = totalDim.getWidth() / 2;
-			double maxX = totalDim.getWidth() / 2;
-			boolean atLeastOne = false;
-			for (Ftile tmp : all) {
+			final double minX = minmax[0];
+			final double maxX = minmax[1];
+			if (Double.isNaN(minX) || Double.isNaN(maxX)) {
+				return;
+			}
+
+			final Snake s = Snake.create(arrowColor).withMerge(MergeStrategy.NONE);
+			s.addPoint(minX, totalDim.getHeight());
+			s.addPoint(maxX, totalDim.getHeight());
+			ug.draw(s);
+		}
+
+		private Double getLeftOut(final StringBounder stringBounder) {
+			final FtileGeometry dim = calculateDimension(stringBounder);
+			if (dim.hasPointOut()) {
+				return dim.getLeft();
+			}
+			return null;
+		}
+
+		private double[] getMinmax(StringBounder stringBounder, double width, List<Ftile> allTiles, Swimlane intoSw,
+				List<Swimlane> allSwimlanes) {
+			final int current = allSwimlanes.indexOf(intoSw);
+			final Double leftOut = getLeftOut(stringBounder);
+			if (leftOut == null)
+				return new double[] { Double.NaN, Double.NaN };
+
+			if (current == -1) {
+				throw new IllegalStateException();
+			}
+			final int first = getFirstSwimlane(stringBounder, allTiles, allSwimlanes);
+			final int last = getLastSwimlane(stringBounder, allTiles, allSwimlanes);
+			if (current < first || current > last)
+				return new double[] { Double.NaN, Double.NaN };
+			double minX = current != first ? 0 : width;
+			double maxX = current != last ? width : 0;
+			minX = Math.min(minX, leftOut);
+			maxX = Math.max(maxX, leftOut);
+			for (Ftile tmp : allTiles) {
 				if (tmp.calculateDimension(stringBounder).hasPointOut() == false) {
 					continue;
 				}
-				if (intoSw != null && tmp.getSwimlanes().contains(intoSw) == false) {
+				if (ftileDoesOutcomeInThatSwimlane(tmp, intoSw) == false) {
 					continue;
 				}
-				if (intoSw != null && tmp.getSwimlaneOut() != intoSw) {
-					continue;
-				}
-				atLeastOne = true;
 				final UTranslate ut = getTranslateFor(tmp, stringBounder);
 				final double out = tmp.calculateDimension(stringBounder).translate(ut).getLeft();
 				minX = Math.min(minX, out);
 				maxX = Math.max(maxX, out);
 			}
-			if (atLeastOne == false) {
-				return;
-			}
-
-			final Snake s = new Snake(arrowHorizontalAlignment(), arrowColor);
-			s.goUnmergeable(MergeStrategy.NONE);
-			final double height = totalDim.getHeight();
-			s.addPoint(minX, height);
-			s.addPoint(maxX, height);
-			ug.draw(s);
+			return new double[] { minX, maxX };
 		}
+
+		private double[] getMinmaxSimple(StringBounder stringBounder, double width, List<Ftile> allTiles) {
+			final Double leftOut = getLeftOut(stringBounder);
+			if (leftOut == null)
+				return new double[] { Double.NaN, Double.NaN };
+			double minX = width / 2;
+			double maxX = width / 2;
+			minX = Math.min(minX, leftOut);
+			maxX = Math.max(maxX, leftOut);
+			for (Ftile tmp : allTiles) {
+				if (tmp.calculateDimension(stringBounder).hasPointOut() == false) {
+					continue;
+				}
+				final UTranslate ut = getTranslateFor(tmp, stringBounder);
+				final double out = tmp.calculateDimension(stringBounder).translate(ut).getLeft();
+				minX = Math.min(minX, out);
+				maxX = Math.max(maxX, out);
+			}
+			return new double[] { minX, maxX };
+		}
+
+		private int getFirstSwimlane(StringBounder stringBounder, List<Ftile> allTiles, List<Swimlane> allSwimlanes) {
+			for (int i = 0; i < allSwimlanes.size(); i++) {
+				if (atLeastOne(stringBounder, allSwimlanes.get(i), allTiles)) {
+					return i;
+				}
+			}
+			throw new IllegalStateException();
+		}
+
+		private int getLastSwimlane(StringBounder stringBounder, List<Ftile> allTiles, List<Swimlane> allSwimlanes) {
+			for (int i = allSwimlanes.size() - 1; i >= 0; i--) {
+				if (atLeastOne(stringBounder, allSwimlanes.get(i), allTiles)) {
+					return i;
+				}
+			}
+			throw new IllegalStateException();
+		}
+
+		private boolean atLeastOne(StringBounder stringBounder, Swimlane intoSw, List<Ftile> allTiles) {
+			for (Ftile tmp : allTiles)
+				if (tmp.calculateDimension(stringBounder).hasPointOut() && ftileDoesOutcomeInThatSwimlane(tmp, intoSw))
+					return true;
+			return false;
+		}
+
+		private boolean ftileDoesOutcomeInThatSwimlane(Ftile ftile, Swimlane swimlane) {
+			return ftile.getSwimlaneOut() == swimlane && ftile.getSwimlanes().contains(swimlane);
+		}
+
 	}
 
 	@Override
 	public Collection<Ftile> getMyChildren() {
-		final List<Ftile> result = new ArrayList<Ftile>(tiles);
+		final List<Ftile> result = new ArrayList<>(tiles);
 		result.add(tile2);
 		return Collections.unmodifiableList(result);
 	}
 
-
 	@Override
 	public UTranslate getTranslateFor(Ftile child, StringBounder stringBounder) {
 		if (child == tile2) {
@@ -515,7 +632,7 @@ class FtileIfLongHorizontal extends AbstractFtile {
 		return trCouple.compose(in);
 	}
 
-	public UTranslate getTranslate1(Ftile tile, StringBounder stringBounder) {
+	private UTranslate getTranslate1(Ftile tile, StringBounder stringBounder) {
 		final int idx = tiles.indexOf(tile);
 		if (idx == -1) {
 			throw new IllegalArgumentException();
@@ -553,18 +670,27 @@ class FtileIfLongHorizontal extends AbstractFtile {
 		for (Ftile couple : couples) {
 			result = Dimension2DDouble.mergeLR(result, couple.calculateDimension(stringBounder));
 		}
-		final FtileGeometry dimTile2 = tile2.calculateDimension(stringBounder);
+		Dimension2D dimTile2 = tile2.calculateDimension(stringBounder);
+		dimTile2 = Dimension2DDouble.delta(dimTile2, 0, getDiamondsHeight(stringBounder) / 2);
 		result = Dimension2DDouble.mergeLR(result, dimTile2);
 		result = Dimension2DDouble.delta(result, xSeparation * couples.size(), 100);
 
 		return new FtileGeometry(result, result.getWidth() / 2, 0);
 	}
 
+	private double getDiamondsHeight(StringBounder stringBounder) {
+		double height = 0;
+		for (Ftile diamond : diamonds) {
+			height = Math.max(height, diamond.calculateDimension(stringBounder).getHeight());
+		}
+		return height;
+	}
+
 	@Override
 	protected FtileGeometry calculateDimensionFtile(StringBounder stringBounder) {
 		final Dimension2D dimTotal = calculateDimensionInternal(stringBounder);
 
-		final List<Ftile> all = new ArrayList<Ftile>(tiles);
+		final List<Ftile> all = new ArrayList<>(tiles);
 		all.add(tile2);
 		for (Ftile tmp : all) {
 			if (tmp.calculateDimension(stringBounder).hasPointOut()) {
