@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -38,6 +38,7 @@ package net.sourceforge.plantuml.activitydiagram3.ftile.vcompact;
 import java.awt.geom.Dimension2D;
 import java.awt.geom.Point2D;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
@@ -52,23 +53,22 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.Connection;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Ftile;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileFactory;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileGeometry;
-import net.sourceforge.plantuml.activitydiagram3.ftile.FtileMinWidth;
+import net.sourceforge.plantuml.activitydiagram3.ftile.FtileMinWidthCentered;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileOverpassing;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileUtils;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Snake;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
-import net.sourceforge.plantuml.activitydiagram3.ftile.vcompact.cond.FtileIfWithLinks;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamond;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondInside3;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.Rainbow;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.svek.ConditionStyle;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 class FtileIfLongVertical extends AbstractFtile {
 
@@ -80,10 +80,16 @@ class FtileIfLongVertical extends AbstractFtile {
 	private final Ftile tile2;
 	private final List<Ftile> diamonds;
 	private final Ftile lastDiamond;
-	// private final List<Ftile> couples = new ArrayList<Ftile>();
 
 	private final Rainbow arrowColor;
 
+	@Override
+	public Collection<Ftile> getMyChildren() {
+		final List<Ftile> result = new ArrayList<>(tiles);
+		result.add(tile2);
+		return Collections.unmodifiableList(result);
+	}
+
 	private FtileIfLongVertical(List<Ftile> diamonds, List<Ftile> tiles, Ftile tile2, Rainbow arrowColor,
 			Ftile lastDiamond) {
 		super(tiles.get(0).skinParam());
@@ -91,35 +97,16 @@ class FtileIfLongVertical extends AbstractFtile {
 			throw new IllegalArgumentException();
 		}
 		this.lastDiamond = lastDiamond;
-		// for (int i = 0; i < diamonds.size(); i++) {
-		// couples.add(new FtileAssemblySimple(diamonds.get(i), tiles.get(i)));
-		// }
 		this.tile2 = tile2;
-		this.diamonds = new ArrayList<Ftile>(diamonds);
-		this.tiles = new ArrayList<Ftile>(tiles);
+		this.diamonds = new ArrayList<>(diamonds);
+		this.tiles = new ArrayList<>(tiles);
 
 		this.arrowColor = arrowColor;
 
 	}
 
-	// private static List<Ftile> alignDiamonds(List<Ftile> diamonds, StringBounder stringBounder) {
-	// double maxOutY = 0;
-	// for (Ftile diamond : diamonds) {
-	// maxOutY = Math.max(maxOutY, diamond.calculateDimension(stringBounder).getOutY());
-	// }
-	// final List<Ftile> result = new ArrayList<Ftile>();
-	// for (int i = 0; i < diamonds.size(); i++) {
-	// Ftile diamond = diamonds.get(i);
-	// final double missing = maxOutY - diamond.calculateDimension(stringBounder).getOutY();
-	// assert missing >= 0;
-	// diamond = FtileUtils.addVerticalMargin(diamond, missing / 2, 20);
-	// result.add(diamond);
-	// }
-	// return result;
-	// }
-
 	public Set<Swimlane> getSwimlanes() {
-		final Set<Swimlane> result = new HashSet<Swimlane>();
+		final Set<Swimlane> result = new HashSet<>();
 		if (getSwimlaneIn() != null) {
 			result.add(getSwimlaneIn());
 		}
@@ -138,21 +125,24 @@ class FtileIfLongVertical extends AbstractFtile {
 		return getSwimlaneIn();
 	}
 
-	static Ftile create(Swimlane swimlane, HtmlColor borderColor, HtmlColor backColor, Rainbow arrowColor,
+	static Ftile create(Swimlane swimlane, HColor borderColor, HColor backColor, Rainbow arrowColor,
 			FtileFactory ftileFactory, ConditionStyle conditionStyle, List<Branch> thens, Branch branch2,
 			FontConfiguration fc, LinkRendering topInlinkRendering, LinkRendering afterEndwhile) {
-		final List<Ftile> tiles = new ArrayList<Ftile>();
+		final List<Ftile> tiles = new ArrayList<>();
 
 		for (Branch branch : thens) {
-			tiles.add(new FtileMinWidth(branch.getFtile(), 30));
+			tiles.add(new FtileMinWidthCentered(branch.getFtile(), 30));
 		}
 
-		final Ftile tile2 = new FtileMinWidth(branch2.getFtile(), 30);
+		final Ftile tile2 = new FtileMinWidthCentered(branch2.getFtile(), 30);
 
-		List<Ftile> diamonds = new ArrayList<Ftile>();
+		List<Ftile> diamonds = new ArrayList<>();
 		for (Branch branch : thens) {
-			final TextBlock tb1 = branch.getLabelPositive().create(fc, HorizontalAlignment.LEFT, ftileFactory.skinParam());
-			final TextBlock tbTest = branch.getLabelTest().create(fc, HorizontalAlignment.LEFT, ftileFactory.skinParam());
+			final TextBlock tb1 = branch.getLabelPositive().create(fc, HorizontalAlignment.LEFT,
+					ftileFactory.skinParam());
+			final TextBlock tbTest = branch.getLabelTest().create(fc,
+					ftileFactory.skinParam().getDefaultTextAlignment(HorizontalAlignment.LEFT),
+					ftileFactory.skinParam());
 			FtileDiamondInside3 diamond = new FtileDiamondInside3(branch.skinParam(), backColor, borderColor, swimlane,
 					tbTest);
 			diamond = diamond.withEast(tb1);
@@ -169,12 +159,12 @@ class FtileIfLongVertical extends AbstractFtile {
 
 		final FtileIfLongVertical result = new FtileIfLongVertical(diamonds, tiles, tile2, arrowColor, lastDiamond);
 
-		final List<Connection> conns = new ArrayList<Connection>();
+		final List<Connection> conns = new ArrayList<>();
 		for (int i = 0; i < thens.size(); i++) {
 			final Ftile ftile = tiles.get(i);
 			final Ftile diam = diamonds.get(i);
 
-			final Rainbow color = FtileIfWithLinks.getInColor(thens.get(i), arrowColor);
+			final Rainbow color = thens.get(i).getInColor(arrowColor);
 			conns.add(result.new ConnectionVerticalIn(diam, ftile, color == null ? arrowColor : color));
 			// conns.add(result.new ConnectionVerticalOut(ftile, arrowColor));
 		}
@@ -188,26 +178,13 @@ class FtileIfLongVertical extends AbstractFtile {
 		}
 
 		final Rainbow topInColor = topInlinkRendering.getRainbow(arrowColor);
-		// for (int i = 0; i < diamonds.size() - 1; i++) {
-		// final Ftile diam1 = diamonds.get(i);
-		// final Ftile diam2 = diamonds.get(i + 1);
-		// conns.add(result.new ConnectionHorizontal(diam1, diam2, topInColor));
-		// }
 		conns.add(result.new ConnectionIn(topInColor));
-		// conns.add(result.new ConnectionLastElseIn(FtileIfWithLinks.getInColor(branch2, arrowColor)));
-		// conns.add(result.new ConnectionLastElseOut(arrowColor));
-		// final HtmlColor horizontalOutColor = LinkRendering.getColor(afterEndwhile, arrowColor);
-		// conns.add(result.new ConnectionHline(horizontalOutColor));
 		conns.add(result.new ConnectionLastElse(topInColor));
 		conns.add(result.new ConnectionLastElseOut(arrowColor));
 
 		return FtileUtils.addConnection(result, conns);
 	}
 
-	static private double getYdiamontOutToLeft(FtileGeometry dimDiamond1, StringBounder stringBounder) {
-		return (dimDiamond1.getInY() + dimDiamond1.getOutY()) / 2;
-	}
-
 	class ConnectionIn extends AbstractConnection {
 
 		private final Rainbow arrowColor;
@@ -220,7 +197,7 @@ class FtileIfLongVertical extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final UTranslate tr = getTranslateDiamond(getFtile2(), ug.getStringBounder());
 			final Point2D p2 = tr.getTranslated(getFtile2().calculateDimension(ug.getStringBounder()).getPointIn());
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			final Point2D p1 = calculateDimensionInternal(ug.getStringBounder()).getPointIn();
 
 			snake.addPoint(p1);
@@ -246,7 +223,7 @@ class FtileIfLongVertical extends AbstractFtile {
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToDown());
+			final Snake snake = Snake.create(color, Arrows.asToDown());
 			snake.addPoint(p1);
 			snake.addPoint(p2.getX(), p1.getY());
 			snake.addPoint(p2);
@@ -256,8 +233,8 @@ class FtileIfLongVertical extends AbstractFtile {
 		private Point2D getP1(StringBounder stringBounder) {
 			final Dimension2D dimDiamond1 = getFtile1().calculateDimension(stringBounder);
 			final double diamondWidth = dimDiamond1.getWidth();
-			return getTranslateDiamond(getFtile1(), stringBounder).getTranslated(
-					new Point2D.Double(diamondWidth, dimDiamond1.getHeight() / 2));
+			return getTranslateDiamond(getFtile1(), stringBounder)
+					.getTranslated(new Point2D.Double(diamondWidth, dimDiamond1.getHeight() / 2));
 		}
 
 		private Point2D getP2(StringBounder stringBounder) {
@@ -281,7 +258,7 @@ class FtileIfLongVertical extends AbstractFtile {
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), color, Arrows.asToDown());
+			final Snake snake = Snake.create(color, Arrows.asToDown());
 			snake.addPoint(p1);
 			snake.addPoint(p2);
 			ug.draw(snake);
@@ -314,10 +291,10 @@ class FtileIfLongVertical extends AbstractFtile {
 			final FtileGeometry dimDiamond = getFtile1().calculateDimension(stringBounder);
 			final Point2D p1 = tr1.getTranslated(dimDiamond.getPointOut());
 
-			final Point2D p2 = getTranslate2(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			final Point2D p2 = getTranslate2(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			snake.addPoint(p1);
 			snake.addPoint(p1.getX(), p2.getY() - 15);
 			snake.addPoint(p2.getX(), p2.getY() - 15);
@@ -343,10 +320,10 @@ class FtileIfLongVertical extends AbstractFtile {
 				return;
 			}
 			final Point2D p1 = getTranslate2(stringBounder).getTranslated(dim1.getPointOut());
-			final Point2D p2 = getTranslateLastDiamond(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			final Point2D p2 = getTranslateLastDiamond(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			snake.addPoint(p1);
 			snake.addPoint(p1.getX(), p2.getY() - 15);
 			snake.addPoint(p2.getX(), p2.getY() - 15);
@@ -374,13 +351,13 @@ class FtileIfLongVertical extends AbstractFtile {
 			final Point2D p1 = getTranslate1(getFtile1(), stringBounder).getTranslated(dim1.getPointOut());
 
 			final FtileGeometry dimLastDiamond = getFtile2().calculateDimension(stringBounder);
-			Point2D p2 = getTranslateLastDiamond(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			Point2D p2 = getTranslateLastDiamond(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
 			p2 = new UTranslate(dimLastDiamond.getWidth() / 2, dimLastDiamond.getHeight() / 2).getTranslated(p2);
 
 			final Dimension2D dimTotal = calculateDimensionInternal(stringBounder);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToLeft());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToLeft());
 			snake.addPoint(p1);
 			snake.addPoint(p1.getX(), p1.getY() + 15);
 			snake.addPoint(dimTotal.getWidth(), p1.getY() + 15);
@@ -411,7 +388,7 @@ class FtileIfLongVertical extends AbstractFtile {
 
 			final Point2D p2 = new Point2D.Double(dimTotal.getWidth(), p1.getY() + 15);
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToRight());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToRight());
 			snake.addPoint(p1);
 			snake.addPoint(p1.getX(), p2.getY());
 			snake.addPoint(p2);
@@ -486,33 +463,8 @@ class FtileIfLongVertical extends AbstractFtile {
 		final FtileGeometry dimTotal = calculateDimensionInternal(stringBounder);
 		final double x = (dimTotal.getWidth() - dim2.getWidth()) / 2;
 		return new UTranslate(x, y1);
-
-		// final Dimension2D dimTotal = calculateDimensionInternal(stringBounder);
-		// final Dimension2D dim2 = tile2.calculateDimension(stringBounder);
-		//
-		// final double x2 = dimTotal.getWidth() - dim2.getWidth();
-		//
-		// final double h = 0; // getAllDiamondsHeight(stringBounder);
-		// final double y2 = (dimTotal.getHeight() - h * 2 - dim2.getHeight()) / 2 + h;
-		//
-		// return new UTranslate(x2, y2);
-
 	}
 
-	// private UTranslate getTranslateCouple1(Ftile candidat, StringBounder stringBounder) {
-	// double x1 = 0;
-	//
-	// for (Ftile couple : couples) {
-	// final FtileGeometry dim1 = couple.calculateDimension(stringBounder);
-	// if (couple == candidat) {
-	// return new UTranslate(x1, 25);
-	// }
-	// x1 += dim1.getWidth() + xSeparation;
-	// }
-	// throw new IllegalArgumentException();
-	//
-	// }
-
 	public void drawU(UGraphic ug) {
 		final StringBounder stringBounder = ug.getStringBounder();
 		for (Ftile tile1 : tiles) {
@@ -527,7 +479,6 @@ class FtileIfLongVertical extends AbstractFtile {
 	}
 
 	private FtileGeometry calculateDimensionInternal(StringBounder stringBounder) {
-		// FtileGeometry result = new FtileGeometry(0, marginy1, 0, 0);
 		double col1 = 0;
 		double col1overpass = 0;
 		double col2 = 0;
@@ -563,18 +514,11 @@ class FtileIfLongVertical extends AbstractFtile {
 		return width;
 	}
 
-	private double allTile1Width(StringBounder stringBounder) {
-		double width = 0;
-		for (Ftile tile1 : tiles) {
-			width = Math.max(width, tile1.calculateDimension(stringBounder).getWidth());
-		}
-		return width;
-	}
-
-	public FtileGeometry calculateDimension(StringBounder stringBounder) {
+	@Override
+	protected FtileGeometry calculateDimensionFtile(StringBounder stringBounder) {
 		final Dimension2D dimTotal = calculateDimensionInternal(stringBounder);
 
-		final List<Ftile> all = new ArrayList<Ftile>(tiles);
+		final List<Ftile> all = new ArrayList<>(tiles);
 		all.add(tile2);
 		for (Ftile tmp : all) {
 			if (tmp.calculateDimension(stringBounder).hasPointOut()) {
