@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -61,27 +61,27 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.MergeStrategy;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Snake;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamond;
-import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondFoo1;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondInside;
+import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondSquare;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.Rainbow;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
+import net.sourceforge.plantuml.graphic.TextBlockUtils;
 import net.sourceforge.plantuml.svek.ConditionStyle;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UEmpty;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 class FtileWhile extends AbstractFtile {
 
 	private final Ftile whileBlock;
 	private final Ftile diamond1;
 	private final Ftile specialOut;
+	private final Ftile backward;
 
 	@Override
 	public Collection<Ftile> getMyChildren() {
@@ -92,7 +92,7 @@ class FtileWhile extends AbstractFtile {
 	}
 
 	public Set<Swimlane> getSwimlanes() {
-		final Set<Swimlane> result = new HashSet<Swimlane>(whileBlock.getSwimlanes());
+		final Set<Swimlane> result = new HashSet<>(whileBlock.getSwimlanes());
 		result.add(getSwimlaneIn());
 		return result;
 	}
@@ -105,60 +105,65 @@ class FtileWhile extends AbstractFtile {
 		return getSwimlaneIn();
 	}
 
-	private FtileWhile(Ftile whileBlock, Ftile diamond1, Ftile specialOut) {
+	private FtileWhile(Ftile whileBlock, Ftile diamond1, Ftile specialOut, Ftile backward) {
 		super(whileBlock.skinParam());
 		this.whileBlock = whileBlock;
 		this.diamond1 = diamond1;
 		this.specialOut = specialOut;
+		this.backward = backward;
 	}
 
-	public static Ftile create(Swimlane swimlane, Ftile whileBlock, Display test, HtmlColor borderColor,
-			HtmlColor backColor, Rainbow arrowColor, Display yes, Display out2, Rainbow endInlinkColor,
-			LinkRendering afterEndwhile, FontConfiguration fontArrow, FtileFactory ftileFactory,
-			ConditionStyle conditionStyle, FontConfiguration fcTest, Instruction specialOut) {
+	public static Ftile create(LinkRendering outColor, Swimlane swimlane, Ftile whileBlock, Display test,
+			HColor borderColor, HColor backColor, Rainbow arrowColor, Display yes, FontConfiguration fontArrow,
+			FtileFactory ftileFactory, ConditionStyle conditionStyle, FontConfiguration fcTest, Instruction specialOut,
+			Ftile backward, LinkRendering incoming1, LinkRendering incoming2) {
 
 		final TextBlock yesTb = yes.create(fontArrow, HorizontalAlignment.LEFT, ftileFactory.skinParam());
-		final TextBlock testTb = test.create(fcTest,
-				whileBlock.skinParam().getDefaultTextAlignment(HorizontalAlignment.LEFT), ftileFactory.skinParam());
-		final TextBlock out = out2.create(fontArrow, HorizontalAlignment.LEFT, ftileFactory.skinParam());
+		final TextBlock testTb = test.isWhite() ? TextBlockUtils.empty(0, 0)
+				: test.create(fcTest, whileBlock.skinParam().getDefaultTextAlignment(HorizontalAlignment.LEFT),
+						ftileFactory.skinParam());
+		final TextBlock outTb = outColor.getDisplay().create(fontArrow, HorizontalAlignment.LEFT,
+				ftileFactory.skinParam());
 
 		final Ftile diamond1;
-		if (conditionStyle == ConditionStyle.INSIDE) {
+		if (conditionStyle == ConditionStyle.INSIDE_HEXAGON) {
 			diamond1 = new FtileDiamondInside(whileBlock.skinParam(), backColor, borderColor, swimlane, testTb)
-					.withNorth(yesTb).withWest(out);
-		} else if (conditionStyle == ConditionStyle.FOO1) {
-			diamond1 = new FtileDiamondFoo1(whileBlock.skinParam(), backColor, borderColor, swimlane, testTb)
-					.withNorth(yesTb).withWest(out);
-		} else if (conditionStyle == ConditionStyle.DIAMOND) {
+					.withNorth(yesTb).withWest(outTb);
+		} else if (conditionStyle == ConditionStyle.INSIDE_DIAMOND) {
+			diamond1 = new FtileDiamondSquare(whileBlock.skinParam(), backColor, borderColor, swimlane, testTb)
+					.withNorth(yesTb).withWest(outTb);
+		} else if (conditionStyle == ConditionStyle.EMPTY_DIAMOND) {
 			diamond1 = new FtileDiamond(whileBlock.skinParam(), backColor, borderColor, swimlane).withNorth(testTb)
-					.withSouth(yesTb).withWest(out);
+					.withSouth(yesTb).withWest(outTb);
 		} else {
 			throw new IllegalStateException();
 		}
 
 		final Ftile special = specialOut == null ? null : specialOut.createFtile(ftileFactory);
-
-		final FtileWhile result = new FtileWhile(whileBlock, diamond1, special);
-		Rainbow afterEndwhileColor = arrowColor;
-		if (afterEndwhile != null && afterEndwhile.getRainbow() != null && afterEndwhile.getRainbow().size() != 0) {
-			afterEndwhileColor = afterEndwhile.getRainbow();
-		}
+		final FtileWhile result = new FtileWhile(whileBlock, diamond1, special, backward);
 
 		final Dimension2D dim = whileBlock.calculateDimension(ftileFactory.getStringBounder());
-		final Display backDisplay = whileBlock.getOutLinkRendering().getDisplay();
-		final TextBlock back = backDisplay.create(fontArrow, HorizontalAlignment.LEFT, ftileFactory.skinParam());
+		final TextBlock back1 = incoming1.getDisplay().create(fontArrow, HorizontalAlignment.LEFT,
+				ftileFactory.skinParam());
 
-		final List<Connection> conns = new ArrayList<Connection>();
+		final List<Connection> conns = new ArrayList<>();
 		if (dim.getWidth() == 0 || dim.getHeight() == 0) {
-			conns.add(result.new ConnectionBackEmpty(endInlinkColor));
+			conns.add(result.new ConnectionBackEmpty(incoming1.getRainbow()));
 		} else {
 			conns.add(result.new ConnectionIn(whileBlock.getInLinkRendering().getRainbow(arrowColor)));
-			conns.add(result.new ConnectionBack(endInlinkColor, back));
+			if (backward == null) {
+				conns.add(result.new ConnectionBackSimple(incoming1.getRainbow(), back1));
+			} else {
+				final TextBlock back2 = incoming2.getDisplay().create(fontArrow, HorizontalAlignment.LEFT,
+						ftileFactory.skinParam());
+				conns.add(result.new ConnectionBackBackward1(incoming1.getRainbow(), back1));
+				conns.add(result.new ConnectionBackBackward2(incoming2.getRainbow(), back2));
+			}
 		}
 		if (specialOut == null) {
-			conns.add(result.new ConnectionOut(afterEndwhileColor));
+			conns.add(result.new ConnectionOut(outColor.getRainbow()));
 		} else {
-			conns.add(result.new ConnectionOutSpecial(afterEndwhileColor));
+			conns.add(result.new ConnectionOutSpecial(outColor.getRainbow()));
 		}
 		return FtileUtils.addConnection(result, conns);
 	}
@@ -172,30 +177,31 @@ class FtileWhile extends AbstractFtile {
 		}
 
 		private Point2D getP1(final StringBounder stringBounder) {
-			return getTranslateDiamond1(stringBounder).getTranslated(
-					getFtile1().calculateDimension(stringBounder).getPointOut());
+			return getTranslateDiamond1(stringBounder)
+					.getTranslated(getFtile1().calculateDimension(stringBounder).getPointOut());
 		}
 
 		private Point2D getP2(final StringBounder stringBounder) {
-			return getTranslateForWhile(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			return getTranslateForWhile(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
 		}
 
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown());
 			snake.addPoint(getP1(stringBounder));
 			snake.addPoint(getP2(stringBounder));
 
 			ug.draw(snake);
 		}
 
+		@Override
 		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
 			final StringBounder stringBounder = ug.getStringBounder();
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown()).withMerge(MergeStrategy.LIMITED);
 			final Point2D mp1a = translate1.getTranslated(p1);
 			final Point2D mp2b = translate2.getTranslated(p2);
 			final double middle = (mp1a.getY() + mp2b.getY()) / 2.0;
@@ -203,16 +209,15 @@ class FtileWhile extends AbstractFtile {
 			snake.addPoint(mp1a.getX(), middle);
 			snake.addPoint(mp2b.getX(), middle);
 			snake.addPoint(mp2b);
-			snake.goUnmergeable(MergeStrategy.LIMITED);
 			ug.draw(snake);
 		}
 	}
 
-	class ConnectionBack extends AbstractConnection implements ConnectionTranslatable {
+	class ConnectionBackSimple extends AbstractConnection implements ConnectionTranslatable {
 		private final Rainbow endInlinkColor;
 		private final TextBlock back;
 
-		public ConnectionBack(Rainbow endInlinkColor, TextBlock back) {
+		public ConnectionBackSimple(Rainbow endInlinkColor, TextBlock back) {
 			super(whileBlock, diamond1);
 			this.endInlinkColor = endInlinkColor;
 			this.back = back;
@@ -238,7 +243,6 @@ class FtileWhile extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), endInlinkColor, Arrows.asToLeft());
 			final Dimension2D dimTotal = calculateDimension(stringBounder);
 			final Point2D p1 = getP1(stringBounder);
 			if (p1 == null) {
@@ -253,7 +257,8 @@ class FtileWhile extends AbstractFtile {
 			final double half = (dimDiamond1.getOutY() - dimDiamond1.getInY()) / 2;
 			final double y2 = p2.getY() + dimDiamond1.getInY() + half;
 
-			snake.setLabel(back);
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToLeft()).emphasizeDirection(Direction.UP)
+					.withLabel(back, arrowHorizontalAlignment());
 			snake.addPoint(x1, y1);
 			final double y1bis = Math.max(y1, getBottom(stringBounder)) + Diamond.diamondHalfSize;
 			snake.addPoint(x1, y1bis);
@@ -261,17 +266,16 @@ class FtileWhile extends AbstractFtile {
 			snake.addPoint(xx, y1bis);
 			snake.addPoint(xx, y2);
 			snake.addPoint(x2, y2);
-			snake.emphasizeDirection(Direction.UP);
 
 			ug.draw(snake);
-
 			ug.apply(new UTranslate(x1, y1bis)).draw(new UEmpty(5, Diamond.diamondHalfSize));
 
 		}
 
+		@Override
 		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
 			final StringBounder stringBounder = ug.getStringBounder();
-			final Snake snake = new Snake(arrowHorizontalAlignment(), endInlinkColor, Arrows.asToLeft());
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToLeft()).withMerge(MergeStrategy.LIMITED);
 			final Dimension2D dimTotal = calculateDimension(stringBounder);
 			final Point2D ap1 = getP1(stringBounder);
 			final Point2D ap2 = getP2(stringBounder);
@@ -292,20 +296,115 @@ class FtileWhile extends AbstractFtile {
 			snake.addPoint(xx, y1 + Diamond.diamondHalfSize);
 			snake.addPoint(xx, y2);
 			snake.addPoint(x2, y2);
-			snake.goUnmergeable(MergeStrategy.LIMITED);
 
 			ug.draw(snake);
 
 			ug.apply(new UTranslate(x1, y1 + Diamond.diamondHalfSize)).draw(new UEmpty(5, Diamond.diamondHalfSize));
 
-			ug = ug.apply(new UChangeColor(endInlinkColor.getColor())).apply(
-					new UChangeBackColor(endInlinkColor.getColor()));
+			ug = ug.apply(endInlinkColor.getColor()).apply(endInlinkColor.getColor().bg());
 			ug.apply(new UTranslate(xx, (y1 + y2) / 2)).draw(Arrows.asToUp());
 
 		}
 
 	}
 
+	class ConnectionBackBackward1 extends AbstractConnection {
+		private final Rainbow endInlinkColor;
+		private final TextBlock back;
+
+		public ConnectionBackBackward1(Rainbow endInlinkColor, TextBlock back) {
+			super(whileBlock, backward);
+			this.endInlinkColor = endInlinkColor;
+			this.back = back;
+		}
+
+		private Point2D getP1(final StringBounder stringBounder) {
+			final FtileGeometry geo = whileBlock.calculateDimension(stringBounder);
+			if (geo.hasPointOut() == false) {
+				return null;
+			}
+			return getTranslateForWhile(stringBounder).getTranslated(geo.getPointOut());
+		}
+
+		private double getBottom(final StringBounder stringBounder) {
+			final FtileGeometry geo = whileBlock.calculateDimension(stringBounder);
+			return getTranslateForWhile(stringBounder).getDy() + geo.getHeight();
+		}
+
+		private Point2D getP2(final StringBounder stringBounder) {
+			final FtileGeometry dim = backward.calculateDimension(stringBounder);
+			return getTranslateBackward(stringBounder).getTranslated(new Point2D.Double(dim.getLeft(), dim.getOutY()));
+		}
+
+		public void drawU(UGraphic ug) {
+			final StringBounder stringBounder = ug.getStringBounder();
+
+			final Point2D p1 = getP1(stringBounder);
+			if (p1 == null) {
+				return;
+			}
+			final Point2D p2 = getP2(stringBounder);
+			final double x1 = p1.getX();
+			final double y1 = p1.getY();
+			final double x2 = p2.getX();
+			final double y2 = p2.getY();
+
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToUp()).withLabel(back,
+					arrowHorizontalAlignment());
+			snake.addPoint(x1, y1);
+			final double y1bis = Math.max(y1, getBottom(stringBounder)) + Diamond.diamondHalfSize;
+			snake.addPoint(x1, y1bis);
+			snake.addPoint(x2, y1bis);
+			snake.addPoint(x2, y2);
+
+			ug.draw(snake);
+			ug.apply(new UTranslate(x1, y1bis)).draw(new UEmpty(5, Diamond.diamondHalfSize));
+		}
+	}
+
+	class ConnectionBackBackward2 extends AbstractConnection {
+		private final Rainbow endInlinkColor;
+		private final TextBlock back;
+
+		public ConnectionBackBackward2(Rainbow endInlinkColor, TextBlock back) {
+			super(backward, diamond1);
+			this.endInlinkColor = endInlinkColor;
+			this.back = back;
+		}
+
+		private Point2D getP1(final StringBounder stringBounder) {
+			final FtileGeometry dim = backward.calculateDimension(stringBounder);
+			return getTranslateBackward(stringBounder).getTranslated(new Point2D.Double(dim.getLeft(), dim.getInY()));
+		}
+
+		private Point2D getP2(final StringBounder stringBounder) {
+			return getTranslateDiamond1(stringBounder).getTranslated(new Point2D.Double(0, 0));
+		}
+
+		public void drawU(UGraphic ug) {
+			final StringBounder stringBounder = ug.getStringBounder();
+
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToLeft()).withLabel(back,
+					arrowHorizontalAlignment());
+
+			final Point2D p1 = getP1(stringBounder);
+			final Point2D p2 = getP2(stringBounder);
+			final FtileGeometry dimDiamond1 = diamond1.calculateDimension(stringBounder);
+
+			final double x1 = p1.getX();
+			final double y1 = p1.getY();
+			final double x2 = p2.getX() + dimDiamond1.getWidth();
+			final double half = (dimDiamond1.getOutY() - dimDiamond1.getInY()) / 2;
+			final double y2 = p2.getY() + dimDiamond1.getInY() + half;
+
+			snake.addPoint(x1, y1);
+			snake.addPoint(x1, y2);
+			snake.addPoint(x2, y2);
+
+			ug.draw(snake);
+		}
+	}
+
 	class ConnectionBackEmpty extends AbstractConnection {
 		private final Rainbow endInlinkColor;
 
@@ -315,8 +414,8 @@ class FtileWhile extends AbstractFtile {
 		}
 
 		private Point2D getP1(final StringBounder stringBounder) {
-			return getTranslateDiamond1(stringBounder).getTranslated(
-					diamond1.calculateDimension(stringBounder).getPointOut());
+			return getTranslateDiamond1(stringBounder)
+					.getTranslated(diamond1.calculateDimension(stringBounder).getPointOut());
 		}
 
 		private double getBottom(final StringBounder stringBounder) {
@@ -331,7 +430,7 @@ class FtileWhile extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), endInlinkColor, Arrows.asToLeft());
+			final Snake snake = Snake.create(endInlinkColor, Arrows.asToLeft()).emphasizeDirection(Direction.UP);
 			final Dimension2D dimTotal = calculateDimension(stringBounder);
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
@@ -351,7 +450,6 @@ class FtileWhile extends AbstractFtile {
 			snake.addPoint(xx, y1bis);
 			snake.addPoint(xx, y2);
 			snake.addPoint(x2, y2);
-			snake.emphasizeDirection(Direction.UP);
 
 			ug.draw(snake);
 
@@ -381,7 +479,8 @@ class FtileWhile extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), afterEndwhileColor);
+			final Snake snake = Snake.create(afterEndwhileColor).withMerge(MergeStrategy.LIMITED)
+					.emphasizeDirection(Direction.DOWN);
 
 			final FtileGeometry dimDiamond1 = diamond1.calculateDimension(stringBounder);
 			final Point2D p1 = getP1(stringBounder);
@@ -396,12 +495,10 @@ class FtileWhile extends AbstractFtile {
 			snake.addPoint(x1, y1);
 			snake.addPoint(Diamond.diamondHalfSize, y1);
 			snake.addPoint(Diamond.diamondHalfSize, y2);
-			snake.emphasizeDirection(Direction.DOWN);
-			snake.goUnmergeable(MergeStrategy.LIMITED);
 
 			ug.draw(snake);
 
-			final Snake snake2 = new Snake(arrowHorizontalAlignment(), afterEndwhileColor);
+			final Snake snake2 = Snake.create(afterEndwhileColor);
 			snake2.addPoint(Diamond.diamondHalfSize, y2);
 			snake2.addPoint(x2, y2);
 			// snake2.goUnmergeable(MergeStrategy.LIMITED);
@@ -423,14 +520,14 @@ class FtileWhile extends AbstractFtile {
 		}
 
 		private Point2D getP2(final StringBounder stringBounder) {
-			return getTranslateForSpecial(stringBounder).getTranslated(
-					specialOut.calculateDimension(stringBounder).getPointIn());
+			return getTranslateForSpecial(stringBounder)
+					.getTranslated(specialOut.calculateDimension(stringBounder).getPointIn());
 		}
 
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), afterEndwhileColor, Arrows.asToDown());
+			final Snake snake = Snake.create(afterEndwhileColor, Arrows.asToDown());
 
 			final FtileGeometry dimDiamond1 = diamond1.calculateDimension(stringBounder);
 			final Point2D p1 = getP1(stringBounder);
@@ -458,6 +555,18 @@ class FtileWhile extends AbstractFtile {
 		if (specialOut != null) {
 			ug.apply(getTranslateForSpecial(stringBounder)).draw(specialOut);
 		}
+		if (backward != null) {
+			ug.apply(getTranslateBackward(stringBounder)).draw(backward);
+		}
+	}
+
+	private UTranslate getTranslateBackward(StringBounder stringBounder) {
+		final Dimension2D dimTotal = calculateDimensionFtile(stringBounder);
+		final Dimension2D dimBackward = backward.calculateDimension(stringBounder);
+		final double x = dimTotal.getWidth() - dimBackward.getWidth();
+		final double y = (dimTotal.getHeight() - dimBackward.getHeight()) / 2;
+
+		return new UTranslate(x, y);
 	}
 
 	@Override
@@ -472,7 +581,12 @@ class FtileWhile extends AbstractFtile {
 		final FtileGeometry geo = geoDiamond1.appendBottom(geoWhile);
 		final double height = geo.getHeight() + 4 * Diamond.diamondHalfSize;
 		final double dx = 2 * Diamond.diamondHalfSize;
-		return new FtileGeometry(xDeltaBecauseSpecial(stringBounder) + geo.getWidth() + dx + Diamond.diamondHalfSize,
+		double backwardWidth = 0;
+		if (backward != null) {
+			backwardWidth += backward.calculateDimension(stringBounder).getWidth();
+		}
+		return new FtileGeometry(
+				xDeltaBecauseSpecial(stringBounder) + geo.getWidth() + dx + Diamond.diamondHalfSize + backwardWidth,
 				height, xDeltaBecauseSpecial(stringBounder) + geo.getLeft() + dx, geoDiamond1.getInY(), height);
 
 	}
@@ -522,7 +636,13 @@ class FtileWhile extends AbstractFtile {
 		final FtileGeometry dimDiamond1 = diamond1.calculateDimension(stringBounder);
 		final double half = (dimDiamond1.getOutY() - dimDiamond1.getInY()) / 2;
 		final double y1 = Math.max(3 * half, 4 * Diamond.diamondHalfSize);
-		final double x1 = getTranslateForWhile(stringBounder).getDx() - xDeltaBecauseSpecial(stringBounder);
+		final double xWhile = getTranslateForWhile(stringBounder).getDx() - Diamond.diamondHalfSize;
+		final double xDiamond = getTranslateDiamond1(stringBounder).getDx();
+		// final double x1 = xWhile - xDeltaBecauseSpecial(stringBounder);
+		final double x1 = Math.min(xWhile, xDiamond) - xDeltaBecauseSpecial(stringBounder);
+		// final double x1 = getTranslateForWhile(stringBounder).getDx() -
+		// dimDiamond1.getWidth()
+		// - xDeltaBecauseSpecial(stringBounder);
 		return new UTranslate(x1, y1);
 	}
 
