@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,6 +36,9 @@
 package net.sourceforge.plantuml.asciiart;
 
 import java.awt.geom.Dimension2D;
+import java.awt.geom.Point2D;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import net.sourceforge.plantuml.FileFormat;
 import net.sourceforge.plantuml.StringUtils;
@@ -43,6 +46,7 @@ import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.sequencediagram.MessageNumber;
 import net.sourceforge.plantuml.skin.Area;
+import net.sourceforge.plantuml.skin.ArrowComponent;
 import net.sourceforge.plantuml.skin.ArrowConfiguration;
 import net.sourceforge.plantuml.skin.ArrowDirection;
 import net.sourceforge.plantuml.skin.ComponentType;
@@ -50,7 +54,7 @@ import net.sourceforge.plantuml.skin.Context2D;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.txt.UGraphicTxt;
 
-public class ComponentTextArrow extends AbstractComponentText {
+public class ComponentTextArrow extends AbstractComponentText implements ArrowComponent {
 
 	private final ComponentType type;
 	private final Display stringsToDisplay;
@@ -60,14 +64,14 @@ public class ComponentTextArrow extends AbstractComponentText {
 
 	public ComponentTextArrow(ComponentType type, ArrowConfiguration config, Display stringsToDisplay,
 			FileFormat fileFormat, int maxAsciiMessageLength) {
+		this.fileFormat = fileFormat;
 		this.maxAsciiMessageLength = maxAsciiMessageLength;
 		this.type = type;
 		this.config = config;
-		this.stringsToDisplay = clean(stringsToDisplay);
-		this.fileFormat = fileFormat;
+		this.stringsToDisplay = cleanAndManageBoldNumber(stringsToDisplay, fileFormat);
 	}
 
-	private static Display clean(Display orig) {
+	public static Display cleanAndManageBoldNumber(Display orig, FileFormat fileFormat) {
 		if (orig.size() == 0 || orig.get(0) instanceof MessageNumber == false) {
 			return orig;
 		}
@@ -75,7 +79,7 @@ public class ComponentTextArrow extends AbstractComponentText {
 		for (int i = 0; i < orig.size(); i++) {
 			CharSequence element = orig.get(i);
 			if (i == 1) {
-				element = removeTag(orig.get(0).toString()) + " " + element;
+				element = removeTagAndManageBoldNumber(orig.get(0).toString(), fileFormat) + " " + element;
 			}
 			if (i != 0) {
 				result = result.add(element);
@@ -84,7 +88,19 @@ public class ComponentTextArrow extends AbstractComponentText {
 		return result;
 	}
 
-	private static String removeTag(String s) {
+	private static String removeTagAndManageBoldNumber(String s, FileFormat fileFormat) {
+		if (fileFormat == FileFormat.UTXT) {
+			final Pattern pattern = Pattern.compile("\\<b\\>([0-9]+)\\</b\\>");
+			final Matcher matcher = pattern.matcher(s);
+			final StringBuffer result = new StringBuffer();
+			while (matcher.find()) {
+				final String num = matcher.group(1);
+				final String replace = StringUtils.toInternalBoldNumber(num);
+				matcher.appendReplacement(result, Matcher.quoteReplacement(replace));
+			}
+			matcher.appendTail(result);
+			s = result.toString();
+		}
 		return s.replaceAll("\\<[^<>]+\\>", "");
 	}
 
@@ -96,7 +112,7 @@ public class ComponentTextArrow extends AbstractComponentText {
 		final UmlCharArea charArea = ((UGraphicTxt) ug).getCharArea();
 		final int width = (int) dimensionToUse.getWidth();
 		final int height = (int) dimensionToUse.getHeight();
-		final int textWidth = StringUtils.getWidth(stringsToDisplay);
+		final int textWidth = StringUtils.getWcWidth(stringsToDisplay);
 
 		final int yarrow = height - 2;
 		charArea.drawHLine(fileFormat == FileFormat.UTXT ? '\u2500' : '-', yarrow, 1, width);
@@ -117,7 +133,12 @@ public class ComponentTextArrow extends AbstractComponentText {
 			throw new UnsupportedOperationException();
 		}
 		// final int position = Math.max(0, (width - textWidth) / 2);
-		charArea.drawStringsLR(stringsToDisplay.as(), (width - textWidth) / 2, 0);
+
+		if (fileFormat == FileFormat.UTXT) {
+			charArea.drawStringsLRUnicode(stringsToDisplay.asList(), (width - textWidth) / 2, 0);
+		} else {
+			charArea.drawStringsLRSimple(stringsToDisplay.asList(), (width - textWidth) / 2, 0);
+		}
 	}
 
 	public double getPreferredHeight(StringBounder stringBounder) {
@@ -125,11 +146,31 @@ public class ComponentTextArrow extends AbstractComponentText {
 	}
 
 	public double getPreferredWidth(StringBounder stringBounder) {
-		final int width = StringUtils.getWidth(stringsToDisplay) + 2;
+		final int width = StringUtils.getWcWidth(stringsToDisplay) + 2;
 		if (maxAsciiMessageLength > 0) {
 			return Math.min(maxAsciiMessageLength, width);
 		}
 		return width;
 	}
 
+	public Point2D getStartPoint(StringBounder stringBounder, Dimension2D dimensionToUse) {
+		return new Point2D.Double(0, 0);
+	}
+
+	public Point2D getEndPoint(StringBounder stringBounder, Dimension2D dimensionToUse) {
+		return new Point2D.Double(0, 0);
+	}
+
+	public double getPaddingY() {
+		throw new UnsupportedOperationException();
+	}
+
+	public double getYPoint(StringBounder stringBounder) {
+		throw new UnsupportedOperationException();
+	}
+
+	public double getPosArrow(StringBounder stringBounder) {
+		throw new UnsupportedOperationException();
+	}
+
 }
