@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,6 +36,7 @@
 package net.sourceforge.plantuml.classdiagram.command;
 
 import net.sourceforge.plantuml.FontParam;
+import net.sourceforge.plantuml.StringLocated;
 import net.sourceforge.plantuml.StringUtils;
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.UrlBuilder;
@@ -45,29 +46,34 @@ import net.sourceforge.plantuml.command.BlocLines;
 import net.sourceforge.plantuml.command.CommandExecutionResult;
 import net.sourceforge.plantuml.command.CommandMultilines2;
 import net.sourceforge.plantuml.command.MultilinesStrategy;
+import net.sourceforge.plantuml.command.regex.IRegex;
 import net.sourceforge.plantuml.command.regex.RegexConcat;
 import net.sourceforge.plantuml.command.regex.RegexLeaf;
+import net.sourceforge.plantuml.command.regex.RegexOptional;
 import net.sourceforge.plantuml.command.regex.RegexOr;
 import net.sourceforge.plantuml.command.regex.RegexResult;
 import net.sourceforge.plantuml.cucadiagram.Code;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.IEntity;
 import net.sourceforge.plantuml.cucadiagram.ILeaf;
+import net.sourceforge.plantuml.cucadiagram.Ident;
 import net.sourceforge.plantuml.cucadiagram.LeafType;
 import net.sourceforge.plantuml.cucadiagram.Link;
 import net.sourceforge.plantuml.cucadiagram.LinkDecor;
 import net.sourceforge.plantuml.cucadiagram.LinkType;
+import net.sourceforge.plantuml.cucadiagram.Stereotag;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.color.ColorParser;
 import net.sourceforge.plantuml.graphic.color.ColorType;
 import net.sourceforge.plantuml.graphic.color.Colors;
 import net.sourceforge.plantuml.skin.VisibilityModifier;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.NoSuchColorException;
 
 public class CommandCreateClassMultilines extends CommandMultilines2<ClassDiagram> {
 
-	private static final String CODE = CommandLinkClass.getSeparator() + "?[\\p{L}0-9_]+" + "(?:"
-			+ CommandLinkClass.getSeparator() + "[\\p{L}0-9_]+)*";
+	private static final String CODE = CommandLinkClass.getSeparator() + "?[%pLN_]+" + "(?:"
+			+ CommandLinkClass.getSeparator() + "[%pLN_]+)*";
 	public static final String CODES = CODE + "(?:\\s*,\\s*" + CODE + ")*";
 
 	enum Mode {
@@ -80,82 +86,116 @@ public class CommandCreateClassMultilines extends CommandMultilines2<ClassDiagra
 
 	@Override
 	public String getPatternEnd() {
-		return "(?i)^[%s]*\\}[%s]*$";
+		return "^[%s]*\\}[%s]*$";
 	}
 
-	private static RegexConcat getRegexConcat() {
-		return new RegexConcat(new RegexLeaf("^"), //
+	private static IRegex getRegexConcat() {
+		return RegexConcat.build(CommandCreateClassMultilines.class.getName(), RegexLeaf.start(), //
 				new RegexLeaf("VISIBILITY", "(" + VisibilityModifier.regexForVisibilityCharacterInClassName() + ")?"), //
-				new RegexLeaf("TYPE", "(interface|enum|abstract[%s]+class|abstract|class|entity)[%s]+"), //
+				new RegexLeaf("TYPE", "(interface|enum|annotation|abstract[%s]+class|abstract|class|entity)"), //
+				RegexLeaf.spaceOneOrMore(), //
 				new RegexOr(//
 						new RegexConcat(//
 								new RegexLeaf("DISPLAY1", CommandCreateClass.DISPLAY_WITH_GENERIC), //
-								new RegexLeaf("[%s]+as[%s]+"), //
+								RegexLeaf.spaceOneOrMore(), //
+								new RegexLeaf("as"), //
+								RegexLeaf.spaceOneOrMore(), //
 								new RegexLeaf("CODE1", "(" + CommandCreateClass.CODE + ")")), //
 						new RegexConcat(//
 								new RegexLeaf("CODE2", "(" + CommandCreateClass.CODE + ")"), //
-								new RegexLeaf("[%s]+as[%s]+"), // //
+								RegexLeaf.spaceOneOrMore(), //
+								new RegexLeaf("as"), //
+								RegexLeaf.spaceOneOrMore(), //
 								new RegexLeaf("DISPLAY2", CommandCreateClass.DISPLAY_WITH_GENERIC)), //
 						new RegexLeaf("CODE3", "(" + CommandCreateClass.CODE + ")"), //
 						new RegexLeaf("CODE4", "[%g]([^%g]+)[%g]")), //
-				new RegexLeaf("GENERIC", "(?:[%s]*\\<(" + GenericRegexProducer.PATTERN + ")\\>)?"), //
-				new RegexLeaf("[%s]*"), //
+				new RegexOptional(new RegexConcat(RegexLeaf.spaceZeroOrMore(),
+						new RegexLeaf("GENERIC", "\\<(" + GenericRegexProducer.PATTERN + ")\\>"))), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("STEREO", "(\\<\\<.+\\>\\>)?"), //
-				new RegexLeaf("[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("TAGS", Stereotag.pattern() + "?"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("URL", "(" + UrlBuilder.getRegexp() + ")?"), //
-				new RegexLeaf("[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				color().getRegex(), //
-				new RegexLeaf("[%s]*"), //
-				new RegexLeaf("LINECOLOR", "(?:##(?:\\[(dotted|dashed|bold)\\])?(\\w+)?)?"), //
-				new RegexLeaf("EXTENDS", "([%s]+(extends)[%s]+(" + CODES + "))?"), //
-				new RegexLeaf("IMPLEMENTS", "([%s]+(implements)[%s]+(" + CODES + "))?"), //
-				new RegexLeaf("[%s]*\\{[%s]*$"));
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexOptional(new RegexConcat(new RegexLeaf("##"),
+						new RegexLeaf("LINECOLOR", "(?:\\[(dotted|dashed|bold)\\])?(\\w+)?"))), //
+				new RegexOptional(new RegexConcat(RegexLeaf.spaceOneOrMore(),
+						new RegexLeaf("EXTENDS", "(extends)[%s]+(" + CommandCreateClassMultilines.CODES + ")"))), //
+				new RegexOptional(new RegexConcat(RegexLeaf.spaceOneOrMore(),
+						new RegexLeaf("IMPLEMENTS", "(implements)[%s]+(" + CommandCreateClassMultilines.CODES + ")"))), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("\\{"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				RegexLeaf.end() //
+		);
+	}
+
+	@Override
+	public boolean syntaxWithFinalBracket() {
+		return true;
 	}
 
 	private static ColorParser color() {
 		return ColorParser.simpleColor(ColorType.BACK);
 	}
 
-	public CommandExecutionResult executeNow(ClassDiagram diagram, BlocLines lines) {
+	@Override
+	protected CommandExecutionResult executeNow(ClassDiagram diagram, BlocLines lines) throws NoSuchColorException {
 		lines = lines.trimSmart(1);
-		lines = lines.removeComments();
-		final RegexResult line0 = getStartingPattern().matcher(StringUtils.trin(lines.getFirst499()));
+		final RegexResult line0 = getStartingPattern().matcher(lines.getFirst().getTrimmed().getString());
 		final IEntity entity = executeArg0(diagram, line0);
 		if (entity == null) {
 			return CommandExecutionResult.error("No such entity");
 		}
 		if (lines.size() > 1) {
+			entity.setCodeLine(lines.getAt(0).getLocation());
 			lines = lines.subExtract(1, 1);
-			final Url url = null;
-//			if (lines.size() > 0) {
-//				final UrlBuilder urlBuilder = new UrlBuilder(diagram.getSkinParam().getValue("topurl"), ModeUrl.STRICT);
-//				url = urlBuilder.getUrl(lines.getFirst499().toString());
-//			} else {
-//				url = null;
-//			}
-//			if (url != null) {
-//				lines = lines.subExtract(1, 0);
-//			}
-			for (CharSequence s : lines) {
-				if (s.length() > 0 && VisibilityModifier.isVisibilityCharacter(s)) {
+			// final Url url = null;
+			// if (lines.size() > 0) {
+			// final UrlBuilder urlBuilder = new
+			// UrlBuilder(diagram.getSkinParam().getValue("topurl"), ModeUrl.STRICT);
+			// url = urlBuilder.getUrl(lines.getFirst499().toString());
+			// } else {
+			// url = null;
+			// }
+			// if (url != null) {
+			// lines = lines.subExtract(1, 0);
+			// }
+			for (StringLocated s : lines) {
+				if (s.getString().length() > 0 && VisibilityModifier.isVisibilityCharacter(s.getString())) {
 					diagram.setVisibilityModifierPresent(true);
 				}
-				entity.getBodier().addFieldOrMethod(s.toString(), entity);
-			}
-			if (url != null) {
-				entity.addUrl(url);
+				entity.getBodier().addFieldOrMethod(s.getString());
 			}
+//			if (url != null) {
+//				entity.addUrl(url);
+//			}
 		}
 
 		manageExtends("EXTENDS", diagram, line0, entity);
 		manageExtends("IMPLEMENTS", diagram, line0, entity);
+		addTags(entity, line0.get("TAGS", 0));
 
 		return CommandExecutionResult.ok();
 	}
 
-	public static void manageExtends(String keyword, ClassDiagram system, RegexResult arg, final IEntity entity) {
-		if (arg.get(keyword, 1) != null) {
-			final Mode mode = arg.get(keyword, 1).equalsIgnoreCase("extends") ? Mode.EXTENDS : Mode.IMPLEMENTS;
+	public static void addTags(IEntity entity, String tags) {
+		if (tags == null) {
+			return;
+		}
+		for (String tag : tags.split("[ ]+")) {
+			assert tag.startsWith("$");
+			tag = tag.substring(1);
+			entity.addStereotag(new Stereotag(tag));
+		}
+	}
+
+	public static void manageExtends(String keyword, ClassDiagram diagram, RegexResult arg, final IEntity entity) {
+		if (arg.get(keyword, 0) != null) {
+			final Mode mode = arg.get(keyword, 0).equalsIgnoreCase("extends") ? Mode.EXTENDS : Mode.IMPLEMENTS;
 			LeafType type2 = LeafType.CLASS;
 			if (mode == Mode.IMPLEMENTS) {
 				type2 = LeafType.INTERFACE;
@@ -163,22 +203,25 @@ public class CommandCreateClassMultilines extends CommandMultilines2<ClassDiagra
 			if (mode == Mode.EXTENDS && entity.getLeafType() == LeafType.INTERFACE) {
 				type2 = LeafType.INTERFACE;
 			}
-			final String codes = arg.get(keyword, 2);
+			final String codes = arg.get(keyword, 1);
 			for (String s : codes.split(",")) {
-				final Code other = Code.of(StringUtils.trin(s));
-				final IEntity cl2 = system.getOrCreateLeaf(other, type2, null);
+				final String idShort = StringUtils.trin(s);
+				final Ident ident = diagram.buildLeafIdent(idShort);
+				final Code other = diagram.V1972() ? ident : diagram.buildCode(idShort);
+				final IEntity cl2 = diagram.getOrCreateLeaf(ident, other, type2, null);
 				LinkType typeLink = new LinkType(LinkDecor.NONE, LinkDecor.EXTENDS);
 				if (type2 == LeafType.INTERFACE && entity.getLeafType() != LeafType.INTERFACE) {
-					typeLink = typeLink.getDashed();
+					typeLink = typeLink.goDashed();
 				}
 				final Link link = new Link(cl2, entity, typeLink, Display.NULL, 2, null, null,
-						system.getLabeldistance(), system.getLabelangle());
-				system.addLink(link);
+						diagram.getLabeldistance(), diagram.getLabelangle(),
+						diagram.getSkinParam().getCurrentStyleBuilder());
+				diagram.addLink(link);
 			}
 		}
 	}
 
-	private IEntity executeArg0(ClassDiagram diagram, RegexResult arg) {
+	private IEntity executeArg0(ClassDiagram diagram, RegexResult arg) throws NoSuchColorException {
 
 		final LeafType type = LeafType.getLeafType(StringUtils.goUpperCase(arg.get("TYPE", 0)));
 		final String visibilityString = arg.get("VISIBILITY", 0);
@@ -187,27 +230,43 @@ public class CommandCreateClassMultilines extends CommandMultilines2<ClassDiagra
 			visibilityModifier = VisibilityModifier.getVisibilityModifier(visibilityString + "FOO", false);
 		}
 
-		final Code code = Code.of(arg.getLazzy("CODE", 0)).eventuallyRemoveStartingAndEndingDoubleQuote("\"([:");
+		final String idShort = StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(arg.getLazzy("CODE", 0),
+				"\"([:");
+		final Ident ident = diagram.buildLeafIdent(idShort);
+		final Code code = diagram.V1972() ? ident : diagram.buildCode(idShort);
 		final String display = arg.getLazzy("DISPLAY", 0);
 		final String genericOption = arg.getLazzy("DISPLAY", 1);
 		final String generic = genericOption != null ? genericOption : arg.get("GENERIC", 0);
 
 		final String stereotype = arg.get("STEREO", 0);
 
-		final ILeaf result;
-		if (diagram.leafExist(code)) {
-			result = diagram.getOrCreateLeaf(code, null, null);
-			if (result.muteToType(type, null) == false) {
-				return null;
+		/* final */ILeaf result;
+		if (diagram.V1972()) {
+			result = diagram.getLeafSmart(ident);
+			if (result != null) {
+				// result = diagram.getOrCreateLeaf(ident, code, null, null);
+				diagram.setLastEntity(result);
+				if (result.muteToType(type, null) == false) {
+					return null;
+				}
+			} else {
+				result = diagram.createLeaf(ident, code, Display.getWithNewlines(display), type, null);
 			}
 		} else {
-			result = diagram.createLeaf(code, Display.getWithNewlines(display), type, null);
+			if (diagram.leafExist(code)) {
+				result = diagram.getOrCreateLeaf(ident, code, null, null);
+				if (result.muteToType(type, null) == false) {
+					return null;
+				}
+			} else {
+				result = diagram.createLeaf(ident, code, Display.getWithNewlines(display), type, null);
+			}
 		}
 		result.setVisibilityModifier(visibilityModifier);
 		if (stereotype != null) {
-			result.setStereotype(new Stereotype(stereotype, diagram.getSkinParam().getCircledCharacterRadius(), diagram
-					.getSkinParam().getFont(null, false, FontParam.CIRCLED_CHARACTER), diagram.getSkinParam()
-					.getIHtmlColorSet()));
+			result.setStereotype(new Stereotype(stereotype, diagram.getSkinParam().getCircledCharacterRadius(),
+					diagram.getSkinParam().getFont(null, false, FontParam.CIRCLED_CHARACTER),
+					diagram.getSkinParam().getIHtmlColorSet()));
 		}
 
 		final String urlString = arg.get("URL", 0);
@@ -217,9 +276,12 @@ public class CommandCreateClassMultilines extends CommandMultilines2<ClassDiagra
 			result.addUrl(url);
 		}
 
-		Colors colors = color().getColor(arg, diagram.getSkinParam().getIHtmlColorSet());
+		Colors colors = color().getColor(diagram.getSkinParam().getThemeStyle(), arg,
+				diagram.getSkinParam().getIHtmlColorSet());
+		final String s = arg.get("LINECOLOR", 1);
 
-		final HtmlColor lineColor = diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(arg.get("LINECOLOR", 1));
+		final HColor lineColor = s == null ? null
+				: diagram.getSkinParam().getIHtmlColorSet().getColor(diagram.getSkinParam().getThemeStyle(), s);
 		if (lineColor != null) {
 			colors = colors.add(ColorType.LINE, lineColor);
 		}
@@ -229,9 +291,11 @@ public class CommandCreateClassMultilines extends CommandMultilines2<ClassDiagra
 		result.setColors(colors);
 
 		// result.setSpecificColorTOBEREMOVED(ColorType.BACK,
-		// diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(arg.get("COLOR", 0)));
+		// diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(arg.get("COLOR",
+		// 0)));
 		// result.setSpecificColorTOBEREMOVED(ColorType.LINE,
-		// diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(arg.get("LINECOLOR", 1)));
+		// diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(arg.get("LINECOLOR",
+		// 1)));
 		// result.applyStroke(arg.get("LINECOLOR", 0));
 
 		if (generic != null) {
