@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,39 +35,48 @@
  */
 package net.sourceforge.plantuml.command;
 
-import net.sourceforge.plantuml.StringUtils;
-import net.sourceforge.plantuml.UmlDiagram;
+import net.sourceforge.plantuml.TitledDiagram;
+import net.sourceforge.plantuml.command.regex.IRegex;
 import net.sourceforge.plantuml.command.regex.RegexConcat;
 import net.sourceforge.plantuml.command.regex.RegexLeaf;
+import net.sourceforge.plantuml.command.regex.RegexOptional;
 import net.sourceforge.plantuml.command.regex.RegexResult;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.DisplayPositionned;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
 import net.sourceforge.plantuml.graphic.VerticalAlignment;
+import net.sourceforge.plantuml.ugraphic.color.NoSuchColorException;
 
-public class CommandMultilinesLegend extends CommandMultilines2<UmlDiagram> {
+public class CommandMultilinesLegend extends CommandMultilines2<TitledDiagram> {
 
 	public CommandMultilinesLegend() {
 		super(getRegexConcat(), MultilinesStrategy.REMOVE_STARTING_QUOTE);
 	}
 
-	private static RegexConcat getRegexConcat() {
-		return new RegexConcat(new RegexLeaf("^"), //
+	private static IRegex getRegexConcat() {
+		return RegexConcat.build(CommandMultilinesLegend.class.getName(), RegexLeaf.start(), //
 				new RegexLeaf("legend"), //
-				new RegexLeaf("VALIGN", "(?:[%s]+(top|bottom))?"), //
-				new RegexLeaf("ALIGN", "(?:[%s]+(left|right|center))?"), //
-				new RegexLeaf("$"));
+				new RegexOptional( //
+						new RegexConcat( //
+								RegexLeaf.spaceOneOrMore(), //
+								new RegexLeaf("VALIGN", "(top|bottom)") //
+						)), //
+				new RegexOptional( //
+						new RegexConcat( //
+								RegexLeaf.spaceOneOrMore(), //
+								new RegexLeaf("ALIGN", "(left|right|center)") //
+						)), RegexLeaf.end());
 	}
 
 	@Override
 	public String getPatternEnd() {
-		return "(?i)^end[%s]?legend$";
+		return "^end[%s]?legend$";
 	}
 
 	@Override
-	protected CommandExecutionResult executeNow(UmlDiagram diagram, BlocLines lines) {
+	protected CommandExecutionResult executeNow(TitledDiagram diagram, BlocLines lines) throws NoSuchColorException {
 		lines = lines.trimSmart(1);
-		final RegexResult line0 = getStartingPattern().matcher(StringUtils.trin(lines.getFirst499()));
+		final RegexResult line0 = getStartingPattern().matcher(lines.getFirst().getTrimmed().getString());
 		final String align = line0.get("ALIGN", 0);
 		final String valign = line0.get("VALIGN", 0);
 		lines = lines.subExtract(1, 1);
@@ -79,7 +88,7 @@ public class CommandMultilinesLegend extends CommandMultilines2<UmlDiagram> {
 			if (alignment == null) {
 				alignment = HorizontalAlignment.CENTER;
 			}
-			diagram.setLegend(DisplayPositionned.single(strings, alignment, valignment));
+			diagram.setLegend(DisplayPositionned.single(strings.replaceBackslashT(), alignment, valignment));
 			return CommandExecutionResult.ok();
 		}
 		return CommandExecutionResult.error("No legend defined");
