@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -37,6 +37,7 @@ package net.sourceforge.plantuml.command.note.sequence;
 
 import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.FontParam;
+import net.sourceforge.plantuml.LineLocation;
 import net.sourceforge.plantuml.StringUtils;
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.UrlBuilder;
@@ -48,9 +49,11 @@ import net.sourceforge.plantuml.command.CommandMultilines2;
 import net.sourceforge.plantuml.command.MultilinesStrategy;
 import net.sourceforge.plantuml.command.SingleLineCommand2;
 import net.sourceforge.plantuml.command.note.SingleMultiFactoryCommand;
+import net.sourceforge.plantuml.command.regex.IRegex;
 import net.sourceforge.plantuml.command.regex.RegexConcat;
 import net.sourceforge.plantuml.command.regex.RegexLeaf;
 import net.sourceforge.plantuml.command.regex.RegexResult;
+import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.graphic.color.ColorParser;
 import net.sourceforge.plantuml.graphic.color.ColorType;
@@ -60,39 +63,48 @@ import net.sourceforge.plantuml.sequencediagram.NotePosition;
 import net.sourceforge.plantuml.sequencediagram.NoteStyle;
 import net.sourceforge.plantuml.sequencediagram.Participant;
 import net.sourceforge.plantuml.sequencediagram.SequenceDiagram;
+import net.sourceforge.plantuml.ugraphic.color.NoSuchColorException;
 
 public final class FactorySequenceNoteCommand implements SingleMultiFactoryCommand<SequenceDiagram> {
 
-	private RegexConcat getRegexConcatMultiLine() {
-		return new RegexConcat(//
-				new RegexLeaf("^"), //
-				new RegexLeaf("VMERGE", "(/)?[%s]*"), //
+	private IRegex getRegexConcatMultiLine() {
+		return RegexConcat.build(FactorySequenceNoteCommand.class.getName() + "multi", RegexLeaf.start(), //
+				new RegexLeaf("PARALLEL", "(&[%s]*)?"), //
+				new RegexLeaf("VMERGE", "(/)?"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("STYLE", "(note|hnote|rnote)"), //
-				new RegexLeaf("[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("STEREO", "(\\<{2}.*\\>{2})?"), //
-				new RegexLeaf("[%s]*"), //
-				new RegexLeaf("POSITION", "(right|left|over)[%s]+"), //
-				new RegexLeaf("PARTICIPANT", "(?:of[%s]+)?([\\p{L}0-9_.@]+|[%g][^%g]+[%g])[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("POSITION", "(right|left|over)"), //
+				RegexLeaf.spaceOneOrMore(), //
+				new RegexLeaf("PARTICIPANT", "(?:of[%s]+)?([%pLN_.@]+|[%g][^%g]+[%g])"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				color().getRegex(), //
-				new RegexLeaf("URL", "[%s]*(" + UrlBuilder.getRegexp() + ")?"), //
-				new RegexLeaf("$"));
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("URL", "(" + UrlBuilder.getRegexp() + ")?"), RegexLeaf.end());
 	}
 
-	private RegexConcat getRegexConcatSingleLine() {
-		return new RegexConcat(//
-				new RegexLeaf("^"), //
-				new RegexLeaf("VMERGE", "(/)?[%s]*"), //
+	private IRegex getRegexConcatSingleLine() {
+		return RegexConcat.build(FactorySequenceNoteCommand.class.getName() + "single", RegexLeaf.start(), //
+				new RegexLeaf("PARALLEL", "(&[%s]*)?"), //
+				new RegexLeaf("VMERGE", "(/)?"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("STYLE", "(note|hnote|rnote)"), //
-				new RegexLeaf("[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("STEREO", "(\\<{2}.*\\>{2})?"), //
-				new RegexLeaf("[%s]*"), //
-				new RegexLeaf("POSITION", "(right|left|over)[%s]+"), //
-				new RegexLeaf("PARTICIPANT", "(?:of[%s])?([\\p{L}0-9_.@]+|[%g][^%g]+[%g])[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("POSITION", "(right|left|over)"), //
+				RegexLeaf.spaceOneOrMore(), //
+				new RegexLeaf("PARTICIPANT", "(?:of[%s])?([%pLN_.@]+|[%g][^%g]+[%g])"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				color().getRegex(), //
-				new RegexLeaf("URL", "[%s]*(" + UrlBuilder.getRegexp() + ")?"), //
-				new RegexLeaf("[%s]*:[%s]*"), //
-				new RegexLeaf("NOTE", "(.*)"), //
-				new RegexLeaf("$"));
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("URL", "(" + UrlBuilder.getRegexp() + ")?"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf(":"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("NOTE", "(.*)"), RegexLeaf.end());
 	}
 
 	private static ColorParser color() {
@@ -105,11 +117,12 @@ public final class FactorySequenceNoteCommand implements SingleMultiFactoryComma
 
 			@Override
 			public String getPatternEnd() {
-				return "(?i)^end[%s]?(note|hnote|rnote)$";
+				return "^end[%s]?(note|hnote|rnote)$";
 			}
 
-			protected CommandExecutionResult executeNow(final SequenceDiagram system, BlocLines lines) {
-				final RegexResult line0 = getStartingPattern().matcher(StringUtils.trin(lines.getFirst499()));
+			protected CommandExecutionResult executeNow(final SequenceDiagram system, BlocLines lines)
+					throws NoSuchColorException {
+				final RegexResult line0 = getStartingPattern().matcher(lines.getFirst().getTrimmed().getString());
 				lines = lines.subExtract(1, 1);
 				lines = lines.removeEmptyColumns();
 				return executeInternal(system, line0, lines);
@@ -121,36 +134,45 @@ public final class FactorySequenceNoteCommand implements SingleMultiFactoryComma
 		return new SingleLineCommand2<SequenceDiagram>(getRegexConcatSingleLine()) {
 
 			@Override
-			protected CommandExecutionResult executeArg(final SequenceDiagram system, RegexResult arg) {
-				return executeInternal(system, arg, BlocLines.getWithNewlines(arg.get("NOTE", 0)));
+			protected CommandExecutionResult executeArg(final SequenceDiagram diagram, LineLocation location,
+					RegexResult arg) throws NoSuchColorException {
+				return executeInternal(diagram, arg, BlocLines.getWithNewlines(arg.get("NOTE", 0)));
 			}
 
 		};
 	}
 
-	private CommandExecutionResult executeInternal(SequenceDiagram diagram, RegexResult arg, BlocLines strings) {
-		final Participant p = diagram.getOrCreateParticipant(StringUtils
-				.eventuallyRemoveStartingAndEndingDoubleQuote(arg.get("PARTICIPANT", 0)));
+	private CommandExecutionResult executeInternal(SequenceDiagram diagram, RegexResult arg, BlocLines strings)
+			throws NoSuchColorException {
+		final Participant p = diagram.getOrCreateParticipant(
+				StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(arg.get("PARTICIPANT", 0)));
 
 		final NotePosition position = NotePosition.valueOf(StringUtils.goUpperCase(arg.get("POSITION", 0)));
 
 		if (strings.size() > 0) {
 			final boolean tryMerge = arg.get("VMERGE", 0) != null;
-			final Note note = new Note(p, position, strings.toDisplay());
-			Colors colors = color().getColor(arg, diagram.getSkinParam().getIHtmlColorSet());
+			final boolean parallel = arg.get("PARALLEL", 0) != null;
+			final Display display = diagram.manageVariable(strings.toDisplay());
+			final Note note = new Note(p, position, display, diagram.getSkinParam().getCurrentStyleBuilder());
+			Colors colors = color().getColor(diagram.getSkinParam().getThemeStyle(), arg,
+					diagram.getSkinParam().getIHtmlColorSet());
 			final String stereotypeString = arg.get("STEREO", 0);
 			if (stereotypeString != null) {
 				final Stereotype stereotype = new Stereotype(stereotypeString);
 				colors = colors.applyStereotypeForNote(stereotype, diagram.getSkinParam(), FontParam.NOTE,
 						ColorParam.noteBackground, ColorParam.noteBorder);
+				note.setStereotype(stereotype);
 			}
 			note.setColors(colors);
-			note.setStyle(NoteStyle.getNoteStyle(arg.get("STYLE", 0)));
+			note.setNoteStyle(NoteStyle.getNoteStyle(arg.get("STYLE", 0)));
 			if (arg.get("URL", 0) != null) {
 				final UrlBuilder urlBuilder = new UrlBuilder(diagram.getSkinParam().getValue("topurl"), ModeUrl.STRICT);
 				final Url urlLink = urlBuilder.getUrl(arg.get("URL", 0));
 				note.setUrl(urlLink);
 			}
+			if (parallel) {
+				note.goParallel();
+			}
 			diagram.addNote(note, tryMerge);
 		}
 		return CommandExecutionResult.ok();
