@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,16 +36,15 @@
 package net.sourceforge.plantuml.graphic;
 
 import java.awt.geom.Dimension2D;
+import java.awt.geom.Point2D;
 
 import net.sourceforge.plantuml.Dimension2DDouble;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UGraphicStencil;
 import net.sourceforge.plantuml.ugraphic.UPath;
 import net.sourceforge.plantuml.ugraphic.URectangle;
-import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColorNone;
 
 class USymbolStack extends USymbol {
 
@@ -56,32 +55,48 @@ class USymbolStack extends USymbol {
 
 	private void drawQueue(UGraphic ug, double width, double height, boolean shadowing, double roundCorner) {
 		final double border = 15;
-		final URectangle rect = new URectangle(width - 2 * border, height, roundCorner, roundCorner);
-		if (shadowing) {
-			rect.setDeltaShadow(3.0);
-		}
-		ug.apply(new UChangeColor(null)).apply(new UTranslate(border, 0)).draw(rect);
+
+		final URectangle rect = new URectangle(width - 2 * border, height).rounded(roundCorner);
+		ug.apply(new HColorNone()).apply(UTranslate.dx(border)).draw(rect);
+
 		final UPath path = new UPath();
-		path.moveTo(0, 0);
-		path.lineTo(border, 0);
-		path.lineTo(border, height);
-		path.lineTo(width - border, height);
-		path.lineTo(width - border, 0);
-		path.lineTo(width, 0);
-		ug.apply(new UChangeBackColor(null)).draw(path);
+		if (roundCorner == 0) {
+			path.moveTo(0, 0);
+			path.lineTo(border, 0);
+			path.lineTo(border, height);
+			path.lineTo(width - border, height);
+			path.lineTo(width - border, 0);
+			path.lineTo(width, 0);
+		} else {
+			path.moveTo(0, 0);
+			path.lineTo(border - roundCorner / 2, 0);
+			path.arcTo(new Point2D.Double(border, roundCorner / 2), roundCorner / 2, 0, 1);
+			path.lineTo(border, height - roundCorner / 2);
+			path.arcTo(new Point2D.Double(border + roundCorner / 2, height), roundCorner / 2, 0, 0);
+			path.lineTo(width - border - roundCorner / 2, height);
+			path.arcTo(new Point2D.Double(width - border, height - roundCorner / 2), roundCorner / 2, 0, 0);
+			path.lineTo(width - border, roundCorner / 2);
+			path.arcTo(new Point2D.Double(width - border + roundCorner / 2, 0), roundCorner / 2, 0, 1);
+			path.lineTo(width, 0);
+		}
+		if (shadowing) {
+			path.setDeltaShadow(3.0);
+		}
+		ug.apply(new HColorNone().bg()).draw(path);
 	}
 
 	private Margin getMargin() {
 		return new Margin(25, 25, 10, 10);
 	}
 
+	@Override
 	public TextBlock asSmall(TextBlock name, final TextBlock label, final TextBlock stereotype,
-			final SymbolContext symbolContext) {
+			final SymbolContext symbolContext, final HorizontalAlignment stereoAlignment) {
 		return new AbstractTextBlock() {
 
 			public void drawU(UGraphic ug) {
 				final Dimension2D dim = calculateDimension(ug.getStringBounder());
-				ug = UGraphicStencil.create(ug, getRectangleStencil(dim), new UStroke());
+				ug = UGraphicStencil.create(ug, dim);
 				ug = symbolContext.apply(ug);
 				drawQueue(ug, dim.getWidth(), dim.getHeight(), symbolContext.isShadowing(),
 						symbolContext.getRoundCorner());
@@ -98,14 +113,29 @@ class USymbolStack extends USymbol {
 		};
 	}
 
-	public TextBlock asBig(final TextBlock title, final TextBlock stereotype, final double width, final double height,
-			final SymbolContext symbolContext) {
-		throw new UnsupportedOperationException();
-	}
-
 	@Override
-	public boolean manageHorizontalLine() {
-		return true;
+	public TextBlock asBig(final TextBlock title, HorizontalAlignment labelAlignment, final TextBlock stereotype,
+			final double width, final double height, final SymbolContext symbolContext,
+			final HorizontalAlignment stereoAlignment) {
+		return new AbstractTextBlock() {
+
+			public void drawU(UGraphic ug) {
+				final Dimension2D dim = calculateDimension(ug.getStringBounder());
+				ug = symbolContext.apply(ug);
+				drawQueue(ug, dim.getWidth(), dim.getHeight(), symbolContext.isShadowing(),
+						symbolContext.getRoundCorner());
+				final Dimension2D dimStereo = stereotype.calculateDimension(ug.getStringBounder());
+				final double posStereo = (width - dimStereo.getWidth()) / 2;
+				stereotype.drawU(ug.apply(new UTranslate(posStereo, 13)));
+				final Dimension2D dimTitle = title.calculateDimension(ug.getStringBounder());
+				final double posTitle = (width - dimTitle.getWidth()) / 2;
+				title.drawU(ug.apply(new UTranslate(posTitle, 13 + dimStereo.getHeight())));
+			}
+
+			public Dimension2D calculateDimension(StringBounder stringBounder) {
+				return new Dimension2DDouble(width, height);
+			}
+		};
 	}
 
 }
\ No newline at end of file
