@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,6 +36,7 @@
 package net.sourceforge.plantuml.sequencediagram.graphic;
 
 import java.awt.geom.Dimension2D;
+import java.awt.geom.Rectangle2D;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -43,6 +44,7 @@ import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import net.sourceforge.plantuml.Dimension2DDouble;
@@ -50,7 +52,9 @@ import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.LineParam;
 import net.sourceforge.plantuml.SkinParamBackcolored;
 import net.sourceforge.plantuml.Url;
+import net.sourceforge.plantuml.graphic.InnerStrategy;
 import net.sourceforge.plantuml.graphic.StringBounder;
+import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.sequencediagram.Englober;
 import net.sourceforge.plantuml.sequencediagram.Event;
 import net.sourceforge.plantuml.sequencediagram.Newpage;
@@ -61,7 +65,8 @@ import net.sourceforge.plantuml.skin.Component;
 import net.sourceforge.plantuml.skin.ComponentType;
 import net.sourceforge.plantuml.skin.Context2D;
 import net.sourceforge.plantuml.skin.SimpleContext2D;
-import net.sourceforge.plantuml.skin.Skin;
+import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.ugraphic.MinMax;
 import net.sourceforge.plantuml.ugraphic.UClip;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UStroke;
@@ -74,28 +79,22 @@ public class DrawableSet {
 	private final Map<Event, GraphicalElement> events = new HashMap<Event, GraphicalElement>();
 	private final Map<Participant, ParticipantEnglober> participantEnglobers2 = new LinkedHashMap<Participant, ParticipantEnglober>();
 
-	private final List<Event> eventsList = new ArrayList<Event>();
-	private final Skin skin;
+	private final List<Event> eventsList = new ArrayList<>();
+	private final Rose skin;
 	private final ISkinParam skinParam;
 	private Dimension2D dimension;
 	private double topStartingY;
 
-	DrawableSet(Skin skin, ISkinParam skinParam) {
-		if (skin == null) {
-			throw new IllegalArgumentException();
-		}
-		if (skinParam == null) {
-			throw new IllegalArgumentException();
-		}
-		this.skin = skin;
-		this.skinParam = skinParam;
+	DrawableSet(Rose skin, ISkinParam skinParam) {
+		this.skin = Objects.requireNonNull(skin);
+		this.skinParam = Objects.requireNonNull(skinParam);
 	}
 
 	public ParticipantBox getVeryfirst() {
 		return participants.values().iterator().next().getParticipantBox();
 	}
 
-	public final Skin getSkin() {
+	public final Rose getSkin() {
 		return skin;
 	}
 
@@ -116,7 +115,7 @@ public class DrawableSet {
 	}
 
 	public Collection<GraphicalElement> getAllGraphicalElements() {
-		final Collection<GraphicalElement> result = new ArrayList<GraphicalElement>();
+		final Collection<GraphicalElement> result = new ArrayList<>();
 		for (Event ev : eventsList) {
 			result.add(events.get(ev));
 		}
@@ -147,14 +146,14 @@ public class DrawableSet {
 		if (englober == null) {
 			return height;
 		}
-		final Component comp = skin.createComponent(ComponentType.ENGLOBER, null, skinParam, englober
-				.getParticipantEnglober().getTitle());
+		final Component comp = skin.createComponent(englober.getUsedStyles(), ComponentType.ENGLOBER, null, skinParam,
+				englober.getParticipantEnglober().getTitle());
 		final double heightEnglober = comp.getPreferredHeight(stringBounder);
 		return height + heightEnglober;
 	}
 
 	public List<Englober> getExistingParticipantEnglober(StringBounder stringBounder) {
-		final List<Englober> result = new ArrayList<Englober>();
+		final List<Englober> result = new ArrayList<>();
 		Englober pending = null;
 		for (Map.Entry<Participant, ParticipantEnglober> ent : participantEnglobers2.entrySet()) {
 			final ParticipantEnglober englober = ent.getValue();
@@ -167,7 +166,8 @@ public class DrawableSet {
 				pending.add(ent.getKey());
 				continue;
 			}
-			pending = Englober.createPuma(englober, ent.getKey(), getSkinParam(), skin, stringBounder);
+			pending = Englober.createPuma(englober, ent.getKey(), getSkinParam(), skin, stringBounder,
+					skinParam.getCurrentStyleBuilder());
 			result.add(pending);
 		}
 		return Collections.unmodifiableList(result);
@@ -176,7 +176,7 @@ public class DrawableSet {
 	public double getOffsetForEnglobers(StringBounder stringBounder) {
 		double result = 0;
 		for (Englober englober : getExistingParticipantEnglober(stringBounder)) {
-			final Component comp = skin.createComponent(ComponentType.ENGLOBER, null, skinParam, englober
+			final Component comp = skin.createComponent(null, ComponentType.ENGLOBER, null, skinParam, englober
 					.getParticipantEnglober().getTitle());
 			final double height = comp.getPreferredHeight(stringBounder);
 			if (height > result) {
@@ -244,32 +244,57 @@ public class DrawableSet {
 		return dimension;
 	}
 
+	TextBlock asTextBlock(final double delta, final double width, final Page page, final boolean showTail) {
+		return new TextBlock() {
+
+			public void drawU(UGraphic ug) {
+				drawU22(ug, delta, width, page, showTail);
+			}
+
+			public Dimension2D calculateDimension(StringBounder stringBounder) {
+				final double height = page.getHeight();
+				return new Dimension2DDouble(width, height);
+			}
+
+			public MinMax getMinMax(StringBounder stringBounder) {
+				throw new UnsupportedOperationException();
+			}
+
+			public Rectangle2D getInnerPosition(String member, StringBounder stringBounder, InnerStrategy strategy) {
+				throw new UnsupportedOperationException();
+			}
+
+		};
+
+	}
+
 	void drawU22(final UGraphic ug, final double delta, double width, Page page, boolean showTail) {
-		// final UGraphic ugOrig = ug;
 		final double height = page.getHeight();
 
 		final UGraphic ugTranslated = clipAndTranslate2(delta, width, page, ug);
 		final SimpleContext2D context = new SimpleContext2D(true);
 		this.drawEnglobers(ug, height - MARGIN_FOR_ENGLOBERS1, context);
 		this.drawPlaygroundU(ugTranslated, context);
-		// ug = ugOrig;
-
 
 		this.drawLineU22(ug, showTail, page);
 		this.drawHeadTailU(ug, page, showTail ? height - getTailHeight(ug.getStringBounder(), true) : 0);
 
-		// ug = clipAndTranslate2(delta, width, page, ug);
 		this.drawPlaygroundU(ugTranslated, new SimpleContext2D(false));
 	}
 
 	private UGraphic clipAndTranslate2(final double delta, double width, Page p, UGraphic ug) {
 		ug = ug.apply(new UClip(0, p.getBodyRelativePosition(), width, p.getBodyHeight() + 1));
-		if (delta > 0) {
-			ug = ug.apply(new UTranslate(0, -delta));
-		}
+		ug = ug.apply(getTranslate4(delta));
 		return ug;
 	}
 
+	private UTranslate getTranslate4(final double delta) {
+		if (delta > 0) {
+			return UTranslate.dy(-delta);
+		}
+		return new UTranslate();
+	}
+
 	private void drawLineU22(UGraphic ug, boolean showTail, Page page) {
 		// http://plantuml.sourceforge.net/qa/?qa=4826/lifelines-broken-for-txt-seq-diagrams-when-create-is-used
 		final boolean isTxt = ug instanceof UGraphicTxt;
@@ -315,7 +340,7 @@ public class DrawableSet {
 			}
 			box.getParticipantBox().drawHeadTailU(ug, topStartingY, showHead, positionTail);
 			if (url != null) {
-				ug.closeAction();
+				ug.closeUrl();
 			}
 		}
 	}
@@ -343,11 +368,10 @@ public class DrawableSet {
 			double x1 = getX1(englober);
 			final double x2 = getX2(ug.getStringBounder(), englober);
 
-			final Component comp = getEngloberComponent(englober.getParticipantEnglober());
+			final Component comp = getEngloberComponent(englober);
 
 			final double width = x2 - x1;
-			final double preferedWidth = getEngloberPreferedWidth(ug.getStringBounder(),
-					englober.getParticipantEnglober());
+			final double preferedWidth = getEngloberPreferedWidth(ug.getStringBounder(), englober);
 			if (preferedWidth > width) {
 				// if (englober.getFirst2() == englober.getLast2()) {
 				x1 -= (preferedWidth - width) / 2;
@@ -361,14 +385,16 @@ public class DrawableSet {
 		}
 	}
 
-	public double getEngloberPreferedWidth(StringBounder stringBounder, ParticipantEnglober englober) {
+	public double getEngloberPreferedWidth(StringBounder stringBounder, Englober englober) {
 		return getEngloberComponent(englober).getPreferredWidth(stringBounder);
 	}
 
-	private Component getEngloberComponent(ParticipantEnglober englober) {
-		final ISkinParam s = englober.getBoxColor() == null ? skinParam : new SkinParamBackcolored(skinParam,
-				englober.getBoxColor());
-		return skin.createComponent(ComponentType.ENGLOBER, null, s, englober.getTitle());
+	private Component getEngloberComponent(Englober englober) {
+		final ParticipantEnglober participantEnglober = englober.getParticipantEnglober();
+		final ISkinParam s = participantEnglober.getBoxColor() == null ? skinParam : new SkinParamBackcolored(
+				skinParam, participantEnglober.getBoxColor());
+		return skin.createComponent(englober.getUsedStyles(), ComponentType.ENGLOBER, null, s,
+				participantEnglober.getTitle());
 	}
 
 	public double getX1(Englober englober) {
@@ -404,7 +430,7 @@ public class DrawableSet {
 	}
 
 	Participant getFirst(Collection<Participant> someParticipants) {
-		final List<Participant> list = new ArrayList<Participant>(participants.keySet());
+		final List<Participant> list = new ArrayList<>(participants.keySet());
 		int min = -1;
 		for (Participant p : someParticipants) {
 			final int n = list.indexOf(p);
@@ -417,7 +443,7 @@ public class DrawableSet {
 	}
 
 	Participant getLast(Collection<Participant> someParticipants) {
-		final List<Participant> list = new ArrayList<Participant>(participants.keySet());
+		final List<Participant> list = new ArrayList<>(participants.keySet());
 		int max = -1;
 		for (Participant p : someParticipants) {
 			final int n = list.indexOf(p);
