@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -37,7 +37,9 @@ package net.sourceforge.plantuml.sequencediagram.graphic;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Objects;
 
+import net.sourceforge.plantuml.AlignmentParam;
 import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.OptionFlags;
@@ -45,6 +47,7 @@ import net.sourceforge.plantuml.PaddingParam;
 import net.sourceforge.plantuml.SkinParamBackcolored;
 import net.sourceforge.plantuml.SkinParamBackcoloredReference;
 import net.sourceforge.plantuml.cucadiagram.Display;
+import net.sourceforge.plantuml.graphic.HorizontalAlignment;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.sequencediagram.AbstractMessage;
 import net.sourceforge.plantuml.sequencediagram.Delay;
@@ -62,6 +65,7 @@ import net.sourceforge.plantuml.sequencediagram.Message;
 import net.sourceforge.plantuml.sequencediagram.MessageExo;
 import net.sourceforge.plantuml.sequencediagram.Newpage;
 import net.sourceforge.plantuml.sequencediagram.Note;
+import net.sourceforge.plantuml.sequencediagram.NotePosition;
 import net.sourceforge.plantuml.sequencediagram.Notes;
 import net.sourceforge.plantuml.sequencediagram.Participant;
 import net.sourceforge.plantuml.sequencediagram.ParticipantEnglober;
@@ -69,7 +73,8 @@ import net.sourceforge.plantuml.sequencediagram.ParticipantType;
 import net.sourceforge.plantuml.sequencediagram.Reference;
 import net.sourceforge.plantuml.skin.Component;
 import net.sourceforge.plantuml.skin.ComponentType;
-import net.sourceforge.plantuml.skin.Skin;
+import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.style.Style;
 
 class DrawableSetInitializer {
 
@@ -85,7 +90,7 @@ class DrawableSetInitializer {
 
 	private ConstraintSet constraintSet;
 
-	public DrawableSetInitializer(Skin skin, ISkinParam skinParam, boolean showTail, double autonewpage) {
+	public DrawableSetInitializer(Rose skin, ISkinParam skinParam, boolean showTail, double autonewpage) {
 		this.drawableSet = new DrawableSet(skin, skinParam);
 		this.showTail = showTail;
 		this.autonewpage = autonewpage;
@@ -139,8 +144,8 @@ class DrawableSetInitializer {
 			prepareParticipant(stringBounder, p);
 		}
 
-		this.freeY2 = new FrontierStackImpl(drawableSet.getHeadHeight(stringBounder), drawableSet.getAllParticipants()
-				.size());
+		this.freeY2 = new FrontierStackImpl(drawableSet.getHeadHeight(stringBounder),
+				drawableSet.getAllParticipants().size());
 
 		this.lastFreeY2 = this.freeY2;
 
@@ -154,14 +159,14 @@ class DrawableSetInitializer {
 			}
 		}
 
-		final List<ParticipantBox> col = new ArrayList<ParticipantBox>();
+		final List<ParticipantBox> col = new ArrayList<>();
 		for (LivingParticipantBox livingParticipantBox : drawableSet.getAllLivingParticipantBox()) {
 			col.add(livingParticipantBox.getParticipantBox());
 		}
 
 		constraintSet = new ConstraintSet(col, freeX);
 
-		for (Event ev : new ArrayList<Event>(drawableSet.getAllEvents())) {
+		for (Event ev : new ArrayList<>(drawableSet.getAllEvents())) {
 			final ParticipantRange range = getParticipantRange(ev);
 			final double diffY = freeY2.getFreeY(range) - lastFreeY2.getFreeY(range);
 			// final double diffY = freeY2.diff(lastFreeY2);
@@ -203,8 +208,8 @@ class DrawableSetInitializer {
 
 		prepareMissingSpace(stringBounder);
 
-		drawableSet.setDimension(new Dimension2DDouble(freeX, getTotalHeight(
-				freeY2.getFreeY(getFullParticipantRange()), stringBounder)));
+		drawableSet.setDimension(new Dimension2DDouble(freeX,
+				getTotalHeight(freeY2.getFreeY(getFullParticipantRange()), stringBounder)));
 		return drawableSet;
 	}
 
@@ -224,15 +229,14 @@ class DrawableSetInitializer {
 	}
 
 	private void takeParticipantEngloberTitleWidth(StringBounder stringBounder) {
-		for (Englober pe : drawableSet.getExistingParticipantEnglober(stringBounder)) {
-			final double preferredWidth = drawableSet.getEngloberPreferedWidth(stringBounder,
-					pe.getParticipantEnglober());
-			final ParticipantBox first = drawableSet.getLivingParticipantBox(pe.getFirst2TOBEPRIVATE())
+		for (Englober englober : drawableSet.getExistingParticipantEnglober(stringBounder)) {
+			final double preferredWidth = drawableSet.getEngloberPreferedWidth(stringBounder, englober);
+			final ParticipantBox first = drawableSet.getLivingParticipantBox(englober.getFirst2TOBEPRIVATE())
 					.getParticipantBox();
-			final ParticipantBox last = drawableSet.getLivingParticipantBox(pe.getLast2TOBEPRIVATE())
+			final ParticipantBox last = drawableSet.getLivingParticipantBox(englober.getLast2TOBEPRIVATE())
 					.getParticipantBox();
-			final double x1 = drawableSet.getX1(pe);
-			final double x2 = drawableSet.getX2(stringBounder, pe);
+			final double x1 = drawableSet.getX1(englober);
+			final double x2 = drawableSet.getX2(stringBounder, englober);
 			final double missing = preferredWidth - (x2 - x1);
 			if (missing > 0) {
 				constraintSet.pushToLeftParticipantBox(missing / 2, first, true);
@@ -247,10 +251,8 @@ class DrawableSetInitializer {
 	}
 
 	public double getYposition(StringBounder stringBounder, Newpage newpage) {
-		if (newpage == null) {
-			throw new IllegalArgumentException();
-		}
-		final GraphicalNewpage graphicalNewpage = (GraphicalNewpage) drawableSet.getEvent(newpage);
+		final GraphicalNewpage graphicalNewpage = (GraphicalNewpage) drawableSet
+				.getEvent(Objects.requireNonNull(newpage));
 		return graphicalNewpage.getStartingY();
 	}
 
@@ -305,8 +307,8 @@ class DrawableSetInitializer {
 	}
 
 	private void prepareNewpage(StringBounder stringBounder, Newpage newpage, ParticipantRange range) {
-		final GraphicalNewpage graphicalNewpage = new GraphicalNewpage(freeY2.getFreeY(range), drawableSet.getSkin()
-				.createComponent(ComponentType.NEWPAGE, null, drawableSet.getSkinParam(), null));
+		final GraphicalNewpage graphicalNewpage = new GraphicalNewpage(freeY2.getFreeY(range),
+				drawableSet.getSkin().createComponentNewPage(drawableSet.getSkinParam()));
 		this.lastFreeY2 = freeY2;
 		freeY2 = freeY2.add(graphicalNewpage.getPreferredHeight(stringBounder), range);
 		drawableSet.addEvent(newpage, graphicalNewpage);
@@ -314,16 +316,17 @@ class DrawableSetInitializer {
 
 	private void prepareNewpageSpecial(StringBounder stringBounder, Newpage newpage, Event justBefore,
 			ParticipantRange range) {
-		final GraphicalNewpage graphicalNewpage = new GraphicalNewpage(freeY2.getFreeY(range), drawableSet.getSkin()
-				.createComponent(ComponentType.NEWPAGE, null, drawableSet.getSkinParam(), null));
+		final GraphicalNewpage graphicalNewpage = new GraphicalNewpage(freeY2.getFreeY(range),
+				drawableSet.getSkin().createComponentNewPage(drawableSet.getSkinParam()));
 		this.lastFreeY2 = freeY2;
 		freeY2 = freeY2.add(graphicalNewpage.getPreferredHeight(stringBounder), range);
 		drawableSet.addEvent(newpage, graphicalNewpage, justBefore);
 	}
 
 	private void prepareDivider(StringBounder stringBounder, Divider divider, ParticipantRange range) {
-		final GraphicalDivider graphicalDivider = new GraphicalDivider(freeY2.getFreeY(range), drawableSet.getSkin()
-				.createComponent(ComponentType.DIVIDER, null, drawableSet.getSkinParam(), divider.getText()));
+		final GraphicalDivider graphicalDivider = new GraphicalDivider(freeY2.getFreeY(range),
+				drawableSet.getSkin().createComponent(divider.getUsedStyles(), ComponentType.DIVIDER, null,
+						drawableSet.getSkinParam(), divider.getText()));
 		freeY2 = freeY2.add(graphicalDivider.getPreferredHeight(stringBounder), range);
 		drawableSet.addEvent(divider, graphicalDivider);
 	}
@@ -336,8 +339,8 @@ class DrawableSetInitializer {
 
 	private void prepareDelay(StringBounder stringBounder, Delay delay, List<ParticipantBox> participants,
 			ParticipantRange range) {
-		final Component compText = drawableSet.getSkin().createComponent(ComponentType.DELAY_TEXT, null,
-				drawableSet.getSkinParam(), delay.getText());
+		final Component compText = drawableSet.getSkin().createComponent(delay.getUsedStyles(),
+				ComponentType.DELAY_TEXT, null, drawableSet.getSkinParam(), delay.getText());
 		final ParticipantBox first = participants.get(0);
 		final ParticipantBox last = participants.get(participants.size() - 1);
 		final GraphicalDelayText graphicalDivider = new GraphicalDelayText(freeY2.getFreeY(range), compText, first,
@@ -351,32 +354,32 @@ class DrawableSetInitializer {
 
 	final private InGroupablesStack inGroupableStack = new InGroupablesStack();
 
-	private void prepareGroupingStart(StringBounder stringBounder, GroupingStart m, ParticipantRange range) {
-		if (m.getType() != GroupingType.START) {
+	private void prepareGroupingStart(StringBounder stringBounder, GroupingStart start, ParticipantRange range) {
+		if (start.getType() != GroupingType.START) {
 			throw new IllegalStateException();
 		}
-		final ISkinParam skinParam = new SkinParamBackcolored(drawableSet.getSkinParam(), m.getBackColorElement(),
-				m.getBackColorGeneral());
+		final ISkinParam skinParam = new SkinParamBackcolored(drawableSet.getSkinParam(), start.getBackColorElement(),
+				start.getBackColorGeneral());
 
-		final Component comp = drawableSet.getSkin().createComponent(ComponentType.GROUPING_SPACE, null, skinParam,
-				Display.create(m.getComment()));
+		final Component comp = drawableSet.getSkin().createComponent(null, ComponentType.GROUPING_SPACE, null,
+				skinParam, Display.create(start.getComment()));
 		final double preferredHeight = comp.getPreferredHeight(stringBounder);
 		freeY2 = freeY2.add(preferredHeight, range);
 
-		final Display strings = m.getTitle().equals("group") ? Display.create(m.getComment()) : Display.create(
-				m.getTitle(), m.getComment());
-		final Component header = drawableSet.getSkin().createComponent(ComponentType.GROUPING_HEADER, null, skinParam,
-				strings);
+		final Display strings = start.getTitle().equals("group") ? Display.create(start.getComment())
+				: Display.create(start.getTitle(), start.getComment());
+		final Component header = drawableSet.getSkin().createComponent(start.getUsedStyles(),
+				ComponentType.GROUPING_HEADER, null, skinParam, strings);
 		final ParticipantBox veryfirst = drawableSet.getVeryfirst();
-		final InGroupableList inGroupableList = new InGroupableList(veryfirst, m, freeY2.getFreeY(range));
+		final InGroupableList inGroupableList = new InGroupableList(veryfirst, start, freeY2.getFreeY(range));
 		inGroupableStack.addList(inGroupableList);
 
 		final GraphicalElement element = new GroupingGraphicalElementHeader(freeY2.getFreeY(range), header,
-				inGroupableList, m.isParallel());
+				inGroupableList, start.isParallel());
 		inGroupableList.setMinWidth(element.getPreferredWidth(stringBounder));
 		freeY2 = freeY2.add(element.getPreferredHeight(stringBounder), range);
-		drawableSet.addEvent(m, element);
-		if (m.isParallel()) {
+		drawableSet.addEvent(start, element);
+		if (start.isParallel()) {
 			freeY2 = ((FrontierStack) freeY2).openBar();
 		}
 
@@ -384,13 +387,14 @@ class DrawableSetInitializer {
 
 	private void prepareGroupingLeaf(StringBounder stringBounder, final GroupingLeaf m, ParticipantRange range) {
 		final GraphicalElement element;
-		final ISkinParam skinParam = new SkinParamBackcolored(drawableSet.getSkinParam(), null, m.getBackColorGeneral());
+		final ISkinParam skinParam = new SkinParamBackcolored(drawableSet.getSkinParam(), null,
+				m.getBackColorGeneral());
 		if (m.getType() == GroupingType.ELSE) {
 			if (m.isParallel()) {
 				freeY2 = ((FrontierStack) freeY2).restore();
 			}
-			final Component compElse = drawableSet.getSkin().createComponent(ComponentType.GROUPING_ELSE, null,
-					skinParam, Display.create(m.getComment()));
+			final Component compElse = drawableSet.getSkin().createComponent(m.getUsedStyles(),
+					ComponentType.GROUPING_ELSE, null, skinParam, Display.create(m.getComment()));
 			final Lazy lazy = new Lazy() {
 				public double getNow() {
 					final GraphicalElement after = drawableSet.getEvent(m.getJustAfter());
@@ -407,11 +411,11 @@ class DrawableSetInitializer {
 			// MODIF42
 			inGroupableStack.addElement((GroupingGraphicalElementElse) element);
 		} else if (m.getType() == GroupingType.END) {
-			final List<Component> notes = new ArrayList<Component>();
+			final List<Component> notes = new ArrayList<>();
 			for (Note noteOnMessage : m.getNoteOnMessages()) {
 				final ISkinParam sk = noteOnMessage.getSkinParamBackcolored(drawableSet.getSkinParam());
-				final Component note = drawableSet.getSkin().createComponent(
-						noteOnMessage.getStyle().getNoteComponentType(), null, sk, noteOnMessage.getStrings());
+				final Component note = drawableSet.getSkin().createComponentNote(noteOnMessage.getUsedStyles(),
+						noteOnMessage.getNoteStyle().getNoteComponentType(), sk, noteOnMessage.getStrings());
 				notes.add(note);
 			}
 			if (m.isParallel()) {
@@ -425,8 +429,8 @@ class DrawableSetInitializer {
 			}
 			element = new GroupingGraphicalElementTail(freeY2.getFreeY(range),
 					inGroupableStack.getTopGroupingStructure(), m.isParallel());
-			final Component comp = drawableSet.getSkin().createComponent(ComponentType.GROUPING_SPACE, null, skinParam,
-					Display.create(m.getComment()));
+			final Component comp = drawableSet.getSkin().createComponent(null, ComponentType.GROUPING_SPACE, null,
+					skinParam, Display.create(m.getComment()));
 			final double preferredHeight = comp.getPreferredHeight(stringBounder);
 			freeY2 = freeY2.add(preferredHeight, range);
 			inGroupableStack.pop();
@@ -459,9 +463,18 @@ class DrawableSetInitializer {
 			}
 		}
 		final ISkinParam skinParam = n.getSkinParamBackcolored(drawableSet.getSkinParam());
-		final ComponentType type = n.getStyle().getNoteComponentType();
-		final NoteBox noteBox = new NoteBox(freeY2.getFreeY(range), drawableSet.getSkin().createComponent(type, null,
-				skinParam, n.getStrings()), p1, p2, n.getPosition(), n.getUrl());
+		final ComponentType type = n.getNoteStyle().getNoteComponentType();
+		if (p1 == null && p2 == null) {
+			for (LivingParticipantBox p : drawableSet.getAllLivingParticipantBox()) {
+				if (p1 == null) {
+					p1 = p;
+				}
+				p2 = p;
+			}
+		}
+		final Component component = drawableSet.getSkin().createComponentNote(n.getUsedStyles(), type, skinParam,
+				n.getStrings(), n.getPosition());
+		final NoteBox noteBox = new NoteBox(freeY2.getFreeY(range), component, p1, p2, n.getPosition(), n.getUrl());
 		return noteBox;
 	}
 
@@ -470,8 +483,8 @@ class DrawableSetInitializer {
 		for (Note n : notes) {
 			final NoteBox noteBox = createNoteBox(stringBounder, n, range);
 			final ParticipantBox p1 = drawableSet.getLivingParticipantBox(n.getParticipant()).getParticipantBox();
-			final ParticipantBox p2 = n.getParticipant2() == null ? null : drawableSet.getLivingParticipantBox(
-					n.getParticipant2()).getParticipantBox();
+			final ParticipantBox p2 = n.getParticipant2() == null ? null
+					: drawableSet.getLivingParticipantBox(n.getParticipant2()).getParticipantBox();
 			notesBoxes.add(noteBox, p1, p2);
 		}
 		notesBoxes.ensureConstraints(stringBounder, constraintSet);
@@ -512,11 +525,11 @@ class DrawableSetInitializer {
 		}
 
 		if (lifeEvent.getType() == LifeEventType.DESTROY) {
-			final Component comp = drawableSet.getSkin().createComponent(ComponentType.DESTROY, null,
+			final Component comp = drawableSet.getSkin().createComponent(null, ComponentType.DESTROY, null,
 					drawableSet.getSkinParam(), null);
 			final double delta = comp.getPreferredHeight(stringBounder) / 2;
-			final LivingParticipantBox livingParticipantBox = drawableSet.getLivingParticipantBox(lifeEvent
-					.getParticipant());
+			final LivingParticipantBox livingParticipantBox = drawableSet
+					.getLivingParticipantBox(lifeEvent.getParticipant());
 			double pos2 = y;
 			if (message == null) {
 				pos2 = y;
@@ -543,17 +556,18 @@ class DrawableSetInitializer {
 	}
 
 	private void prepareReference(StringBounder stringBounder, Reference reference, ParticipantRange range) {
-		final LivingParticipantBox p1 = drawableSet.getLivingParticipantBox(drawableSet.getFirst(reference
-				.getParticipant()));
-		final LivingParticipantBox p2 = drawableSet.getLivingParticipantBox(drawableSet.getLast(reference
-				.getParticipant()));
+		final LivingParticipantBox p1 = drawableSet
+				.getLivingParticipantBox(drawableSet.getFirst(reference.getParticipant()));
+		final LivingParticipantBox p2 = drawableSet
+				.getLivingParticipantBox(drawableSet.getLast(reference.getParticipant()));
 		final ISkinParam skinParam = new SkinParamBackcoloredReference(drawableSet.getSkinParam(),
 				reference.getBackColorElement(), reference.getBackColorGeneral());
 
 		Display strings = Display.empty();
 		strings = strings.add("ref");
 		strings = strings.addAll(reference.getStrings());
-		final Component comp = drawableSet.getSkin().createComponent(ComponentType.REFERENCE, null, skinParam, strings);
+		final Component comp = drawableSet.getSkin().createComponent(reference.getUsedStyles(), ComponentType.REFERENCE,
+				null, skinParam, strings);
 		final GraphicalReference graphicalReference = new GraphicalReference(freeY2.getFreeY(range), comp, p1, p2,
 				reference.getUrl());
 
@@ -613,20 +627,26 @@ class DrawableSetInitializer {
 
 		final ISkinParam skinParam = p.getSkinParamBackcolored(drawableSet.getSkinParam());
 		final Display participantDisplay = p.getDisplay(skinParam.forceSequenceParticipantUnderlined());
-		final Component head = drawableSet.getSkin().createComponent(headType, null, skinParam, participantDisplay);
-		final Component tail = drawableSet.getSkin().createComponent(tailType, null, skinParam, participantDisplay);
-		final Component line = drawableSet.getSkin().createComponent(this.defaultLineType, null,
+		final Component head = drawableSet.getSkin().createComponent(p.getUsedStyles(), headType, null, skinParam,
+				participantDisplay);
+		final Component tail = drawableSet.getSkin().createComponent(p.getUsedStyles(), tailType, null, skinParam,
+				participantDisplay);
+		final Style style = this.defaultLineType.getDefaultStyleDefinition()
+				.getMergedStyle(skinParam.getCurrentStyleBuilder());
+		final Component line = drawableSet.getSkin().createComponent(new Style[] { style }, this.defaultLineType, null,
 				drawableSet.getSkinParam(), participantDisplay);
-		final Component delayLine = drawableSet.getSkin().createComponent(ComponentType.DELAY_LINE, null,
+		final Component delayLine = drawableSet.getSkin().createComponent(null, ComponentType.DELAY_LINE, null,
 				drawableSet.getSkinParam(), participantDisplay);
 		final ParticipantBox box = new ParticipantBox(head, line, tail, delayLine, this.freeX,
 				skinParam.maxAsciiMessageLength() > 0 ? 1 : 5);
 
-		final Component comp = drawableSet.getSkin().createComponent(ComponentType.ALIVE_BOX_CLOSE_CLOSE, null,
-				drawableSet.getSkinParam(), null);
+		final Component comp = drawableSet.getSkin().createComponent(
+				new Style[] { ComponentType.ALIVE_BOX_CLOSE_CLOSE.getDefaultStyleDefinition()
+						.getMergedStyle(drawableSet.getSkinParam().getCurrentStyleBuilder()) },
+				ComponentType.ALIVE_BOX_CLOSE_CLOSE, null, drawableSet.getSkinParam(), null);
 
-		final LifeLine lifeLine = new LifeLine(box, comp.getPreferredWidth(stringBounder), drawableSet.getSkinParam()
-				.shadowing());
+		final LifeLine lifeLine = new LifeLine(box, comp.getPreferredWidth(stringBounder),
+				drawableSet.getSkinParam().shadowing(p.getStereotype()));
 		drawableSet.setLivingParticipantBox(p, new LivingParticipantBox(box, lifeLine));
 
 		this.freeX = box.getMaxX(stringBounder);
