@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -30,6 +30,7 @@
  *
  *
  * Original Author:  Arnaud Roques
+ * Contribution :  Hisashi Miyashita
  *
  * 
  */
@@ -46,6 +47,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import net.sourceforge.plantuml.AlignmentParam;
@@ -53,48 +55,52 @@ import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.FontParam;
 import net.sourceforge.plantuml.ISkinParam;
-import net.sourceforge.plantuml.LineParam;
 import net.sourceforge.plantuml.SkinParamUtils;
-import net.sourceforge.plantuml.StringUtils;
 import net.sourceforge.plantuml.UmlDiagramType;
 import net.sourceforge.plantuml.Url;
+import net.sourceforge.plantuml.UseStyle;
 import net.sourceforge.plantuml.cucadiagram.EntityPosition;
 import net.sourceforge.plantuml.cucadiagram.EntityUtils;
 import net.sourceforge.plantuml.cucadiagram.IEntity;
 import net.sourceforge.plantuml.cucadiagram.IGroup;
-import net.sourceforge.plantuml.cucadiagram.Member;
-import net.sourceforge.plantuml.cucadiagram.MethodsOrFieldsArea;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.cucadiagram.dot.GraphvizVersion;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
-import net.sourceforge.plantuml.graphic.HtmlColorTransparent;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
-import net.sourceforge.plantuml.graphic.TextBlockEmpty;
-import net.sourceforge.plantuml.graphic.TextBlockWidth;
 import net.sourceforge.plantuml.graphic.USymbol;
 import net.sourceforge.plantuml.graphic.color.ColorType;
 import net.sourceforge.plantuml.graphic.color.Colors;
 import net.sourceforge.plantuml.posimo.Moveable;
 import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.style.PName;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleSignature;
 import net.sourceforge.plantuml.svek.image.EntityImageState;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UComment;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.ULine;
 import net.sourceforge.plantuml.ugraphic.URectangle;
 import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorBackground;
+import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
 import net.sourceforge.plantuml.utils.UniqueSequence;
 
 public class Cluster implements Moveable {
 
-	private final Cluster parent;
+	private static final String RANK_SAME = "same";
+	private static final String RANK_SOURCE = "source";
+	private static final String RANK_SINK = "sink";
+	private static final String ID_EE = "ee";
+	public final static String CENTER_ID = "za";
+
+	private final Cluster parentCluster;
 	private final IGroup group;
-	private final List<Shape> shapes = new ArrayList<Shape>();
-	private final List<Cluster> children = new ArrayList<Cluster>();
+	private final List<SvekNode> nodes = new ArrayList<>();
+	private final List<Cluster> children = new ArrayList<>();
 	private final int color;
 	private final int colorTitle;
 	private final ISkinParam skinParam;
@@ -122,26 +128,27 @@ public class Cluster implements Moveable {
 
 	}
 
-	private boolean hasEntryOrExitPoint() {
-		for (Shape sh : shapes) {
+	private Set<EntityPosition> entityPositionsExceptNormal() {
+		final Set<EntityPosition> result = EnumSet.<EntityPosition>noneOf(EntityPosition.class);
+		for (SvekNode sh : nodes) {
 			if (sh.getEntityPosition() != EntityPosition.NORMAL) {
-				return true;
+				result.add(sh.getEntityPosition());
 			}
 		}
-		return false;
+		return Collections.unmodifiableSet(result);
 	}
 
 	public Cluster(ColorSequence colorSequence, ISkinParam skinParam, IGroup root) {
-		this(null, root, colorSequence, skinParam);
+		this(null, colorSequence, skinParam, root);
 	}
 
 	private ColorParam border;
 
-	private Cluster(Cluster parent, IGroup group, ColorSequence colorSequence, ISkinParam skinParam) {
+	private Cluster(Cluster parentCluster, ColorSequence colorSequence, ISkinParam skinParam, IGroup group) {
 		if (group == null) {
 			throw new IllegalStateException();
 		}
-		this.parent = parent;
+		this.parentCluster = parentCluster;
 		this.group = group;
 		if (group.getUSymbol() != null) {
 			border = group.getUSymbol().getColorParamBorder();
@@ -156,46 +163,43 @@ public class Cluster implements Moveable {
 		return super.toString() + " " + group;
 	}
 
-	public final Cluster getParent() {
-		return parent;
+	public final Cluster getParentCluster() {
+		return parentCluster;
 	}
 
-	public void addShape(Shape sh) {
-		if (sh == null) {
-			throw new IllegalArgumentException();
-		}
-		this.shapes.add(sh);
-		sh.setCluster(this);
+	public void addNode(SvekNode node) {
+		this.nodes.add(Objects.requireNonNull(node));
+		node.setCluster(this);
 	}
 
-	public final List<Shape> getShapes() {
-		return Collections.unmodifiableList(shapes);
+	public final List<SvekNode> getNodes() {
+		return Collections.unmodifiableList(nodes);
 	}
 
-	private List<Shape> getShapesOrderedTop(Collection<Line> lines) {
-		final List<Shape> firsts = new ArrayList<Shape>();
-		final Set<String> tops = new HashSet<String>();
-		final Map<String, Shape> shs = new HashMap<String, Shape>();
+	private List<SvekNode> getNodesOrderedTop(Collection<SvekLine> lines) {
+		final List<SvekNode> firsts = new ArrayList<>();
+		final Set<String> tops = new HashSet<>();
+		final Map<String, SvekNode> shs = new HashMap<String, SvekNode>();
 
-		for (final Iterator<Shape> it = shapes.iterator(); it.hasNext();) {
-			final Shape sh = it.next();
-			shs.put(sh.getUid(), sh);
-			if (sh.isTop() && sh.getEntityPosition() == EntityPosition.NORMAL) {
-				firsts.add(sh);
-				tops.add(sh.getUid());
+		for (final Iterator<SvekNode> it = nodes.iterator(); it.hasNext();) {
+			final SvekNode node = it.next();
+			shs.put(node.getUid(), node);
+			if (node.isTop() && node.getEntityPosition() == EntityPosition.NORMAL) {
+				firsts.add(node);
+				tops.add(node.getUid());
 			}
 		}
 
-		for (Line l : lines) {
+		for (SvekLine l : lines) {
 			if (tops.contains(l.getStartUidPrefix())) {
-				final Shape sh = shs.get(l.getEndUidPrefix());
+				final SvekNode sh = shs.get(l.getEndUidPrefix());
 				if (sh != null && sh.getEntityPosition() == EntityPosition.NORMAL) {
 					firsts.add(0, sh);
 				}
 			}
 
 			if (l.isInverted()) {
-				final Shape sh = shs.get(l.getStartUidPrefix());
+				final SvekNode sh = shs.get(l.getStartUidPrefix());
 				if (sh != null && sh.getEntityPosition() == EntityPosition.NORMAL) {
 					firsts.add(0, sh);
 				}
@@ -205,25 +209,13 @@ public class Cluster implements Moveable {
 		return firsts;
 	}
 
-	private List<Shape> getShapesEntryExit(EnumSet<EntityPosition> positions) {
-		final List<Shape> result = new ArrayList<Shape>();
+	private List<SvekNode> getNodesOrderedWithoutTop(Collection<SvekLine> lines) {
+		final List<SvekNode> all = new ArrayList<>(nodes);
+		final Set<String> tops = new HashSet<>();
+		final Map<String, SvekNode> shs = new HashMap<String, SvekNode>();
 
-		for (final Iterator<Shape> it = shapes.iterator(); it.hasNext();) {
-			final Shape sh = it.next();
-			if (positions.contains(sh.getEntityPosition())) {
-				result.add(sh);
-			}
-		}
-		return result;
-	}
-
-	private List<Shape> getShapesOrderedWithoutTop(Collection<Line> lines) {
-		final List<Shape> all = new ArrayList<Shape>(shapes);
-		final Set<String> tops = new HashSet<String>();
-		final Map<String, Shape> shs = new HashMap<String, Shape>();
-
-		for (final Iterator<Shape> it = all.iterator(); it.hasNext();) {
-			final Shape sh = it.next();
+		for (final Iterator<SvekNode> it = all.iterator(); it.hasNext();) {
+			final SvekNode sh = it.next();
 			if (sh.getEntityPosition() != EntityPosition.NORMAL) {
 				it.remove();
 				continue;
@@ -235,16 +227,16 @@ public class Cluster implements Moveable {
 			}
 		}
 
-		for (Line l : lines) {
+		for (SvekLine l : lines) {
 			if (tops.contains(l.getStartUidPrefix())) {
-				final Shape sh = shs.get(l.getEndUidPrefix());
+				final SvekNode sh = shs.get(l.getEndUidPrefix());
 				if (sh != null) {
 					all.remove(sh);
 				}
 			}
 
 			if (l.isInverted()) {
-				final Shape sh = shs.get(l.getStartUidPrefix());
+				final SvekNode sh = shs.get(l.getStartUidPrefix());
 				if (sh != null) {
 					all.remove(sh);
 				}
@@ -258,9 +250,9 @@ public class Cluster implements Moveable {
 		return Collections.unmodifiableList(children);
 	}
 
-	public Cluster createChild(IGroup g, int titleAndAttributeWidth, int titleAndAttributeHeight, TextBlock title,
-			TextBlock stereo, ColorSequence colorSequence, ISkinParam skinParam) {
-		final Cluster child = new Cluster(this, g, colorSequence, skinParam);
+	public Cluster createChild(int titleAndAttributeWidth, int titleAndAttributeHeight, TextBlock title,
+			TextBlock stereo, ColorSequence colorSequence, ISkinParam skinParam, IGroup g) {
+		final Cluster child = new Cluster(this, colorSequence, skinParam, g);
 		child.titleAndAttributeWidth = titleAndAttributeWidth;
 		child.titleAndAttributeHeight = titleAndAttributeHeight;
 		child.ztitle = title;
@@ -269,8 +261,8 @@ public class Cluster implements Moveable {
 		return child;
 	}
 
-	public final IGroup getGroup() {
-		return group;
+	public final Set<IGroup> getGroups() {
+		return Collections.singleton(group);
 	}
 
 	public final int getTitleAndAttributeWidth() {
@@ -298,26 +290,47 @@ public class Cluster implements Moveable {
 		this.yTitle = y;
 	}
 
-	private static HtmlColor getColor(ColorParam colorParam, ISkinParam skinParam, Stereotype stereotype) {
-		return SkinParamUtils.getColor(skinParam, colorParam, stereotype);
+	private static HColor getColor(ColorParam colorParam, ISkinParam skinParam, Stereotype stereotype) {
+		return SkinParamUtils.getColor(skinParam, stereotype, colorParam);
 	}
 
-	public void drawU(UGraphic ug, UStroke stroke, final UmlDiagramType umlDiagramType, final ISkinParam skinParam2) {
+	static public StyleSignature getDefaultStyleDefinition(SName styleName) {
+		if (styleName == SName.stateDiagram)
+			return StyleSignature.of(SName.root, SName.element, SName.stateDiagram, SName.state, SName.group);
+		return StyleSignature.of(SName.root, SName.element, styleName, SName.group);
+	}
+
+	static public StyleSignature getDefaultStyleDefinitionStateGroup(Stereotype stereotype) {
+		if (stereotype == null)
+			return StyleSignature.of(SName.root, SName.element, SName.stateDiagram, SName.state, SName.group);
+		return StyleSignature.of(SName.root, SName.element, SName.stateDiagram, SName.state, SName.group)
+				.with(stereotype);
+	}
+
+	public void drawU(UGraphic ug, UStroke strokeForState, UmlDiagramType umlDiagramType, ISkinParam skinParam2) {
 		if (group.isHidden()) {
 			return;
 		}
-		final String fullName = group.getCode().getFullName();
+		final String fullName = group.getCodeGetName();
 		if (fullName.startsWith("##") == false) {
 			ug.draw(new UComment("cluster " + fullName));
 		}
-		final Stereotype stereotype = group.getStereotype();
-		HtmlColor borderColor;
-		if (umlDiagramType == UmlDiagramType.STATE) {
-			borderColor = getColor(ColorParam.stateBorder, skinParam, stereotype);
-		} else if (umlDiagramType == UmlDiagramType.ACTIVITY) {
-			borderColor = getColor(ColorParam.packageBorder, skinParam, stereotype);
+		HColor borderColor;
+		Style style = null;
+		if (UseStyle.useBetaStyle()) {
+			style = getDefaultStyleDefinition(umlDiagramType.getStyleName())
+					.getMergedStyle(skinParam.getCurrentStyleBuilder());
+			borderColor = style.value(PName.LineColor).asColor(skinParam2.getThemeStyle(),
+					skinParam2.getIHtmlColorSet());
+
 		} else {
-			borderColor = getColor(ColorParam.packageBorder, skinParam, stereotype);
+			if (umlDiagramType == UmlDiagramType.STATE) {
+				borderColor = getColor(ColorParam.stateBorder, skinParam, group.getStereotype());
+			} else if (umlDiagramType == UmlDiagramType.ACTIVITY) {
+				borderColor = getColor(ColorParam.packageBorder, skinParam, group.getStereotype());
+			} else {
+				borderColor = getColor(ColorParam.packageBorder, skinParam, group.getStereotype());
+			}
 		}
 
 		final Url url = group.getUrl99();
@@ -325,7 +338,7 @@ public class Cluster implements Moveable {
 			ug.startUrl(url);
 		}
 		try {
-			if (hasEntryOrExitPoint()) {
+			if (entityPositionsExceptNormal().size() > 0) {
 				manageEntryExitPoint(ug.getStringBounder());
 			}
 			if (skinParam.useSwimlanes(umlDiagramType)) {
@@ -335,74 +348,81 @@ public class Cluster implements Moveable {
 			final boolean isState = umlDiagramType == UmlDiagramType.STATE;
 			if (isState) {
 				if (group.getColors(skinParam).getSpecificLineStroke() != null) {
-					stroke = group.getColors(skinParam).getSpecificLineStroke();
+					strokeForState = group.getColors(skinParam).getSpecificLineStroke();
 				}
 				if (group.getColors(skinParam).getColor(ColorType.LINE) != null) {
 					borderColor = group.getColors(skinParam).getColor(ColorType.LINE);
 				}
-				drawUState(ug, borderColor, skinParam2, stroke, umlDiagramType);
+				drawUState(ug, borderColor, skinParam2, strokeForState, umlDiagramType);
 				return;
 			}
-			PackageStyle style = group.getPackageStyle();
-			if (style == null) {
-				style = skinParam2.getPackageStyle();
+			PackageStyle packageStyle = group.getPackageStyle();
+			if (packageStyle == null) {
+				packageStyle = skinParam2.packageStyle();
 			}
 			if (border != null) {
-				final HtmlColor tmp = skinParam2.getHtmlColor(border, group.getStereotype(), false);
+				final HColor tmp = skinParam2.getHtmlColor(border, group.getStereotype(), false);
 				if (tmp != null) {
 					borderColor = tmp;
 				}
 			}
 
-			final boolean shadowing = group.getUSymbol() == null ? skinParam2.shadowing() : skinParam2.shadowing2(group
-					.getUSymbol().getSkinParameter());
+			final double shadowing;
+			final UStroke stroke;
+			if (UseStyle.useBetaStyle()) {
+				shadowing = style.value(PName.Shadowing).asDouble();
+				stroke = style.getStroke();
+			} else {
+				if (group.getUSymbol() == null) {
+					shadowing = skinParam2.shadowing2(group.getStereotype(), USymbol.PACKAGE.getSkinParameter()) ? 3
+							: 0;
+				} else {
+					shadowing = skinParam2.shadowing2(group.getStereotype(), group.getUSymbol().getSkinParameter()) ? 3
+							: 0;
+				}
+				stroke = getStrokeInternal(group, skinParam2);
+			}
+			HColor backColor = getBackColor(umlDiagramType, style);
+			backColor = getBackColor(backColor, skinParam2, group.getStereotype(), umlDiagramType.getStyleName());
 			if (ztitle != null || zstereo != null) {
-				final HtmlColor back = getBackColor(getBackColor(umlDiagramType), skinParam2, group.getStereotype());
-				final double roundCorner = group.getUSymbol() == null ? 0 : group.getUSymbol().getSkinParameter()
-						.getRoundCorner(skinParam, stereotype);
+				final double roundCorner = group.getUSymbol() == null ? 0
+						: group.getUSymbol().getSkinParameter().getRoundCorner(skinParam, group.getStereotype());
 
-				final UStroke stroke2 = getStrokeInternal(skinParam2);
-				final ClusterDecoration decoration = new ClusterDecoration(style, group.getUSymbol(), ztitle, zstereo,
-						minX, minY, maxX, maxY, stroke2);
-				decoration.drawU(ug, back, borderColor, shadowing, roundCorner,
-						skinParam2.getHorizontalAlignment(AlignmentParam.packageTitleAlignment, null));
+				final ClusterDecoration decoration = new ClusterDecoration(packageStyle, group.getUSymbol(), ztitle,
+						zstereo, minX, minY, maxX, maxY, stroke);
+				decoration.drawU(ug, backColor, borderColor, shadowing, roundCorner,
+						skinParam2.getHorizontalAlignment(AlignmentParam.packageTitleAlignment, null, false, null),
+						skinParam2.getStereotypeAlignment());
 				return;
 			}
 			final URectangle rect = new URectangle(maxX - minX, maxY - minY);
-			if (shadowing) {
-				rect.setDeltaShadow(3.0);
-			}
-			final HtmlColor backColor = getBackColor(getBackColor(umlDiagramType), skinParam2, group.getStereotype());
-			ug = ug.apply(new UChangeBackColor(backColor)).apply(new UChangeColor(borderColor));
+			rect.setDeltaShadow(shadowing);
+			ug = ug.apply(backColor.bg()).apply(borderColor);
 			ug.apply(new UStroke(2)).apply(new UTranslate(minX, minY)).draw(rect);
 
 		} finally {
 			if (url != null) {
-				ug.closeAction();
+				ug.closeUrl();
 			}
 		}
 
 	}
 
-	private UStroke getStrokeInternal(ISkinParam skinParam) {
+	static public UStroke getStrokeInternal(IGroup group, ISkinParam skinParam) {
 		final Colors colors = group.getColors(skinParam);
 		if (colors.getSpecificLineStroke() != null) {
 			return colors.getSpecificLineStroke();
 		}
-		if (group.getUSymbol() != null) {
+		if (group.getUSymbol() != null && group.getUSymbol() != USymbol.PACKAGE) {
 			return group.getUSymbol().getSkinParameter().getStroke(skinParam, group.getStereotype());
 		}
-		UStroke stroke = skinParam.getThickness(LineParam.packageBorder, group.getStereotype());
-		if (stroke == null) {
-			stroke = new UStroke(1.5);
-		}
-		return stroke;
+		return GeneralImageBuilder.getForcedStroke(group.getStereotype(), skinParam);
 	}
 
 	public void manageEntryExitPoint(StringBounder stringBounder) {
-		final Collection<ClusterPosition> insides = new ArrayList<ClusterPosition>();
-		final List<Point2D> points = new ArrayList<Point2D>();
-		for (Shape sh : shapes) {
+		final Collection<ClusterPosition> insides = new ArrayList<>();
+		final List<Point2D> points = new ArrayList<>();
+		for (SvekNode sh : nodes) {
 			if (sh.getEntityPosition() == EntityPosition.NORMAL) {
 				insides.add(sh.getClusterPosition());
 			} else {
@@ -427,22 +447,22 @@ public class Cluster implements Moveable {
 		xTitle = minX + ((maxX - minX - widthTitle) / 2);
 	}
 
-	private void drawSwinLinesState(UGraphic ug, HtmlColor borderColor) {
+	private void drawSwinLinesState(UGraphic ug, HColor borderColor) {
 		if (ztitle != null) {
-			ztitle.drawU(ug.apply(new UTranslate(xTitle, 0)));
+			ztitle.drawU(ug.apply(UTranslate.dx(xTitle)));
 		}
-		final ULine line = new ULine(0, maxY - minY);
-		ug = ug.apply(new UChangeColor(borderColor));
-		ug.apply(new UTranslate(minX, 0)).draw(line);
-		ug.apply(new UTranslate(maxX, 0)).draw(line);
+		final ULine line = ULine.vline(maxY - minY);
+		ug = ug.apply(borderColor);
+		ug.apply(UTranslate.dx(minX)).draw(line);
+		ug.apply(UTranslate.dx(maxX)).draw(line);
 
 	}
 
-	private HtmlColor getColor(ISkinParam skinParam, ColorParam colorParam, Stereotype stereo) {
-		return new Rose().getHtmlColor(skinParam, colorParam, stereo);
+	private HColor getColorLegacy(ISkinParam skinParam, ColorParam colorParam, Stereotype stereo) {
+		return new Rose().getHtmlColor(skinParam, stereo, colorParam);
 	}
 
-	private void drawUState(UGraphic ug, HtmlColor borderColor, ISkinParam skinParam2, UStroke stroke,
+	private void drawUState(UGraphic ug, HColor borderColor, ISkinParam skinParam2, UStroke stroke,
 			UmlDiagramType umlDiagramType) {
 		final Dimension2D total = new Dimension2DDouble(maxX - minX, maxY - minY);
 		final double suppY;
@@ -453,56 +473,58 @@ public class Cluster implements Moveable {
 					+ IEntityImage.MARGIN_LINE;
 		}
 
-		HtmlColor stateBack = getBackColor(umlDiagramType);
+		final Style styleGroup = getDefaultStyleDefinitionStateGroup(group.getStereotype())
+				.getMergedStyle(skinParam.getCurrentStyleBuilder());
+
+		HColor stateBack = getBackColor(umlDiagramType, styleGroup);
 		if (stateBack == null) {
-			stateBack = getColor(skinParam2, ColorParam.stateBackground, group.getStereotype());
+			stateBack = getColorLegacy(skinParam2, ColorParam.stateBackground, group.getStereotype());
 		}
-		final HtmlColor background = getColor(skinParam2, ColorParam.background, null);
-		final TextBlockWidth attribute = getTextBlockAttribute(skinParam2);
+		final HColor background = getColorLegacy(skinParam2, ColorParam.background, null);
+
+		// final Style style = getStyle(FontParam.STATE_ATTRIBUTE, skinParam2);
+
+		final TextBlock attribute = GeneralImageBuilder.stateHeader(group, styleGroup, skinParam2);
 		final double attributeHeight = attribute.calculateDimension(ug.getStringBounder()).getHeight();
-		final RoundedContainer r = new RoundedContainer(total, suppY, attributeHeight
-				+ (attributeHeight > 0 ? IEntityImage.MARGIN : 0), borderColor, stateBack, background, stroke);
-		r.drawU(ug.apply(new UTranslate(minX, minY)), skinParam2.shadowing());
+		if (total.getWidth() == 0) {
+			System.err.println("Cluster::drawUState issue");
+			return;
+		}
+		final RoundedContainer r = new RoundedContainer(total, suppY,
+				attributeHeight + (attributeHeight > 0 ? IEntityImage.MARGIN : 0), borderColor, stateBack, background,
+				stroke);
+		r.drawU(ug.apply(new UTranslate(minX, minY)), skinParam2.shadowing(group.getStereotype()));
 
 		if (ztitle != null) {
 			ztitle.drawU(ug.apply(new UTranslate(xTitle, yTitle)));
 		}
 
 		if (attributeHeight > 0) {
-			attribute.asTextBlock(total.getWidth()).drawU(
+			attribute.drawU(
 					ug.apply(new UTranslate(minX + IEntityImage.MARGIN, minY + suppY + IEntityImage.MARGIN / 2.0)));
 		}
 
 		final Stereotype stereotype = group.getStereotype();
 		final boolean withSymbol = stereotype != null && stereotype.isWithOOSymbol();
 		if (withSymbol) {
-			EntityImageState.drawSymbol(ug.apply(new UChangeColor(borderColor)), maxX, maxY);
+			EntityImageState.drawSymbol(ug.apply(borderColor), maxX, maxY);
 		}
 
 	}
 
-	private TextBlockWidth getTextBlockAttribute(ISkinParam skinParam) {
-		final TextBlockWidth attribute;
-		final List<Member> members = group.getBodier().getFieldsToDisplay();
-		if (members.size() == 0) {
-			attribute = new TextBlockEmpty();
-		} else {
-			attribute = new MethodsOrFieldsArea(members, FontParam.STATE_ATTRIBUTE, skinParam, group.getStereotype(),
-					null);
-		}
-		return attribute;
-	}
-
 	public void setPosition(double minX, double minY, double maxX, double maxY) {
 		this.minX = minX;
 		this.maxX = maxX;
 		this.minY = minY;
 		this.maxY = maxY;
-
 	}
 
-	private boolean isThereALinkFromOrToGroup(Collection<Line> lines) {
-		for (Line line : lines) {
+	private Style getStyle(FontParam fontParam, ISkinParam skinParam) {
+		return fontParam.getStyleDefinition(SName.stateDiagram).getMergedStyle(skinParam.getCurrentStyleBuilder());
+	}
+
+	private boolean isThereALinkFromOrToGroup(Collection<SvekLine> lines) {
+		for (SvekLine line : lines) {
 			if (line.isLinkFromOrTo(group)) {
 				return true;
 			}
@@ -510,26 +532,27 @@ public class Cluster implements Moveable {
 		return false;
 	}
 
-	public void printCluster1(StringBuilder sb, Collection<Line> lines, StringBounder stringBounder) {
-		for (Shape sh : getShapesOrderedTop(lines)) {
-			sh.appendShape(sb, stringBounder);
+	public void printCluster1(StringBuilder sb, Collection<SvekLine> lines, StringBounder stringBounder) {
+		for (SvekNode node : getNodesOrderedTop(lines)) {
+			node.appendShape(sb, stringBounder);
 		}
 	}
 
-	private List<IShapePseudo> addProtection(List<Shape> entries, double width) {
-		final List<IShapePseudo> result = new ArrayList<IShapePseudo>();
+	private List<IShapePseudo> addProtection(List<? extends IShapePseudo> entries, double width) {
+		final List<IShapePseudo> result = new ArrayList<>();
 		result.add(entries.get(0));
 		for (int i = 1; i < entries.size(); i++) {
+			// Pseudo space for the label
 			result.add(new ShapePseudoImpl("psd" + UniqueSequence.getValue(), width, 5));
 			result.add(entries.get(i));
 		}
 		return result;
 	}
 
-	private double getMaxWidthFromLabelForEntryExit(List<Shape> entries, StringBounder stringBounder) {
+	private double getMaxWidthFromLabelForEntryExit(List<? extends IShapePseudo> entries, StringBounder stringBounder) {
 		double result = -Double.MAX_VALUE;
-		for (Shape shape : entries) {
-			final double w = getMaxWidthFromLabelForEntryExit(shape, stringBounder);
+		for (IShapePseudo node : entries) {
+			final double w = getMaxWidthFromLabelForEntryExit(node, stringBounder);
 			if (w > result) {
 				result = w;
 			}
@@ -537,51 +560,77 @@ public class Cluster implements Moveable {
 		return result;
 	}
 
-	private double getMaxWidthFromLabelForEntryExit(Shape shape, StringBounder stringBounder) {
-		return shape.getMaxWidthFromLabelForEntryExit(stringBounder);
+	private double getMaxWidthFromLabelForEntryExit(IShapePseudo node, StringBounder stringBounder) {
+		return node.getMaxWidthFromLabelForEntryExit(stringBounder);
 	}
 
-	public void printClusterEntryExit(StringBuilder sb, StringBounder stringBounder) {
-		final List<Shape> shapesEntryExitList = getShapesEntryExit(EntityPosition.getInputs());
-		final double maxWith = getMaxWidthFromLabelForEntryExit(shapesEntryExitList, stringBounder);
-		final double naturalSpace = 70;
-		final List<? extends IShapePseudo> entries;
-		if (maxWith > naturalSpace) {
-			entries = addProtection(shapesEntryExitList, maxWith - naturalSpace);
-		} else {
-			entries = shapesEntryExitList;
-		}
+	private void printRanks(String rank, List<? extends IShapePseudo> entries, StringBuilder sb,
+			StringBounder stringBounder) {
 		if (entries.size() > 0) {
-			sb.append("{rank=source;");
-			for (IShapePseudo sh : entries) {
-				sb.append(sh.getUid() + ";");
+			sb.append("{rank=" + rank + ";");
+			for (IShapePseudo sh1 : entries) {
+				sb.append(sh1.getUid() + ";");
 			}
 			sb.append("}");
-			for (IShapePseudo sh : entries) {
-				sh.appendShape(sb, stringBounder);
+			SvekUtils.println(sb);
+			for (IShapePseudo sh2 : entries) {
+				sh2.appendShape(sb, stringBounder);
 			}
-		}
-		final List<Shape> exits = getShapesEntryExit(EntityPosition.getOutputs());
-		if (exits.size() > 0) {
-			sb.append("{rank=sink;");
-			for (Shape sh : exits) {
-				sb.append(sh.getUid() + ";");
-			}
-			sb.append("}");
-			for (Shape sh : exits) {
-				sh.appendShape(sb, stringBounder);
+			SvekUtils.println(sb);
+			if (hasPort()) {
+				boolean arrow = false;
+				String node = null;
+				for (IShapePseudo sh : entries) {
+					if (arrow) {
+						sb.append("->");
+					}
+					arrow = true;
+					node = sh.getUid();
+					sb.append(node);
+				}
+				sb.append(';');
+				SvekUtils.println(sb);
+				sb.append(node + "->" + empty() + ";");
+				SvekUtils.println(sb);
 			}
 		}
 	}
 
-	public boolean printCluster2(StringBuilder sb, Collection<Line> lines, StringBounder stringBounder,
-			DotMode dotMode, GraphvizVersion graphvizVersion, UmlDiagramType type) {
-		// Log.println("Cluster::printCluster " + this);
+	private List<? extends IShapePseudo> withPositionProtected(StringBounder stringBounder,
+			Set<EntityPosition> targets) {
+		final List<SvekNode> result = withPosition(targets);
+		final double maxWith = getMaxWidthFromLabelForEntryExit(result, stringBounder);
+		final double naturalSpace = 70;
+		if (maxWith > naturalSpace) {
+			return addProtection(result, maxWith - naturalSpace);
+		}
+		return result;
+	}
 
-		boolean added = false;
-		for (Shape sh : getShapesOrderedWithoutTop(lines)) {
-			sh.appendShape(sb, stringBounder);
-			added = true;
+	private List<SvekNode> withPosition(Set<EntityPosition> positions) {
+		final List<SvekNode> result = new ArrayList<>();
+		for (final Iterator<SvekNode> it = nodes.iterator(); it.hasNext();) {
+			final SvekNode sh = it.next();
+			if (positions.contains(sh.getEntityPosition())) {
+				result.add(sh);
+			}
+		}
+		return result;
+	}
+
+	private void printClusterEntryExit(StringBuilder sb, StringBounder stringBounder) {
+		printRanks(RANK_SOURCE, withPositionProtected(stringBounder, EntityPosition.getInputs()), sb, stringBounder);
+		printRanks(RANK_SAME, withPositionProtected(stringBounder, EntityPosition.getSame()), sb, stringBounder);
+		printRanks(RANK_SINK, withPositionProtected(stringBounder, EntityPosition.getOutputs()), sb, stringBounder);
+	}
+
+	public SvekNode printCluster2(StringBuilder sb, Collection<SvekLine> lines, StringBounder stringBounder,
+			DotMode dotMode, GraphvizVersion graphvizVersion, UmlDiagramType type) {
+
+		SvekNode added = null;
+		for (SvekNode node : getNodesOrderedWithoutTop(lines)) {
+			node.appendShape(sb, stringBounder);
+			added = node;
 		}
 
 		if (skinParam.useRankSame() && dotMode != DotMode.NO_LEFT_RIGHT_AND_XLABEL
@@ -596,16 +645,16 @@ public class Cluster implements Moveable {
 		return added;
 	}
 
-	private void appendRankSame(StringBuilder sb, Collection<Line> lines) {
+	private void appendRankSame(StringBuilder sb, Collection<SvekLine> lines) {
 		for (String same : getRankSame(lines)) {
 			sb.append(same);
 			SvekUtils.println(sb);
 		}
 	}
 
-	private Set<String> getRankSame(Collection<Line> lines) {
-		final Set<String> rankSame = new HashSet<String>();
-		for (Line l : lines) {
+	private Set<String> getRankSame(Collection<SvekLine> lines) {
+		final Set<String> rankSame = new HashSet<>();
+		for (SvekLine l : lines) {
 			if (l.hasEntryPoint()) {
 				continue;
 			}
@@ -622,7 +671,7 @@ public class Cluster implements Moveable {
 	}
 
 	public void fillRankMin(Set<String> rankMin) {
-		for (Shape sh : getShapes()) {
+		for (SvekNode sh : getNodes()) {
 			if (sh.isTop()) {
 				rankMin.add(sh.getUid());
 			}
@@ -634,8 +683,8 @@ public class Cluster implements Moveable {
 	}
 
 	private boolean isInCluster(String uid) {
-		for (Shape sh : shapes) {
-			if (sh.getUid().equals(uid)) {
+		for (SvekNode node : nodes) {
+			if (node.getUid().equals(uid)) {
 				return true;
 			}
 		}
@@ -650,8 +699,6 @@ public class Cluster implements Moveable {
 		return CENTER_ID + group.getUid();
 	}
 
-	public final static String CENTER_ID = "za";
-
 	private boolean protection0(UmlDiagramType type) {
 		if (skinParam.useSwimlanes(type)) {
 			return false;
@@ -697,8 +744,8 @@ public class Cluster implements Moveable {
 		return null;
 	}
 
-	private void printInternal(StringBuilder sb, Collection<Line> lines, StringBounder stringBounder, DotMode dotMode,
-			GraphvizVersion graphvizVersion, UmlDiagramType type) {
+	private void printInternal(StringBuilder sb, Collection<SvekLine> lines, StringBounder stringBounder,
+			DotMode dotMode, GraphvizVersion graphvizVersion, UmlDiagramType type) {
 		final boolean thereALinkFromOrToGroup2 = isThereALinkFromOrToGroup(lines);
 		boolean thereALinkFromOrToGroup1 = thereALinkFromOrToGroup2;
 		final boolean useProtectionWhenThereALinkFromOrToGroup = graphvizVersion
@@ -708,11 +755,11 @@ public class Cluster implements Moveable {
 		}
 		// final boolean thereALinkFromOrToGroup1 = false;
 		if (thereALinkFromOrToGroup1) {
-			subgraphCluster(sb, "a");
+			subgraphClusterNoLabel(sb, "a");
 		}
-		final boolean hasEntryOrExitPoint = hasEntryOrExitPoint();
-		if (hasEntryOrExitPoint) {
-			for (Line line : lines) {
+		final Set<EntityPosition> entityPositionsExceptNormal = entityPositionsExceptNormal();
+		if (entityPositionsExceptNormal.size() > 0) {
+			for (SvekLine line : lines) {
 				if (line.isLinkFromOrTo(group)) {
 					line.setProjectionCluster(this);
 				}
@@ -720,7 +767,7 @@ public class Cluster implements Moveable {
 		}
 		boolean protection0 = protection0(type);
 		boolean protection1 = protection1(type);
-		if (hasEntryOrExitPoint || useProtectionWhenThereALinkFromOrToGroup == false) {
+		if (entityPositionsExceptNormal.size() > 0 || useProtectionWhenThereALinkFromOrToGroup == false) {
 			protection0 = false;
 			protection1 = false;
 		}
@@ -729,46 +776,45 @@ public class Cluster implements Moveable {
 		// protection1 = false;
 		// }
 		if (protection0) {
-			subgraphCluster(sb, "p0");
+			subgraphClusterNoLabel(sb, "p0");
 		}
 		sb.append("subgraph " + getClusterId() + " {");
 		sb.append("style=solid;");
-		sb.append("color=\"" + StringUtils.getAsHtml(color) + "\";");
+		sb.append("color=\"" + DotStringFactory.sharp000000(color) + "\";");
 
 		final String label;
 		if (isLabel()) {
 			final StringBuilder sblabel = new StringBuilder("<");
-			Line.appendTable(sblabel, getTitleAndAttributeWidth(), getTitleAndAttributeHeight() - 5, colorTitle);
+			SvekLine.appendTable(sblabel, getTitleAndAttributeWidth(), getTitleAndAttributeHeight() - 5, colorTitle);
 			sblabel.append(">");
 			label = sblabel.toString();
-			final HorizontalAlignment align = skinParam
-					.getHorizontalAlignment(AlignmentParam.packageTitleAlignment, null);
+			final HorizontalAlignment align = skinParam.getHorizontalAlignment(AlignmentParam.packageTitleAlignment,
+					null, false, null);
 			sb.append("labeljust=\"" + align.getGraphVizValue() + "\";");
 		} else {
 			label = "\"\"";
 		}
 
-		if (hasEntryOrExitPoint) {
+		if (entityPositionsExceptNormal.size() > 0) {
 			printClusterEntryExit(sb, stringBounder);
-			subgraphCluster(sb, "ee", label);
+			if (hasPort()) {
+				subgraphClusterNoLabel(sb, ID_EE);
+			} else {
+				subgraphClusterWithLabel(sb, ID_EE, label);
+			}
 		} else {
 			sb.append("label=" + label + ";");
 			SvekUtils.println(sb);
 		}
 
-		// if (hasEntryOrExitPoint) {
-		// printClusterEntryExit(sb);
-		// subgraphCluster(sb, "ee");
-		// }
-
 		if (thereALinkFromOrToGroup2) {
 			sb.append(getSpecialPointId(group) + " [shape=point,width=.01,label=\"\"];");
 		}
 		if (thereALinkFromOrToGroup1) {
-			subgraphCluster(sb, "i");
+			subgraphClusterNoLabel(sb, "i");
 		}
 		if (protection1) {
-			subgraphCluster(sb, "p1");
+			subgraphClusterNoLabel(sb, "p1");
 		}
 		if (skinParam.useSwimlanes(type)) {
 			sb.append("{rank = source; ");
@@ -786,11 +832,19 @@ public class Cluster implements Moveable {
 		}
 		SvekUtils.println(sb);
 		printCluster1(sb, lines, stringBounder);
-		final boolean added = printCluster2(sb, lines, stringBounder, dotMode, graphvizVersion, type);
-		if (hasEntryOrExitPoint && added == false) {
-			final String empty = "empty" + color;
-			sb.append(empty + " [shape=point,width=.01,label=\"\"];");
+
+		final SvekNode added = printCluster2(sb, lines, stringBounder, dotMode, graphvizVersion, type);
+		if (entityPositionsExceptNormal.size() > 0) {
+			if (hasPort()) {
+				sb.append(empty() + " [shape=rect,width=.01,height=.01,label=");
+				sb.append(label);
+				sb.append("];");
+			} else if (added == null) {
+				sb.append(empty() + " [shape=point,width=.01,label=\"\"];");
+			}
+			SvekUtils.println(sb);
 		}
+
 		sb.append("}");
 		if (protection1) {
 			sb.append("}");
@@ -799,7 +853,7 @@ public class Cluster implements Moveable {
 			sb.append("}");
 			sb.append("}");
 		}
-		if (hasEntryOrExitPoint) {
+		if (entityPositionsExceptNormal.size() > 0) {
 			sb.append("}");
 		}
 		if (protection0) {
@@ -808,17 +862,33 @@ public class Cluster implements Moveable {
 		SvekUtils.println(sb);
 	}
 
+	private boolean hasPort() {
+		for (EntityPosition pos : entityPositionsExceptNormal()) {
+			if (pos.isPort()) {
+				return true;
+			}
+		}
+		return false;
+	}
+
+	private String empty() {
+		// return "empty" + color;
+		// We use the same node with one for thereALinkFromOrToGroup2 as an empty
+		// because we cannot put a new node in the nested inside of the cluster
+		// if thereALinkFromOrToGroup2 is enabled.
+		return getSpecialPointId(group);
+	}
+
 	public boolean isLabel() {
 		return getTitleAndAttributeHeight() > 0 && getTitleAndAttributeWidth() > 0;
 	}
 
-	private void subgraphCluster(StringBuilder sb, String id) {
-		subgraphCluster(sb, id, "\"\"");
+	private void subgraphClusterNoLabel(StringBuilder sb, String id) {
+		subgraphClusterWithLabel(sb, id, "\"\"");
 	}
 
-	private void subgraphCluster(StringBuilder sb, String id, String label) {
-		final String uid = getClusterId() + id;
-		sb.append("subgraph " + uid + " {");
+	private void subgraphClusterWithLabel(StringBuilder sb, String id, String label) {
+		sb.append("subgraph " + getClusterId() + id + " {");
 		sb.append("label=" + label + ";");
 	}
 
@@ -830,26 +900,31 @@ public class Cluster implements Moveable {
 		return colorTitle;
 	}
 
-	private final HtmlColor getBackColor(final UmlDiagramType umlDiagramType) {
+	private final HColor getBackColor(UmlDiagramType umlDiagramType, Style style) {
 		if (EntityUtils.groupRoot(group)) {
 			return null;
 		}
-		final HtmlColor result = group.getColors(skinParam).getColor(ColorType.BACK);
+		final HColor result = group.getColors(skinParam).getColor(ColorType.BACK);
 		if (result != null) {
 			return result;
 		}
 		final Stereotype stereo = group.getStereotype();
+
+		if (UseStyle.useBetaStyle()) {
+			return style.value(PName.BackGroundColor).asColor(skinParam.getThemeStyle(), skinParam.getIHtmlColorSet());
+		}
+
 		final USymbol sym = group.getUSymbol() == null ? USymbol.PACKAGE : group.getUSymbol();
-		final ColorParam backparam = umlDiagramType == UmlDiagramType.ACTIVITY ? ColorParam.partitionBackground : sym
-				.getColorParamBack();
-		final HtmlColor c1 = skinParam.getHtmlColor(backparam, stereo, false);
+		final ColorParam backparam = umlDiagramType == UmlDiagramType.ACTIVITY ? ColorParam.partitionBackground
+				: sym.getColorParamBack();
+		final HColor c1 = skinParam.getHtmlColor(backparam, stereo, false);
 		if (c1 != null) {
 			return c1;
 		}
-		if (parent == null) {
+		if (parentCluster == null) {
 			return null;
 		}
-		return parent.getBackColor(umlDiagramType);
+		return parentCluster.getBackColor(umlDiagramType, style);
 	}
 
 	public boolean isClusterOf(IEntity ent) {
@@ -859,15 +934,28 @@ public class Cluster implements Moveable {
 		return group == ent;
 	}
 
-	public static HtmlColor getBackColor(HtmlColor backColor, ISkinParam skinParam, Stereotype stereotype) {
+	public static HColor getBackColor(HColor backColor, ISkinParam skinParam, Stereotype stereotype, SName styleName) {
+		if (UseStyle.useBetaStyle()) {
+			final Style style = getDefaultStyleDefinition(styleName).getMergedStyle(skinParam.getCurrentStyleBuilder());
+			if (backColor == null) {
+				backColor = style.value(PName.BackGroundColor).asColor(skinParam.getThemeStyle(),
+						skinParam.getIHtmlColorSet());
+			}
+			if (backColor == null || backColor.equals(HColorUtils.transparent())) {
+				backColor = new HColorBackground(skinParam.getBackgroundColor(false));
+			}
+			return backColor;
+		}
 		if (backColor == null) {
 			backColor = skinParam.getHtmlColor(ColorParam.packageBackground, stereotype, false);
 		}
 		if (backColor == null) {
 			backColor = skinParam.getHtmlColor(ColorParam.background, stereotype, false);
 		}
-		if (backColor == null /* || stateBack instanceof HtmlColorTransparent */) {
-			backColor = new HtmlColorTransparent();
+		if (backColor == null
+				|| backColor.equals(HColorUtils.transparent()) /* || stateBack instanceof HtmlColorTransparent */) {
+			final HColor tmp = skinParam.getBackgroundColor(false);
+			backColor = new HColorBackground(tmp);
 		}
 		return backColor;
 	}
