@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -39,51 +39,59 @@ import java.awt.geom.Dimension2D;
 import java.awt.geom.Rectangle2D;
 
 import net.sourceforge.plantuml.ColorParam;
+import net.sourceforge.plantuml.CornerParam;
 import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.FontParam;
 import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.LineConfigurable;
 import net.sourceforge.plantuml.LineParam;
-import net.sourceforge.plantuml.CornerParam;
 import net.sourceforge.plantuml.SkinParamUtils;
 import net.sourceforge.plantuml.Url;
+import net.sourceforge.plantuml.UseStyle;
 import net.sourceforge.plantuml.creole.Stencil;
 import net.sourceforge.plantuml.cucadiagram.EntityPortion;
 import net.sourceforge.plantuml.cucadiagram.ILeaf;
+import net.sourceforge.plantuml.cucadiagram.LeafType;
 import net.sourceforge.plantuml.cucadiagram.PortionShower;
 import net.sourceforge.plantuml.cucadiagram.dot.GraphvizVersion;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.InnerStrategy;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.graphic.color.ColorType;
+import net.sourceforge.plantuml.style.PName;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleSignature;
 import net.sourceforge.plantuml.svek.AbstractEntityImage;
 import net.sourceforge.plantuml.svek.Margins;
 import net.sourceforge.plantuml.svek.Ports;
 import net.sourceforge.plantuml.svek.ShapeType;
 import net.sourceforge.plantuml.svek.WithPorts;
 import net.sourceforge.plantuml.ugraphic.Shadowable;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UComment;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UGraphicStencil;
+import net.sourceforge.plantuml.ugraphic.UGroupType;
 import net.sourceforge.plantuml.ugraphic.URectangle;
 import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorNone;
 
 public class EntityImageClass extends AbstractEntityImage implements Stencil, WithPorts {
 
 	final private TextBlock body;
 	final private Margins shield;
-	final private EntityImageClassHeader2 header;
+	final private EntityImageClassHeader header;
 	final private Url url;
 	final private double roundCorner;
+	final private LeafType leafType;
 
 	final private LineConfigurable lineConfig;
 
 	public EntityImageClass(GraphvizVersion version, ILeaf entity, ISkinParam skinParam, PortionShower portionShower) {
 		super(entity, entity.getColors(skinParam).mute(skinParam));
+		this.leafType = entity.getLeafType();
 		this.lineConfig = entity;
 		this.roundCorner = getSkinParam().getRoundCorner(CornerParam.DEFAULT, null);
 		this.shield = version != null && version.useShield() && entity.hasNearDecoration() ? Margins.uniform(16)
@@ -91,14 +99,12 @@ public class EntityImageClass extends AbstractEntityImage implements Stencil, Wi
 		final boolean showMethods = portionShower.showPortion(EntityPortion.METHOD, entity);
 		final boolean showFields = portionShower.showPortion(EntityPortion.FIELD, entity);
 		this.body = entity.getBodier().getBody(FontParam.CLASS_ATTRIBUTE, getSkinParam(), showMethods, showFields,
-				entity.getStereotype());
+				entity.getStereotype(), getStyle());
 
-		header = new EntityImageClassHeader2(entity, getSkinParam(), portionShower);
+		header = new EntityImageClassHeader(entity, getSkinParam(), portionShower);
 		this.url = entity.getUrl99();
 	}
 
-	// private int marginEmptyFieldsOrMethod = 13;
-
 	public Dimension2D calculateDimension(StringBounder stringBounder) {
 		final Dimension2D dimHeader = header.calculateDimension(stringBounder);
 		final Dimension2D dimBody = body == null ? new Dimension2DDouble(0, 0) : body.calculateDimension(stringBounder);
@@ -117,22 +123,30 @@ public class EntityImageClass extends AbstractEntityImage implements Stencil, Wi
 			return result;
 		}
 		final Dimension2D dimHeader = header.calculateDimension(stringBounder);
-		final UTranslate translate = new UTranslate(0, dimHeader.getHeight());
+		final UTranslate translate = UTranslate.dy(dimHeader.getHeight());
 		return translate.apply(result);
 	}
 
 	final public void drawU(UGraphic ug) {
-		ug.draw(new UComment("class " + getEntity().getCode().getFullName()));
+		ug.draw(new UComment("class " + getEntity().getCodeGetName()));
 		if (url != null) {
 			ug.startUrl(url);
 		}
+
+		ug.startGroup(UGroupType.CLASS, "elem " + getEntity().getCode() + " selected");
 		drawInternal(ug);
+		ug.closeGroup();
 
 		if (url != null) {
-			ug.closeAction();
+			ug.closeUrl();
 		}
 	}
 
+	private Style getStyle() {
+		return StyleSignature.of(SName.root, SName.element, SName.classDiagram, SName.class_)
+				.with(getEntity().getStereotype()).getMergedStyle(getSkinParam().getCurrentStyleBuilder());
+	}
+
 	private void drawInternal(UGraphic ug) {
 		final StringBounder stringBounder = ug.getStringBounder();
 		final Dimension2D dimTotal = calculateDimension(stringBounder);
@@ -140,48 +154,84 @@ public class EntityImageClass extends AbstractEntityImage implements Stencil, Wi
 
 		final double widthTotal = dimTotal.getWidth();
 		final double heightTotal = dimTotal.getHeight();
-		final Shadowable rect = new URectangle(widthTotal, heightTotal, roundCorner, roundCorner, getEntity().getCode()
-				.getFullName());
-		if (getSkinParam().shadowing()) {
+		final Shadowable rect = new URectangle(widthTotal, heightTotal).rounded(roundCorner)
+				.withCommentAndCodeLine(getEntity().getCodeGetName(), getEntity().getCodeLine());
+		if (getSkinParam().shadowing(getEntity().getStereotype())) {
 			rect.setDeltaShadow(4);
 		}
 
-		HtmlColor classBorder = lineConfig.getColors(getSkinParam()).getColor(ColorType.LINE);
+		HColor classBorder = lineConfig.getColors(getSkinParam()).getColor(ColorType.LINE);
+		HColor headerBackcolor = getEntity().getColors(getSkinParam()).getColor(ColorType.HEADER);
+
 		if (classBorder == null) {
-			classBorder = SkinParamUtils.getColor(getSkinParam(), ColorParam.classBorder, getStereo());
+			if (UseStyle.useBetaStyle())
+				classBorder = getStyle().value(PName.LineColor).asColor(getSkinParam().getThemeStyle(),
+						getSkinParam().getIHtmlColorSet());
+			else
+				classBorder = SkinParamUtils.getColor(getSkinParam(), getStereo(), ColorParam.classBorder);
 		}
-		ug = ug.apply(new UChangeColor(classBorder));
-		HtmlColor backcolor = getEntity().getColors(getSkinParam()).getColor(ColorType.BACK);
+		HColor backcolor = getEntity().getColors(getSkinParam()).getColor(ColorType.BACK);
 		if (backcolor == null) {
-			backcolor = SkinParamUtils.getColor(getSkinParam(), ColorParam.classBackground, getStereo());
+			if (UseStyle.useBetaStyle())
+				backcolor = getStyle().value(PName.BackGroundColor).asColor(getSkinParam().getThemeStyle(),
+						getSkinParam().getIHtmlColorSet());
+			else {
+				if (leafType == LeafType.ENUM) {
+					backcolor = SkinParamUtils.getColor(getSkinParam(), getStereo(), ColorParam.enumBackground,
+							ColorParam.classBackground);
+				} else {
+					backcolor = SkinParamUtils.getColor(getSkinParam(), getStereo(), ColorParam.classBackground);
+				}
+			}
 		}
-		ug = ug.apply(new UChangeBackColor(backcolor));
+
+		ug = ug.apply(classBorder);
+		ug = ug.apply(backcolor.bg());
 
 		final UStroke stroke = getStroke();
-		ug.apply(stroke).draw(rect);
 
-		HtmlColor headerBackcolor = getEntity().getColors(getSkinParam()).getColor(ColorType.HEADER);
 		if (headerBackcolor == null) {
-			headerBackcolor = getSkinParam().getHtmlColor(ColorParam.classHeaderBackground, getStereo(), false);
+			if (UseStyle.useBetaStyle())
+				headerBackcolor = getStyle().value(PName.BackGroundColor).asColor(getSkinParam().getThemeStyle(),
+						getSkinParam().getIHtmlColorSet());
+			else
+				headerBackcolor = getSkinParam().getHtmlColor(ColorParam.classHeaderBackground, getStereo(), false);
 		}
 		UGraphic ugHeader = ug;
-		if (headerBackcolor != null) {
+		if (roundCorner == 0 && headerBackcolor != null && backcolor.equals(headerBackcolor) == false) {
+			ug.apply(stroke).draw(rect);
 			final Shadowable rect2 = new URectangle(widthTotal, dimHeader.getHeight());
-			ugHeader = ugHeader.apply(new UChangeBackColor(headerBackcolor));
+			rect2.setDeltaShadow(0);
+			ugHeader = ugHeader.apply(headerBackcolor.bg());
 			ugHeader.apply(stroke).draw(rect2);
+		} else if (roundCorner != 0 && headerBackcolor != null && backcolor.equals(headerBackcolor) == false) {
+			ug.apply(stroke).draw(rect);
+			final Shadowable rect2 = new URectangle(widthTotal, dimHeader.getHeight()).rounded(roundCorner);
+			final URectangle rect3 = new URectangle(widthTotal, roundCorner / 2);
+			rect2.setDeltaShadow(0);
+			rect3.setDeltaShadow(0);
+			ugHeader = ugHeader.apply(headerBackcolor.bg()).apply(headerBackcolor);
+			ugHeader.apply(stroke).draw(rect2);
+			ugHeader.apply(stroke).apply(UTranslate.dy(dimHeader.getHeight() - rect3.getHeight())).draw(rect3);
+			rect.setDeltaShadow(0);
+			ug.apply(stroke).apply(new HColorNone().bg()).draw(rect);
+		} else {
+			ug.apply(stroke).draw(rect);
 		}
 		header.drawU(ugHeader, dimTotal.getWidth(), dimHeader.getHeight());
 
 		if (body != null) {
 			final UGraphic ug2 = UGraphicStencil.create(ug, this, stroke);
-			final UTranslate translate = new UTranslate(0, dimHeader.getHeight());
+			final UTranslate translate = UTranslate.dy(dimHeader.getHeight());
 			body.drawU(ug2.apply(translate));
 		}
 	}
 
 	public Ports getPorts(StringBounder stringBounder) {
 		final Dimension2D dimHeader = header.calculateDimension(stringBounder);
-		return ((WithPorts) body).getPorts(stringBounder).translateY(dimHeader.getHeight());
+		if (body instanceof WithPorts)
+			return ((WithPorts) body).getPorts(stringBounder).translateY(dimHeader.getHeight());
+		return new Ports();
 	}
 
 	private UStroke getStroke() {
