@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -38,27 +38,29 @@ package net.sourceforge.plantuml.svek.image;
 import java.awt.geom.Dimension2D;
 import java.awt.geom.Line2D;
 import java.awt.geom.Point2D;
+import java.util.Objects;
 
+import net.sourceforge.plantuml.AlignmentParam;
 import net.sourceforge.plantuml.ColorParam;
+import net.sourceforge.plantuml.CornerParam;
 import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.Direction;
 import net.sourceforge.plantuml.FontParam;
 import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.LineParam;
-import net.sourceforge.plantuml.CornerParam;
 import net.sourceforge.plantuml.SkinParamBackcolored;
 import net.sourceforge.plantuml.SkinParamUtils;
+import net.sourceforge.plantuml.UmlDiagramType;
 import net.sourceforge.plantuml.Url;
+import net.sourceforge.plantuml.UseStyle;
 import net.sourceforge.plantuml.creole.Stencil;
-import net.sourceforge.plantuml.cucadiagram.BodyEnhanced2;
+import net.sourceforge.plantuml.cucadiagram.BodyFactory;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.IEntity;
 import net.sourceforge.plantuml.cucadiagram.ILeaf;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
-import net.sourceforge.plantuml.graph2.GeomUtils;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.graphic.TextBlockEmpty;
@@ -66,22 +68,27 @@ import net.sourceforge.plantuml.graphic.color.ColorType;
 import net.sourceforge.plantuml.graphic.color.Colors;
 import net.sourceforge.plantuml.posimo.DotPath;
 import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.style.PName;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleSignature;
 import net.sourceforge.plantuml.svek.AbstractEntityImage;
-import net.sourceforge.plantuml.svek.Line;
-import net.sourceforge.plantuml.svek.Shape;
 import net.sourceforge.plantuml.svek.ShapeType;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
+import net.sourceforge.plantuml.svek.SvekLine;
+import net.sourceforge.plantuml.svek.SvekNode;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UGraphicStencil;
+import net.sourceforge.plantuml.ugraphic.UGroupType;
 import net.sourceforge.plantuml.ugraphic.UPath;
 import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public class EntityImageNote extends AbstractEntityImage implements Stencil {
 
-	private final HtmlColor noteBackgroundColor;
-	private final HtmlColor borderColor;
+	private final HColor noteBackgroundColor;
+	private final HColor borderColor;
+	private final double shadowing;
 	private final int marginX1 = 6;
 	private final int marginX2 = 15;
 	private final int marginY = 5;
@@ -90,30 +97,45 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 
 	private final TextBlock textBlock;
 
-	public EntityImageNote(ILeaf entity, ISkinParam skinParam) {
+	public EntityImageNote(ILeaf entity, ISkinParam skinParam, UmlDiagramType umlDiagramType) {
 		super(entity, getSkin(getISkinParam(skinParam, entity), entity));
 		this.skinParam = getISkinParam(skinParam, entity);
 
-		this.withShadow = getSkinParam().shadowing();
+		this.withShadow = getSkinParam().shadowing(getEntity().getStereotype());
 		final Display strings = entity.getDisplay();
 
 		final Rose rose = new Rose();
 
-		if (entity.getColors(getSkinParam()).getColor(ColorType.BACK) == null) {
-			noteBackgroundColor = rose.getHtmlColor(getSkinParam(), ColorParam.noteBackground);
+		if (UseStyle.useBetaStyle()) {
+			final Style style = getDefaultStyleDefinition(umlDiagramType.getStyleName())
+					.getMergedStyle(skinParam.getCurrentStyleBuilder());
+			if (entity.getColors(getSkinParam()).getColor(ColorType.BACK) == null) {
+				this.noteBackgroundColor = style.value(PName.BackGroundColor).asColor(skinParam.getThemeStyle(),
+						skinParam.getIHtmlColorSet());
+			} else {
+				this.noteBackgroundColor = entity.getColors(getSkinParam()).getColor(ColorType.BACK);
+			}
+			this.borderColor = style.value(PName.LineColor).asColor(skinParam.getThemeStyle(),
+					skinParam.getIHtmlColorSet());
+			this.shadowing = style.value(PName.Shadowing).asDouble();
 		} else {
-			noteBackgroundColor = entity.getColors(getSkinParam()).getColor(ColorType.BACK);
+			this.shadowing = skinParam.shadowing(getEntity().getStereotype()) ? 4 : 0;
+			if (entity.getColors(getSkinParam()).getColor(ColorType.BACK) == null) {
+				this.noteBackgroundColor = rose.getHtmlColor(getSkinParam(), ColorParam.noteBackground);
+			} else {
+				this.noteBackgroundColor = entity.getColors(getSkinParam()).getColor(ColorType.BACK);
+			}
+			this.borderColor = SkinParamUtils.getColor(getSkinParam(), null, ColorParam.noteBorder);
 		}
-		// this.borderColor = rose.getHtmlColor(skinParam, ColorParam.noteBorder);
-		this.borderColor = SkinParamUtils.getColor(getSkinParam(), ColorParam.noteBorder, null);
-		// final HtmlColor fontColor = rose.getFontColor(getSkinParam(), FontParam.NOTE);
-		// final UFont fontNote = getSkinParam().getFont(FontParam.NOTE, null, false);
 
 		if (strings.size() == 1 && strings.get(0).length() == 0) {
 			textBlock = new TextBlockEmpty();
 		} else {
-			textBlock = new BodyEnhanced2(strings, FontParam.NOTE, getSkinParam(), HorizontalAlignment.LEFT,
-					new FontConfiguration(getSkinParam(), FontParam.NOTE, null), getSkinParam().wrapWidth());
+			final FontConfiguration fc = new FontConfiguration(getSkinParam(), FontParam.NOTE, null);
+			final HorizontalAlignment align = skinParam.getHorizontalAlignment(AlignmentParam.noteTextAlignment, null,
+					false, null);
+			textBlock = BodyFactory.create3(strings, FontParam.NOTE, getSkinParam(), align, fc,
+					getSkinParam().wrapWidth());
 		}
 	}
 
@@ -126,7 +148,7 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 
 	static ISkinParam getSkin(ISkinParam skinParam, IEntity entity) {
 		final Stereotype stereotype = entity.getStereotype();
-		HtmlColor back = entity.getColors(skinParam).getColor(ColorType.BACK);
+		HColor back = entity.getColors(skinParam).getColor(ColorType.BACK);
 		if (back != null) {
 			return new SkinParamBackcolored(skinParam, back);
 		}
@@ -137,9 +159,9 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 		return skinParam;
 	}
 
-	private static HtmlColor getColorStatic(ISkinParam skinParam, ColorParam colorParam, Stereotype stereo) {
+	private static HColor getColorStatic(ISkinParam skinParam, ColorParam colorParam, Stereotype stereo) {
 		final Rose rose = new Rose();
-		return rose.getHtmlColor(skinParam, colorParam, stereo);
+		return rose.getHtmlColor(skinParam, stereo, colorParam);
 	}
 
 	final public double getPreferredWidth(StringBounder stringBounder) {
@@ -181,8 +203,15 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 		return new Dimension2DDouble(width, height);
 	}
 
+	private StyleSignature getDefaultStyleDefinition(SName sname) {
+		return StyleSignature.of(SName.root, SName.element, sname, SName.note);
+	}
+
 	final public void drawU(UGraphic ug) {
 		final Url url = getEntity().getUrl99();
+
+		ug.startGroup(UGroupType.CLASS, "elem " + getEntity().getCode() + " selected");
+
 		if (url != null) {
 			ug.startUrl(url);
 		}
@@ -192,7 +221,7 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 		} else {
 			final StringBounder stringBounder = ug.getStringBounder();
 			DotPath path = opaleLine.getDotPath();
-			path.moveSvek(-shape.getMinX(), -shape.getMinY());
+			path.moveSvek(-node.getMinX(), -node.getMinY());
 			Point2D p1 = path.getStartPoint();
 			Point2D p2 = path.getEndPoint();
 			final double textWidth = getTextWidth(stringBounder);
@@ -206,18 +235,20 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 			final Direction strategy = getOpaleStrategy(textWidth, textHeight, p1);
 			final Point2D pp1 = path.getStartPoint();
 			final Point2D pp2 = path.getEndPoint();
-			final Point2D newRefpp2 = move(pp2, shape.getMinX(), shape.getMinY());
-			final Point2D projection = move(other.projection(newRefpp2, stringBounder), -shape.getMinX(),
-					-shape.getMinY());
-			final Opale opale = new Opale(borderColor, noteBackgroundColor, textBlock, skinParam.shadowing(), true);
+			final Point2D newRefpp2 = move(pp2, node.getMinX(), node.getMinY());
+			final Point2D projection = move(other.projection(newRefpp2, stringBounder), -node.getMinX(),
+					-node.getMinY());
+			final Opale opale = new Opale(shadowing, borderColor, noteBackgroundColor, textBlock, true);
 			opale.setRoundCorner(getRoundCorner());
 			opale.setOpale(strategy, pp1, projection);
 			final UGraphic stroked = applyStroke(ug2);
 			opale.drawU(Colors.applyStroke(stroked, getEntity().getColors(skinParam)));
 		}
 		if (url != null) {
-			ug.closeAction();
+			ug.closeUrl();
 		}
+
+		ug.closeGroup();
 	}
 
 	private double getRoundCorner() {
@@ -235,7 +266,7 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 		if (withShadow) {
 			polygon.setDeltaShadow(4);
 		}
-		ug = ug.apply(new UChangeBackColor(noteBackgroundColor)).apply(new UChangeColor(borderColor));
+		ug = ug.apply(noteBackgroundColor.bg()).apply(borderColor);
 		final UGraphic stroked = applyStroke(ug);
 		stroked.draw(polygon);
 		ug.draw(Opale.getCorner(getTextWidth(stringBounder), getRoundCorner()));
@@ -252,10 +283,10 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 	}
 
 	private Direction getOpaleStrategy(double width, double height, Point2D pt) {
-		final double d1 = GeomUtils.getOrthoDistance(new Line2D.Double(width, 0, width, height), pt);
-		final double d2 = GeomUtils.getOrthoDistance(new Line2D.Double(0, height, width, height), pt);
-		final double d3 = GeomUtils.getOrthoDistance(new Line2D.Double(0, 0, 0, height), pt);
-		final double d4 = GeomUtils.getOrthoDistance(new Line2D.Double(0, 0, width, 0), pt);
+		final double d1 = getOrthoDistance(new Line2D.Double(width, 0, width, height), pt);
+		final double d2 = getOrthoDistance(new Line2D.Double(0, height, width, height), pt);
+		final double d3 = getOrthoDistance(new Line2D.Double(0, 0, 0, height), pt);
+		final double d4 = getOrthoDistance(new Line2D.Double(0, 0, width, 0), pt);
 		if (d3 <= d1 && d3 <= d2 && d3 <= d4) {
 			return Direction.LEFT;
 		}
@@ -272,21 +303,36 @@ public class EntityImageNote extends AbstractEntityImage implements Stencil {
 
 	}
 
+	private static double getOrthoDistance(Line2D.Double seg, Point2D pt) {
+		if (isHorizontal(seg)) {
+			return Math.abs(seg.getP1().getY() - pt.getY());
+		}
+		if (isVertical(seg)) {
+			return Math.abs(seg.getP1().getX() - pt.getX());
+		}
+		throw new IllegalArgumentException();
+	}
+
+	private static boolean isHorizontal(Line2D.Double seg) {
+		return seg.getP1().getY() == seg.getP2().getY();
+	}
+
+	private static boolean isVertical(Line2D.Double seg) {
+		return seg.getP1().getX() == seg.getP2().getX();
+	}
+
 	public ShapeType getShapeType() {
 		return ShapeType.RECTANGLE;
 	}
 
-	private Line opaleLine;
-	private Shape shape;
-	private Shape other;
+	private SvekLine opaleLine;
+	private SvekNode node;
+	private SvekNode other;
 
-	public void setOpaleLine(Line line, Shape shape, Shape other) {
-		if (other == null) {
-			throw new IllegalArgumentException();
-		}
+	public void setOpaleLine(SvekLine line, SvekNode node, SvekNode other) {
 		this.opaleLine = line;
-		this.shape = shape;
-		this.other = other;
+		this.node = node;
+		this.other = Objects.requireNonNull(other);
 	}
 
 	public double getStartingX(StringBounder stringBounder, double y) {
