@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -37,40 +37,31 @@ package net.sourceforge.plantuml.svek.image;
 
 import java.awt.geom.Dimension2D;
 
-import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.FontParam;
 import net.sourceforge.plantuml.ISkinParam;
-import net.sourceforge.plantuml.LineConfigurable;
-import net.sourceforge.plantuml.SkinParamUtils;
-import net.sourceforge.plantuml.Url;
+import net.sourceforge.plantuml.UseStyle;
+import net.sourceforge.plantuml.creole.CreoleMode;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.IEntity;
-import net.sourceforge.plantuml.cucadiagram.Member;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
-import net.sourceforge.plantuml.graphic.color.ColorType;
-import net.sourceforge.plantuml.svek.AbstractEntityImage;
-import net.sourceforge.plantuml.svek.ShapeType;
-import net.sourceforge.plantuml.ugraphic.Shadowable;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleSignature;
 import net.sourceforge.plantuml.ugraphic.UEllipse;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
+import net.sourceforge.plantuml.ugraphic.UGroupType;
 import net.sourceforge.plantuml.ugraphic.ULine;
-import net.sourceforge.plantuml.ugraphic.URectangle;
 import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
 
-public class EntityImageState extends AbstractEntityImage {
+public class EntityImageState extends EntityImageStateCommon {
 
-	final private TextBlock desc;
 	final private TextBlock fields;
-	final private Url url;
 
 	final private static int MIN_WIDTH = 50;
 	final private static int MIN_HEIGHT = 50;
@@ -82,29 +73,26 @@ public class EntityImageState extends AbstractEntityImage {
 	final static private double smallMarginX = 7;
 	final static private double smallMarginY = 4;
 
-	final private LineConfigurable lineConfig;
-
 	public EntityImageState(IEntity entity, ISkinParam skinParam) {
 		super(entity, skinParam);
-		this.lineConfig = entity;
+
 		final Stereotype stereotype = entity.getStereotype();
+
 		this.withSymbol = stereotype != null && stereotype.isWithOOSymbol();
+		final Display list = Display.create(entity.getBodier().getRawBody());
+		final FontConfiguration fontConfiguration;
 
-		this.desc = entity.getDisplay().create(new FontConfiguration(getSkinParam(), FontParam.STATE, stereotype),
-				HorizontalAlignment.CENTER, skinParam);
+		if (UseStyle.useBetaStyle())
+			fontConfiguration = getStyleMember().getFontConfiguration(getSkinParam().getThemeStyle(),
+					getSkinParam().getIHtmlColorSet());
+		else
+			fontConfiguration = new FontConfiguration(getSkinParam(), FontParam.STATE_ATTRIBUTE, stereotype);
 
-		Display list = Display.empty();
-		for (Member att : entity.getBodier().getFieldsToDisplay()) {
-			list = list.addAll(Display.getWithNewlines(att.getDisplay(true)));
-		}
-
-		this.url = entity.getUrl99();
-
-		this.fields = list.create(new FontConfiguration(getSkinParam(), FontParam.STATE_ATTRIBUTE, stereotype),
-				HorizontalAlignment.LEFT, skinParam);
+		this.fields = list.create8(fontConfiguration, HorizontalAlignment.LEFT, skinParam, CreoleMode.FULL,
+				skinParam.wrapWidth());
 
 	}
-
+	
 	public Dimension2D calculateDimension(StringBounder stringBounder) {
 		final Dimension2D dim = Dimension2DDouble.mergeTB(desc.calculateDimension(stringBounder),
 				fields.calculateDimension(stringBounder));
@@ -117,6 +105,7 @@ public class EntityImageState extends AbstractEntityImage {
 	}
 
 	final public void drawU(UGraphic ug) {
+		ug.startGroup(UGroupType.ID, getEntity().getIdent().toString("."));
 		if (url != null) {
 			ug.startUrl(url);
 		}
@@ -124,28 +113,11 @@ public class EntityImageState extends AbstractEntityImage {
 		final Dimension2D dimTotal = calculateDimension(stringBounder);
 		final Dimension2D dimDesc = desc.calculateDimension(stringBounder);
 
-		final double widthTotal = dimTotal.getWidth();
-		final double heightTotal = dimTotal.getHeight();
-		final Shadowable rect = new URectangle(widthTotal, heightTotal, CORNER, CORNER);
-		if (getSkinParam().shadowing()) {
-			rect.setDeltaShadow(4);
-		}
-
-		HtmlColor classBorder = lineConfig.getColors(getSkinParam()).getColor(ColorType.LINE);
-		if (classBorder == null) {
-			classBorder = SkinParamUtils.getColor(getSkinParam(), ColorParam.stateBorder, getStereo());
-		}
-		ug = ug.apply(getStroke()).apply(new UChangeColor(classBorder));
-		HtmlColor backcolor = getEntity().getColors(getSkinParam()).getColor(ColorType.BACK);
-		if (backcolor == null) {
-			backcolor = SkinParamUtils.getColor(getSkinParam(), ColorParam.stateBackground, getStereo());
-		}
-		ug = ug.apply(new UChangeBackColor(backcolor));
-
-		ug.draw(rect);
+		ug = applyColor(ug);
+		ug.draw(getShape(dimTotal));
 
 		final double yLine = MARGIN + dimDesc.getHeight() + MARGIN_LINE;
-		ug.apply(new UTranslate(0, yLine)).draw(new ULine(widthTotal, 0));
+		ug.apply(UTranslate.dy(yLine)).draw(ULine.hline(dimTotal.getWidth()));
 
 		ug = ug.apply(new UStroke());
 
@@ -155,7 +127,7 @@ public class EntityImageState extends AbstractEntityImage {
 			drawSymbol(ug, xSymbol, ySymbol);
 		}
 
-		final double xDesc = (widthTotal - dimDesc.getWidth()) / 2;
+		final double xDesc = (dimTotal.getWidth() - dimDesc.getWidth()) / 2;
 		final double yDesc = MARGIN;
 		desc.drawU(ug.apply(new UTranslate(xDesc, yDesc)));
 
@@ -164,16 +136,9 @@ public class EntityImageState extends AbstractEntityImage {
 		fields.drawU(ug.apply(new UTranslate(xFields, yFields)));
 
 		if (url != null) {
-			ug.closeAction();
+			ug.closeUrl();
 		}
-	}
-
-	private UStroke getStroke() {
-		UStroke stroke = lineConfig.getColors(getSkinParam()).getSpecificLineStroke();
-		if (stroke == null) {
-			stroke = new UStroke(1.5);
-		}
-		return stroke;
+		ug.closeGroup();
 	}
 
 	public static void drawSymbol(UGraphic ug, double xSymbol, double ySymbol) {
@@ -182,11 +147,7 @@ public class EntityImageState extends AbstractEntityImage {
 		final UEllipse small = new UEllipse(2 * smallRadius, 2 * smallRadius);
 		ug.apply(new UTranslate(xSymbol, ySymbol)).draw(small);
 		ug.apply(new UTranslate(xSymbol + smallLine + 2 * smallRadius, ySymbol)).draw(small);
-		ug.apply(new UTranslate(xSymbol + 2 * smallRadius, ySymbol + smallLine)).draw(new ULine(smallLine, 0));
-	}
-
-	public ShapeType getShapeType() {
-		return ShapeType.ROUND_RECTANGLE;
+		ug.apply(new UTranslate(xSymbol + 2 * smallRadius, ySymbol + smallLine)).draw(ULine.hline(smallLine));
 	}
 
 }
