@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -46,53 +46,83 @@ import net.sourceforge.plantuml.Direction;
 import net.sourceforge.plantuml.FontParam;
 import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.LineBreakStrategy;
+import net.sourceforge.plantuml.UmlDiagramType;
+import net.sourceforge.plantuml.UseStyle;
 import net.sourceforge.plantuml.command.Position;
-import net.sourceforge.plantuml.cucadiagram.BodyEnhanced2;
+import net.sourceforge.plantuml.cucadiagram.BodyFactory;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.IEntity;
 import net.sourceforge.plantuml.cucadiagram.ILeaf;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.InnerStrategy;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.graphic.color.ColorType;
 import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.style.PName;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleSignature;
 import net.sourceforge.plantuml.svek.AbstractEntityImage;
 import net.sourceforge.plantuml.svek.Bibliotekon;
-import net.sourceforge.plantuml.svek.Shape;
 import net.sourceforge.plantuml.svek.ShapeType;
+import net.sourceforge.plantuml.svek.SvekNode;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public class EntityImageTips extends AbstractEntityImage {
 
 	final private Rose rose = new Rose();
 	private final ISkinParam skinParam;
 
-	private final HtmlColor noteBackgroundColor;
-	private final HtmlColor borderColor;
+	private final HColor noteBackgroundColor;
+	private final HColor borderColor;
 
 	private final Bibliotekon bibliotekon;
 
 	private final double ySpacing = 10;
 
-	public EntityImageTips(ILeaf entity, ISkinParam skinParam, Bibliotekon bibliotekon) {
+	public EntityImageTips(ILeaf entity, ISkinParam skinParam, Bibliotekon bibliotekon, UmlDiagramType type) {
 		super(entity, EntityImageNote.getSkin(skinParam, entity));
 		this.skinParam = skinParam;
 		this.bibliotekon = bibliotekon;
 
-		if (entity.getColors(skinParam).getColor(ColorType.BACK) == null) {
-			noteBackgroundColor = rose.getHtmlColor(skinParam, ColorParam.noteBackground);
+		if (UseStyle.useBetaStyle()) {
+			final Style style = getDefaultStyleDefinition(type.getStyleName())
+					.getMergedStyle(skinParam.getCurrentStyleBuilder());
+
+			if (entity.getColors(skinParam).getColor(ColorType.BACK) == null) {
+				this.noteBackgroundColor = style.value(PName.BackGroundColor).asColor(skinParam.getThemeStyle(),
+						skinParam.getIHtmlColorSet());
+
+			} else {
+				this.noteBackgroundColor = entity.getColors(skinParam).getColor(ColorType.BACK);
+			}
+
+			this.borderColor = style.value(PName.LineColor).asColor(skinParam.getThemeStyle(),
+					skinParam.getIHtmlColorSet());
+
 		} else {
-			noteBackgroundColor = entity.getColors(skinParam).getColor(ColorType.BACK);
+
+			if (entity.getColors(skinParam).getColor(ColorType.BACK) == null) {
+				this.noteBackgroundColor = rose.getHtmlColor(skinParam, ColorParam.noteBackground);
+			} else {
+				this.noteBackgroundColor = entity.getColors(skinParam).getColor(ColorType.BACK);
+			}
+
+			this.borderColor = rose.getHtmlColor(skinParam, ColorParam.noteBorder);
+
 		}
-		this.borderColor = rose.getHtmlColor(skinParam, ColorParam.noteBorder);
+	}
+
+	private StyleSignature getDefaultStyleDefinition(SName sname) {
+		return StyleSignature.of(SName.root, SName.element, sname, SName.note);
 	}
 
 	private Position getPosition() {
-		if (getEntity().getCode().getFullName().endsWith(Position.RIGHT.name())) {
+		if (getEntity().getCodeGetName().endsWith(Position.RIGHT.name())) {
 			return Position.RIGHT;
 		}
 		return Position.LEFT;
@@ -120,17 +150,21 @@ public class EntityImageTips extends AbstractEntityImage {
 
 		final IEntity other = bibliotekon.getOnlyOther(getEntity());
 
-		final Shape shapeMe = bibliotekon.getShape(getEntity());
-		final Shape shapeOther = bibliotekon.getShape(other);
-		final Point2D positionMe = shapeMe.getPosition();
-		final Point2D positionOther = shapeOther.getPosition();
-		bibliotekon.getShape(getEntity());
+		final SvekNode nodeMe = bibliotekon.getNode(getEntity());
+		final SvekNode nodeOther = bibliotekon.getNode(other);
+		final Point2D positionMe = nodeMe.getPosition();
+		if (nodeOther == null) {
+			System.err.println("Error in EntityImageTips");
+			return;
+		}
+		final Point2D positionOther = nodeOther.getPosition();
+		bibliotekon.getNode(getEntity());
 		final Position position = getPosition();
 		Direction direction = position.reverseDirection();
 		double height = 0;
 		for (Map.Entry<String, Display> ent : getEntity().getTips().entrySet()) {
 			final Display display = ent.getValue();
-			final Rectangle2D memberPosition = shapeOther.getImage().getInnerPosition(ent.getKey(), stringBounder,
+			final Rectangle2D memberPosition = nodeOther.getImage().getInnerPosition(ent.getKey(), stringBounder,
 					InnerStrategy.STRICT);
 			if (memberPosition == null) {
 				return;
@@ -151,7 +185,7 @@ public class EntityImageTips extends AbstractEntityImage {
 			final Point2D pp2 = new Point2D.Double(x, y);
 			opale.setOpale(direction, pp1, pp2);
 			opale.drawU(ug);
-			ug = ug.apply(new UTranslate(0, dim.getHeight() + ySpacing));
+			ug = ug.apply(UTranslate.dy(dim.getHeight() + ySpacing));
 			height += dim.getHeight();
 			height += ySpacing;
 		}
@@ -159,11 +193,11 @@ public class EntityImageTips extends AbstractEntityImage {
 	}
 
 	private Opale getOpale(final Display display) {
-		// final HtmlColor fontColor = rose.getFontColor(skinParam, FontParam.NOTE);
-		// final UFont fontNote = skinParam.getFont(FontParam.NOTE, null, false);
-		final TextBlock textBlock = new BodyEnhanced2(display, FontParam.NOTE, skinParam, HorizontalAlignment.LEFT,
-				new FontConfiguration(skinParam, FontParam.NOTE, null), LineBreakStrategy.NONE);
-		final Opale opale = new Opale(borderColor, noteBackgroundColor, textBlock, skinParam.shadowing(), true);
+		final FontConfiguration fc = new FontConfiguration(skinParam, FontParam.NOTE, null);
+		final TextBlock textBlock = BodyFactory.create3(display, FontParam.NOTE, skinParam, HorizontalAlignment.LEFT,
+				fc, LineBreakStrategy.NONE);
+		final double shadowing = skinParam.shadowing(getEntity().getStereotype()) ? 4 : 0;
+		final Opale opale = new Opale(shadowing, borderColor, noteBackgroundColor, textBlock, true);
 		return opale;
 	}
 
