@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,6 +35,8 @@
  */
 package net.sourceforge.plantuml.tikz;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
+
 import java.awt.Color;
 import java.awt.geom.PathIterator;
 import java.io.IOException;
@@ -45,9 +47,9 @@ import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 
 import net.sourceforge.plantuml.Log;
-import net.sourceforge.plantuml.SignatureUtils;
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.eps.EpsGraphics;
 import net.sourceforge.plantuml.ugraphic.UPath;
@@ -60,7 +62,7 @@ public class TikzGraphics {
 	// https://www.sharelatex.com/blog/2013/08/27/tikz-series-pt1.html
 	// http://cremeronline.com/LaTeX/minimaltikz.pdf
 
-	private final List<String> cmd = new ArrayList<String>();
+	private final List<String> cmd = new ArrayList<>();
 	private final boolean withPreamble;
 
 	private Color color = Color.BLACK;
@@ -109,10 +111,7 @@ public class TikzGraphics {
 			return "black";
 		}
 		final String result = colornames.get(c);
-		if (result == null) {
-			throw new IllegalArgumentException();
-		}
-		return result;
+		return Objects.requireNonNull(result);
 	}
 
 	public void createData(OutputStream os) throws IOException {
@@ -136,8 +135,7 @@ public class TikzGraphics {
 			out(os, "                \\p2=(sourcenode.south east),");
 			out(os, "                \\n1={\\x2-\\x1},");
 			out(os, "                \\n2={\\y2-\\y1} in");
-			out(os,
-					"            node [inner sep=0pt, outer sep=0pt,anchor=north west,at=(\\p1)] {\\href{#1}{\\XeTeXLinkBox{\\phantom{\\rule{\\n1}{\\n2}}}}}");
+			out(os, "            node [inner sep=0pt, outer sep=0pt,anchor=north west,at=(\\p1)] {\\href{#1}{\\XeTeXLinkBox{\\phantom{\\rule{\\n1}{\\n2}}}}}");
 			out(os, "                    %xelatex needs \\XeTeXLinkBox, won't create a link unless it");
 			out(os, "                    %finds text --- rules don't work without \\XeTeXLinkBox.");
 			out(os, "                    %Still builds correctly with pdflatex and lualatex");
@@ -152,8 +150,7 @@ public class TikzGraphics {
 			out(os, "                \\p2=(sourcenode.south east),");
 			out(os, "                \\n1={\\x2-\\x1},");
 			out(os, "                \\n2={\\y2-\\y1} in");
-			out(os,
-					"            node [inner sep=0pt, outer sep=0pt,anchor=north west,at=(\\p1)] {\\hyperref [#1]{\\XeTeXLinkBox{\\phantom{\\rule{\\n1}{\\n2}}}}}");
+			out(os, "            node [inner sep=0pt, outer sep=0pt,anchor=north west,at=(\\p1)] {\\hyperref [#1]{\\XeTeXLinkBox{\\phantom{\\rule{\\n1}{\\n2}}}}}");
 			out(os, "                    %xelatex needs \\XeTeXLinkBox, won't create a link unless it");
 			out(os, "                    %finds text --- rules don't work without \\XeTeXLinkBox.");
 			out(os, "                    %Still builds correctly with pdflatex and lualatex");
@@ -271,13 +268,13 @@ public class TikzGraphics {
 		return "(" + format(x) + "pt," + format(y) + "pt)";
 	}
 
-	private String format(double x) {
-		return EpsGraphics.format(x);
+	public static String format(double x) {
+		return EpsGraphics.formatSimple4(x);
 	}
 
 	private void out(OutputStream os, String s) throws IOException {
-		os.write(s.getBytes("UTF-8"));
-		os.write("\n".getBytes("UTF-8"));
+		os.write(s.getBytes(UTF_8));
+		os.write("\n".getBytes(UTF_8));
 	}
 
 	public void text(double x, double y, String text, boolean underline, boolean italic, boolean bold) {
@@ -318,6 +315,18 @@ public class TikzGraphics {
 		addCommand(sb);
 	}
 
+	public void appendRaw(double x, double y, String formula) {
+		Objects.requireNonNull(formula);
+		final StringBuilder sb = new StringBuilder("\\node at " + couple(x, y));
+		sb.append("[below right");
+		sb.append("]{");
+		sb.append("{");
+		sb.append(formula);
+		sb.append("}");
+		sb.append("};");
+		addCommand(sb);
+	}
+
 	private void appendPendingUrl(final StringBuilder sb) {
 		if (Url.isLatex(pendingUrl)) {
 			sb.append("\\hyperref[");
@@ -347,7 +356,12 @@ public class TikzGraphics {
 		text = text.replaceAll("&", "\\\\&");
 		text = text.replaceAll("%", "\\\\%");
 		text = text.replace("$", "\\$");
-		text = text.replaceAll("~", "\\\\~{}");
+		text = text.replace("{", "\\{");
+		text = text.replace("}", "\\}");
+		// text = text.replaceAll("~", "\\\\~{}");
+		text = text.replace("~", "{\\raise.35ex\\hbox{$\\scriptstyle\\mathtt{\\sim}$}}");
+		// {\raise.35ex\hbox{$\scriptstyle\mathtt{\sim}$}}
+		// {\\raise.35ex\\hbox{$\\scriptstyle\\mathtt{\\sim}$}}
 		return text;
 	}
 
@@ -371,6 +385,7 @@ public class TikzGraphics {
 
 	public void polygon(double[] points) {
 
+		assert points.length % 2 == 0;
 		final StringBuilder sb = new StringBuilder();
 		appendShadeOrDraw(sb);
 		sb.append("line width=" + thickness + "pt]");
@@ -384,6 +399,7 @@ public class TikzGraphics {
 	}
 
 	private void round(double r, double[] points) {
+		assert points.length % 2 == 0;
 		final StringBuilder sb = new StringBuilder();
 		appendShadeOrDraw(sb);
 		sb.append("line width=" + thickness + "pt]");
@@ -472,7 +488,8 @@ public class TikzGraphics {
 			} else if (type == USegmentType.SEG_QUADTO) {
 				throw new UnsupportedOperationException();
 			} else if (type == USegmentType.SEG_CUBICTO) {
-				// curvetoNoMacro(coord[0] + x, coord[1] + y, coord[2] + x, coord[3] + y, coord[4] + x, coord[5] + y);
+				// curvetoNoMacro(coord[0] + x, coord[1] + y, coord[2] + x, coord[3] + y,
+				// coord[4] + x, coord[5] + y);
 				sb.append(" ..controls ");
 				sb.append(couple(coord[0] + x, coord[1] + y));
 				sb.append(" and ");
@@ -481,8 +498,10 @@ public class TikzGraphics {
 				sb.append(couple(coord[4] + x, coord[5] + y));
 			} else if (type == USegmentType.SEG_CLOSE) {
 				// Nothing
+			} else if (type == USegmentType.SEG_ARCTO) {
+				// Nothing
 			} else {
-				Log.println("unknown " + seg);
+				Log.println("unknown4 " + seg);
 			}
 		}
 		sb.append(";");
@@ -503,6 +522,20 @@ public class TikzGraphics {
 		addCommand(sb);
 	}
 
+	public void arc(double x, double y, int angleStart, int angleEnd, double radius) {
+		final StringBuilder sb = new StringBuilder();
+		sb.append("\\draw[");
+		if (color != null) {
+			sb.append("color=" + getColorName(color) + ",");
+		}
+		if (fillcolor != null) {
+			sb.append("fill=" + getColorName(fillcolor) + ",");
+		}
+		sb.append("line width=" + thickness + "pt] " + couple(x, y) + " arc (" + angleStart + ":" + angleEnd + ":"
+				+ format(radius) + "pt);");
+		addCommand(sb);
+	}
+
 	public void drawSingleCharacter(double x, double y, char c) {
 		final StringBuilder sb = new StringBuilder();
 		sb.append("\\node at ");
@@ -512,8 +545,8 @@ public class TikzGraphics {
 	}
 
 	public void drawPathIterator(double x, double y, PathIterator path) {
-		final StringBuilder sb = new StringBuilder("\\draw[color=" + getColorName(color) + ",fill="
-				+ getColorName(color) + "] ");
+		final StringBuilder sb = new StringBuilder(
+				"\\draw[color=" + getColorName(color) + ",fill=" + getColorName(color) + "] ");
 		final double coord[] = new double[6];
 		while (path.isDone() == false) {
 			final int code = path.currentSegment(coord);
@@ -568,9 +601,7 @@ public class TikzGraphics {
 	}
 
 	public void setStrokeColor(Color c) {
-		// if (c == null) {
-		// throw new IllegalArgumentException();
-		// }
+		// Objects.requireNonNull(c);
 		this.color = c;
 		addColor(c);
 	}
@@ -596,12 +627,9 @@ public class TikzGraphics {
 	private boolean hasUrl = false;
 
 	public void openLink(String url, String title) {
-		if (url == null) {
-			throw new IllegalArgumentException();
-		}
 		this.hasUrl = true;
 		this.urlIgnoreText = false;
-		this.pendingUrl = url;
+		this.pendingUrl = Objects.requireNonNull(url);
 		//
 		// if (pendingLink2.size() > 0) {
 		// closeLink();
