@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -37,6 +37,11 @@ package net.sourceforge.plantuml.ugraphic;
 
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.graphic.StringBounder;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapper;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+
+import java.io.IOException;
+import java.io.OutputStream;
 
 public interface UGraphic {
 
@@ -52,11 +57,17 @@ public interface UGraphic {
 
 	public void startUrl(Url url);
 
-	public void closeAction();
+	public void closeUrl();
+
+	public void startGroup(UGroupType type, String ident);
+
+	public void closeGroup();
 
 	public void flushUg();
 
 	public boolean matchesProperty(String propertyName);
-	
-	public double dpiFactor();
+
+	public HColor getDefaultBackground();
+
+	public void writeToStream(OutputStream os, String metadata, int dpi) throws IOException;
 }
