@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,17 +36,16 @@ package net.sourceforge.plantuml.ugraphic.svg;
 
 import java.awt.geom.Rectangle2D;
 
-import net.sourceforge.plantuml.StringUtils;
-import net.sourceforge.plantuml.graphic.HtmlColor;
-import net.sourceforge.plantuml.graphic.HtmlColorGradient;
 import net.sourceforge.plantuml.svg.SvgGraphics;
 import net.sourceforge.plantuml.ugraphic.ClipContainer;
-import net.sourceforge.plantuml.ugraphic.ColorMapper;
 import net.sourceforge.plantuml.ugraphic.UClip;
 import net.sourceforge.plantuml.ugraphic.UDriver;
 import net.sourceforge.plantuml.ugraphic.UParam;
 import net.sourceforge.plantuml.ugraphic.URectangle;
 import net.sourceforge.plantuml.ugraphic.UShape;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapper;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorGradient;
 
 public class DriverRectangleSvg implements UDriver<SvgGraphics> {
 
@@ -64,18 +63,17 @@ public class DriverRectangleSvg implements UDriver<SvgGraphics> {
 		double width = rect.getWidth();
 		double height = rect.getHeight();
 
-		final String color = StringUtils.getAsSvg(mapper, param.getColor());
-		final HtmlColor back = param.getBackcolor();
-		if (back instanceof HtmlColorGradient) {
-			final HtmlColorGradient gr = (HtmlColorGradient) back;
-			final String id = svg.createSvgGradient(StringUtils.getAsHtml(mapper.getMappedColor(gr.getColor1())),
-					StringUtils.getAsHtml(mapper.getMappedColor(gr.getColor2())), gr.getPolicy());
+		final HColor back = param.getBackcolor();
+		if (back instanceof HColorGradient) {
+			final HColorGradient gr = (HColorGradient) back;
+			final String id = svg.createSvgGradient(mapper.toRGB(gr.getColor1()), mapper.toRGB(gr.getColor2()),
+					gr.getPolicy());
 			svg.setFillColor("url(#" + id + ")");
-			svg.setStrokeColor(color);
+			applyColor(svg, mapper, param);
 		} else {
-			final String backcolor = StringUtils.getAsSvg(mapper, back);
+			final String backcolor = mapper.toSvg(back);
 			svg.setFillColor(backcolor);
-			svg.setStrokeColor(color);
+			applyColor(svg, mapper, param);
 		}
 
 		svg.setStrokeWidth(param.getStroke().getThickness(), param.getStroke().getDasharraySvg());
@@ -91,6 +89,19 @@ public class DriverRectangleSvg implements UDriver<SvgGraphics> {
 				return;
 			}
 		}
-		svg.svgRectangle(x, y, width, height, rx / 2, ry / 2, rect.getDeltaShadow(), rect.getComment());
+		svg.svgRectangle(x, y, width, height, rx / 2, ry / 2, rect.getDeltaShadow(), rect.getComment(),
+				rect.getCodeLine());
+	}
+
+	public static void applyColor(SvgGraphics svg, ColorMapper mapper, UParam param) {
+		final HColor color = param.getColor();
+		if (color instanceof HColorGradient) {
+			final HColorGradient gr = (HColorGradient) color;
+			final String id = svg.createSvgGradient(mapper.toRGB(gr.getColor1()), mapper.toRGB(gr.getColor2()),
+					gr.getPolicy());
+			svg.setStrokeColor("url(#" + id + ")");
+		} else {
+			svg.setStrokeColor(mapper.toSvg(color));
+		}
 	}
 }
