@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -34,91 +34,88 @@
  */
 package net.sourceforge.plantuml.version;
 
+import static net.sourceforge.plantuml.graphic.GraphicPosition.BACKGROUND_CORNER_BOTTOM_RIGHT;
+
 import java.awt.image.BufferedImage;
-import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.io.OutputStream;
-import java.net.HttpURLConnection;
-import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
-import javax.imageio.ImageIO;
-
-import net.sourceforge.plantuml.AbstractPSystem;
-import net.sourceforge.plantuml.FileFormatOption;
-import net.sourceforge.plantuml.FileSystem;
-import net.sourceforge.plantuml.Log;
 import net.sourceforge.plantuml.OptionFlags;
 import net.sourceforge.plantuml.OptionPrint;
+import net.sourceforge.plantuml.PlainStringsDiagram;
 import net.sourceforge.plantuml.Run;
 import net.sourceforge.plantuml.core.DiagramDescription;
-import net.sourceforge.plantuml.core.ImageData;
+import net.sourceforge.plantuml.core.UmlSource;
 import net.sourceforge.plantuml.cucadiagram.dot.GraphvizUtils;
-import net.sourceforge.plantuml.graphic.GraphicPosition;
-import net.sourceforge.plantuml.graphic.GraphicStrings;
-import net.sourceforge.plantuml.preproc.PreprocessorInclude;
+import net.sourceforge.plantuml.dedication.PSystemDedication;
 import net.sourceforge.plantuml.preproc.Stdlib;
-import net.sourceforge.plantuml.svek.TextBlockBackcolored;
-import net.sourceforge.plantuml.ugraphic.ColorMapperIdentity;
-import net.sourceforge.plantuml.ugraphic.ImageBuilder;
+import net.sourceforge.plantuml.preproc2.PreprocessorUtils;
+import net.sourceforge.plantuml.security.ImageIO;
+import net.sourceforge.plantuml.security.SFile;
+import net.sourceforge.plantuml.security.SecurityProfile;
+import net.sourceforge.plantuml.security.SecurityUtils;
+import net.sourceforge.plantuml.svek.GraphvizCrash;
 
-public class PSystemVersion extends AbstractPSystem {
+public class PSystemVersion extends PlainStringsDiagram {
 
-	private final List<String> strings = new ArrayList<String>();
-	private BufferedImage image;
-
-	PSystemVersion(boolean withImage, List<String> args) {
-		strings.addAll(args);
+	PSystemVersion(UmlSource source, boolean withImage, List<String> args) {
+		super(source);
+		this.strings.addAll(args);
 		if (withImage) {
-			image = getPlantumlImage();
+			this.image = getPlantumlImage();
+			this.imagePosition = BACKGROUND_CORNER_BOTTOM_RIGHT;
 		}
 	}
 
+	private PSystemVersion(UmlSource source, List<String> args, BufferedImage image) {
+		super(source);
+		this.strings.addAll(args);
+		this.image = image;
+		this.imagePosition = BACKGROUND_CORNER_BOTTOM_RIGHT;
+	}
+
 	public static BufferedImage getPlantumlImage() {
-		try {
-			final InputStream is = PSystemVersion.class.getResourceAsStream("logo.png");
-			final BufferedImage image = ImageIO.read(is);
-			is.close();
-			return image;
-		} catch (IOException e) {
-			e.printStackTrace();
-		}
-		return new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
+		return getImage("logo.png");
 	}
 
 	public static BufferedImage getCharlieImage() {
-		try {
-			final InputStream is = PSystemVersion.class.getResourceAsStream("charlie.png");
-			final BufferedImage image = ImageIO.read(is);
-			is.close();
-			return image;
-		} catch (IOException e) {
-			e.printStackTrace();
-		}
-		return new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
+		return getImage("charlie.png");
 	}
 
-	public static BufferedImage getTime() {
-		try {
-			final InputStream is = PSystemVersion.class.getResourceAsStream("time00.png");
-			final BufferedImage image = ImageIO.read(is);
-			is.close();
-			return image;
-		} catch (IOException e) {
-			e.printStackTrace();
-		}
-		return new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
+	public static BufferedImage getTime01() {
+		return getImage("time01.png");
+	}
+
+	public static BufferedImage getTime15() {
+		return getImage("time15.png");
 	}
 
 	public static BufferedImage getPlantumlSmallIcon() {
+		return getImage("favicon.png");
+	}
+
+	public static BufferedImage getArecibo() {
+		return getImage("arecibo.png");
+	}
+
+	public static BufferedImage getDotc() {
+		return getImage("dotc.png");
+	}
+
+	public static BufferedImage getDotd() {
+		return getImage("dotd.png");
+	}
+
+	public static BufferedImage getApple2Image() {
+		return getImageWebp("apple2.png");
+	}
+
+	private static BufferedImage getImage(final String name) {
 		try {
-			final InputStream is = PSystemVersion.class.getResourceAsStream("favicon.png");
+			final InputStream is = PSystemVersion.class.getResourceAsStream(name);
 			final BufferedImage image = ImageIO.read(is);
 			is.close();
 			return image;
@@ -128,6 +125,15 @@ public class PSystemVersion extends AbstractPSystem {
 		return new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
 	}
 
+	private static BufferedImage getImageWebp(final String name) {
+		try (InputStream is = PSystemVersion.class.getResourceAsStream(name)) {
+			return PSystemDedication.getBufferedImage(is);
+		} catch (IOException e) {
+			e.printStackTrace();
+		}
+		return new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
+	}
+
 	private static BufferedImage transparentIcon;
 
 	public static BufferedImage getPlantumlSmallIcon2() {
@@ -150,34 +156,28 @@ public class PSystemVersion extends AbstractPSystem {
 		return transparentIcon;
 	}
 
-	@Override
-	final protected ImageData exportDiagramNow(OutputStream os, int num, FileFormatOption fileFormat, long seed)
-			throws IOException {
-		final TextBlockBackcolored result = GraphicStrings.createBlackOnWhite(strings, image,
-				GraphicPosition.BACKGROUND_CORNER_BOTTOM_RIGHT);
-		final ImageBuilder imageBuilder = new ImageBuilder(new ColorMapperIdentity(), 1.0, result.getBackcolor(),
-				getMetadata(), null, 0, 0, null, false);
-		imageBuilder.setUDrawable(result);
-		return imageBuilder.writeImageTOBEMOVED(fileFormat, seed, os);
-	}
-
-	public static PSystemVersion createShowVersion() {
-		final List<String> strings = new ArrayList<String>();
+	public static PSystemVersion createShowVersion2(UmlSource source) {
+		final List<String> strings = new ArrayList<>();
 		strings.add("<b>PlantUML version " + Version.versionString() + "</b> (" + Version.compileTimeString() + ")");
 		strings.add("(" + License.getCurrent() + " source distribution)");
-		strings.add("Loaded from " + Version.getJarPath());
-		if (OptionFlags.getInstance().isWord()) {
-			strings.add("Word Mode");
-			strings.add("Command Line: " + Run.getCommandLine());
-			strings.add("Current Dir: " + FileSystem.getInstance().getCurrentDir().getAbsolutePath());
-			strings.add("plantuml.include.path: " + PreprocessorInclude.getenv("plantuml.include.path"));
+		GraphvizCrash.checkOldVersionWarning(strings);
+		if (OptionFlags.ALLOW_INCLUDE) {
+			if (SecurityUtils.getSecurityProfile() == SecurityProfile.UNSECURE) {
+				strings.add("Loaded from " + Version.getJarPath());
+			}
+			if (OptionFlags.getInstance().isWord()) {
+				strings.add("Word Mode");
+				strings.add("Command Line: " + Run.getCommandLine());
+				strings.add("Current Dir: " + new SFile(".").getAbsolutePath());
+				strings.add("plantuml.include.path: " + PreprocessorUtils.getenv("plantuml.include.path"));
+			}
 		}
 		strings.add(" ");
-		strings.add("<b>Stdlib:");
-		Stdlib.addInfoVersion(strings, false);
-		strings.add(" ");
+		// strings.add("<b>Stdlib:");
+		// Stdlib.addInfoVersion(strings, false);
+		// strings.add(" ");
 
-		strings.addAll(GraphvizUtils.getTestDotStrings(true));
+		GraphvizUtils.addDotStatus(strings, true);
 		strings.add(" ");
 		for (String name : OptionPrint.interestingProperties()) {
 			strings.add(name);
@@ -185,26 +185,26 @@ public class PSystemVersion extends AbstractPSystem {
 		for (String v : OptionPrint.interestingValues()) {
 			strings.add(v);
 		}
-
-		return new PSystemVersion(true, strings);
+		
+		return new PSystemVersion(source, true, strings);
 	}
 
-	public static PSystemVersion createStdLib() {
-		final List<String> strings = new ArrayList<String>();
+	public static PSystemVersion createStdLib(UmlSource source) {
+		final List<String> strings = new ArrayList<>();
 		Stdlib.addInfoVersion(strings, true);
 		strings.add(" ");
 
-		return new PSystemVersion(true, strings);
+		return new PSystemVersion(source, true, strings);
 	}
 
-	public static PSystemVersion createShowAuthors() {
+	public static PSystemVersion createShowAuthors2(UmlSource source) {
 		// Duplicate in OptionPrint
 		final List<String> strings = getAuthorsStrings(true);
-		return new PSystemVersion(true, strings);
+		return new PSystemVersion(source, true, strings);
 	}
 
 	public static List<String> getAuthorsStrings(boolean withTag) {
-		final List<String> strings = new ArrayList<String>();
+		final List<String> strings = new ArrayList<>();
 		add(strings, "<b>PlantUML version " + Version.versionString() + "</b> (" + Version.compileTimeString() + ")",
 				withTag);
 		add(strings, "(" + License.getCurrent() + " source distribution)", withTag);
@@ -223,7 +223,7 @@ public class PSystemVersion extends AbstractPSystem {
 		add(strings, "<u>Logo</u>: Benjamin Croizet", withTag);
 
 		add(strings, " ", withTag);
-		add(strings, "http://plantuml.com", withTag);
+		add(strings, "https://plantuml.com", withTag);
 		add(strings, " ", withTag);
 		return strings;
 	}
@@ -236,85 +236,49 @@ public class PSystemVersion extends AbstractPSystem {
 
 	}
 
-	public static PSystemVersion createCheckVersions(String host, String port) {
-		final List<String> strings = new ArrayList<String>();
-		strings.add("<b>PlantUML version " + Version.versionString() + "</b> (" + Version.compileTimeString() + ")");
+	public static PSystemVersion createTestDot(UmlSource source) throws IOException {
+		final List<String> strings = new ArrayList<>();
+		strings.add(Version.fullDescription());
+		GraphvizUtils.addDotStatus(strings, true);
+		return new PSystemVersion(source, false, strings);
+	}
 
-		final int lastversion = extractDownloadableVersion(host, port);
+//	public static PSystemVersion createDumpStackTrace() throws IOException {
+//		final List<String> strings = new ArrayList<>();
+//		final Throwable creationPoint = new Throwable();
+//		creationPoint.fillInStackTrace();
+//		for (StackTraceElement ste : creationPoint.getStackTrace()) {
+//			strings.add(ste.toString());
+//		}
+//		return new PSystemVersion(false, strings);
+//	}
 
-		int lim = 7;
-		if (lastversion == -1) {
-			strings.add("<b><color:red>Error");
-			strings.add("<color:red>Cannot connect to http://plantuml.com/");
-			strings.add("Maybe you should set your proxy ?");
-			strings.add("@startuml");
-			strings.add("checkversion(proxy=myproxy.com,port=8080)");
-			strings.add("@enduml");
-			lim = 9;
-		} else if (lastversion == 0) {
-			strings.add("<b><color:red>Error</b>");
-			strings.add("Cannot retrieve last version from http://plantuml.com/");
+	public static PSystemVersion createKeyDistributor(UmlSource source) throws IOException {
+		final LicenseInfo license = LicenseInfo.retrieveDistributor();
+		BufferedImage im = null;
+		final List<String> strings = new ArrayList<>();
+		if (license == null) {
+			strings.add("No license found");
 		} else {
-			strings.add("<b>Last available version for download</b> : " + lastversion);
-			strings.add(" ");
-			if (Version.version() >= lastversion) {
-				strings.add("<b><color:green>Your version is up to date.");
-			} else {
-				strings.add("<b><color:red>A newer version is available for download.");
-			}
+			strings.add(license.getOwner());
+			strings.add(license.getContext());
+			strings.add(license.getGenerationDate().toString());
+			strings.add(license.getExpirationDate().toString());
+			im = LicenseInfo.retrieveDistributorImage(license);
 		}
-
-		while (strings.size() < lim) {
-			strings.add(" ");
-		}
-
-		return new PSystemVersion(true, strings);
+		return new PSystemVersion(source, strings, im);
 	}
 
-	public static int extractDownloadableVersion(String host, String port) {
-		if (host != null && port != null) {
-			System.setProperty("http.proxyHost", host);
-			System.setProperty("http.proxyPort", port);
-		}
-
-		try {
-			final URL url = new URL("http://plantuml.com/download");
-			final HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
-			urlConnection.setUseCaches(false);
-			urlConnection.connect();
-			if (urlConnection.getResponseCode() == HttpURLConnection.HTTP_OK) {
-				final BufferedReader in = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));
-				final int lastversion = extractVersion(in);
-				in.close();
-				urlConnection.disconnect();
-				return lastversion;
-			}
-		} catch (IOException e) {
-			Log.error(e.toString());
-		}
-		return -1;
-	}
-
-	private static int extractVersion(BufferedReader in) throws IOException {
-		String s;
-		final Pattern p = Pattern.compile(".*\\.([1-9]\\d?)\\.(20\\d\\d)\\.([1-9]?\\d)\\..*");
-		while ((s = in.readLine()) != null) {
-			final Matcher m = p.matcher(s);
-			if (m.matches()) {
-				final String a = m.group(1);
-				final String b = m.group(2);
-				final String c = m.group(3);
-				return Integer.parseInt(a) * 1000000 + Integer.parseInt(b) * 100 + Integer.parseInt(c);
-			}
-		}
-		return 0;
-	}
-
-	public static PSystemVersion createTestDot() throws IOException {
-		final List<String> strings = new ArrayList<String>();
-		strings.addAll(GraphvizUtils.getTestDotStrings(true));
-		return new PSystemVersion(false, strings);
-	}
+//	public static PSystemVersion createPath(UmlSource source) throws IOException {
+//		final List<String> strings = new ArrayList<>();
+//		strings.add("<u>Current Dir</u>: " + new SFile(".").getPrintablePath());
+//		strings.add(" ");
+//		strings.add("<u>Default path</u>:");
+//		for (SFile f : ImportedFiles.createImportedFiles(null).getPath()) {
+//			strings.add(f.getPrintablePath());
+//		}
+//		return new PSystemVersion(source, true, strings);
+//	}
 
 	public DiagramDescription getDescription() {
 		return new DiagramDescription("(Version)");
